@@ -1,9 +1,16 @@
+from __future__ import division, absolute_import, print_function
+import sys
+if sys.version_info < (3,):
+    range = xrange
+else:
+    unicode = str
+
 from matplotlib import rc
 from matplotlib import rcParams
 
 font_size=14
-rcParams["backend"] = "Qt4Agg"
-rcParams["figure.figsize"] = (6,4.5)
+rcParams["backend"] = "PDF"
+rcParams["figure.figsize"] = (4, 3)
 rcParams["font.family"] = "Serif"
 rcParams["font.serif"] = ["Palatino"]
 rcParams["font.size"] = font_size
@@ -20,6 +27,8 @@ rcParams["figure.subplot.top"] = 0.95
 rcParams["figure.subplot.left"] = 0.2
 rcParams["figure.subplot.bottom"] = 0.2
 
+rcParams["image.cmap"] = "hot"
+
 rcParams["text.usetex"] = True
 
 rcParams["ps.usedistiller"] = "xpdf"
