@@ -7,7 +7,7 @@
 //  Authors: Douglas Gregor
 //           Andrew Lumsdaine
 
-// Hacked by Tiago Peixoto, to enable openmp support for graph-tool
+//  Modified by Tiago Peixoto, to enable openmp support for graph-tool
 
 #ifndef BOOST_GRAPH_BRANDES_BETWEENNESS_CENTRALITY_HPP
 #define BOOST_GRAPH_BRANDES_BETWEENNESS_CENTRALITY_HPP
@@ -66,7 +66,7 @@ namespace detail { namespace graph {
      * for w is set to {(v, w)} and the shortest path count of w is set to
      * the number of paths that reach {v}.
      */
-    void edge_relaxed(edge_descriptor e, const Graph& g)
+    void edge_relaxed(const edge_descriptor& e, const Graph& g)
     {
       vertex_descriptor v = source(e, g), w = target(e, g);
       incoming[w].clear();
@@ -80,7 +80,7 @@ namespace detail { namespace graph {
      * incoming edges of w and add all of the shortest paths to v to the
      * shortest path count of w.
      */
-    void edge_not_relaxed(edge_descriptor e, const Graph& g)
+    void edge_not_relaxed(const edge_descriptor& e, const Graph& g)
     {
       typedef typename property_traits<WeightMap>::value_type weight_type;
       typedef typename property_traits<DistanceMap>::value_type distance_type;
@@ -172,7 +172,7 @@ namespace detail { namespace graph {
           path_count(ipath_count), ordered_vertices(iordered_vertices) { }
 
       /// Keep track of vertices as they are reached
-      void examine_vertex(vertex_descriptor v, Graph&)
+      void examine_vertex(vertex_descriptor v, const Graph&)
       {
         ordered_vertices.push(v);
       }
@@ -182,7 +182,7 @@ namespace detail { namespace graph {
        * incoming edge list for w is set to {(v, w)} and the shortest
        * path count of w is set to the number of paths that reach {v}.
        */
-      void tree_edge(edge_descriptor e, Graph& g)
+      void tree_edge(const edge_descriptor& e, const Graph& g)
       {
         vertex_descriptor v = source(e, g);
         vertex_descriptor w = target(e, g);
@@ -198,7 +198,7 @@ namespace detail { namespace graph {
        * in the incoming edge list of w and add all of the shortest
        * paths to v to the shortest path count of w.
        */
-      void non_tree_edge(edge_descriptor e, Graph& g)
+      void non_tree_edge(const edge_descriptor& e, const Graph& g)
       {
         vertex_descriptor v = source(e, g);
         vertex_descriptor w = target(e, g);
@@ -270,8 +270,12 @@ namespace detail { namespace graph {
   // When we have a real edge centrality map, add the value to the map
   template<typename CentralityMap, typename Key, typename T>
   inline void
-  update_centrality(CentralityMap centrality_map, Key k, const T& x)
-  { put(centrality_map, k, get(centrality_map, k) + x); }
+  update_centrality(CentralityMap& centrality_map, const Key& k, const T& x)
+  {
+      auto& val = get(centrality_map, k);
+      #pragma omp atomic
+      val += x;
+  }
 
   template<typename Iter>
   inline void
@@ -305,27 +309,33 @@ namespace detail { namespace graph {
                                       ShortestPaths shortest_paths)
   {
     typedef typename graph_traits<Graph>::vertex_iterator vertex_iterator;
-    typedef typename graph_traits<Graph>::edge_iterator edge_iterator;
     typedef typename graph_traits<Graph>::vertex_descriptor vertex_descriptor;
 
     // Initialize centrality
     init_centrality_map(vertices(g), centrality);
     init_centrality_map(edges(g), edge_centrality_map);
 
+    vector<typename property_traits<IncomingMap>::value_type> vincoming(num_vertices(g));
+    vector<typename property_traits<DistanceMap>::value_type> vdistance(num_vertices(g));
+    vector<typename property_traits<DependencyMap>::value_type> vdependency(num_vertices(g));
+    vector<typename property_traits<PathCountMap>::value_type> vpath_count(num_vertices(g));
+
     int i, N = num_vertices(g);
-    #pragma omp parallel for default(shared) private(i) schedule(dynamic)
+    #pragma omp parallel for default(shared) private(i) \
+        firstprivate(vincoming, vdistance, vdependency, vpath_count) \
+        schedule(runtime)
     for (i = 0; i < N; ++i)
     {
-      typename graph_traits<Graph>::vertex_descriptor s = vertex(i, g);
+      auto s = vertex(i, g);
       if (s == graph_traits<Graph>::null_vertex())
           continue;
 
       std::stack<vertex_descriptor> ordered_vertices;
 
-      vector_property_map<typename property_traits<IncomingMap>::value_type,VertexIndexMap> incoming(vertex_index);
-      vector_property_map<typename property_traits<DistanceMap>::value_type,VertexIndexMap> distance(vertex_index);
-      vector_property_map<typename property_traits<DependencyMap>::value_type,VertexIndexMap> dependency(vertex_index);
-      vector_property_map<typename property_traits<PathCountMap>::value_type,VertexIndexMap> path_count(vertex_index);
+      auto incoming = make_iterator_property_map(vincoming.begin(), vertex_index);
+      auto distance = make_iterator_property_map(vdistance.begin(), vertex_index);
+      auto dependency = make_iterator_property_map(vdependency.begin(), vertex_index);
+      auto path_count = make_iterator_property_map(vpath_count.begin(), vertex_index);
 
       // Initialize for this iteration
       vertex_iterator w, w_end;
@@ -347,20 +357,16 @@ namespace detail { namespace graph {
           vertex_descriptor u = ordered_vertices.top();
           ordered_vertices.pop();
 
-          typedef typename property_traits<IncomingMap>::value_type
-              incoming_type;
-          typedef typename incoming_type::iterator incoming_iterator;
           typedef typename property_traits<DependencyMap>::value_type
               dependency_type;
 
-          for (incoming_iterator vw = incoming[u].begin();
-               vw != incoming[u].end(); ++vw) {
-              vertex_descriptor v = source(*vw, g);
-              dependency_type factor = dependency_type(get(path_count, v))
+          for (const auto& vw : incoming[u]) {
+              auto v = source(vw, g);
+              auto factor = dependency_type(get(path_count, v))
                   / dependency_type(get(path_count, u));
               factor *= (dependency_type(1) + get(dependency, u));
               put(dependency, v, get(dependency, v) + factor);
-              update_centrality(edge_centrality_map, *vw, factor);
+              update_centrality(edge_centrality_map, vw, factor);
           }
 
           if (u != s) {
@@ -443,7 +449,6 @@ namespace detail { namespace graph {
                                            VertexIndexMap vertex_index)
   {
     typedef typename graph_traits<Graph>::degree_size_type degree_size_type;
-    typedef typename graph_traits<Graph>::vertex_descriptor vertex_descriptor;
     typedef typename graph_traits<Graph>::edge_descriptor edge_descriptor;
     typedef typename mpl::if_c<(is_same<CentralityMap,
                                         dummy_property_map>::value),
@@ -479,7 +484,6 @@ namespace detail { namespace graph {
                                            VertexIndexMap vertex_index)
   {
     typedef typename graph_traits<Graph>::degree_size_type degree_size_type;
-    typedef typename graph_traits<Graph>::vertex_descriptor vertex_descriptor;
     typedef typename graph_traits<Graph>::edge_descriptor edge_descriptor;
     typedef typename mpl::if_c<(is_same<CentralityMap,
                                         dummy_property_map>::value),
