@@ -24,12 +24,28 @@
 #include <boost/mpl/for_each.hpp>
 #include <boost/python/object.hpp>
 #include <boost/bind.hpp>
+#include <string>
 #include <exception>
 #include <set>
+#include <unordered_set>
+
+// Base64 Encoding
+
+#include "base64.hh"
 
 namespace boost
 {
 
+typedef mpl::vector<uint8_t, int16_t, int32_t, int64_t, double, long double,
+                    std::vector<uint8_t>, std::vector<int16_t>,
+                    std::vector<int32_t>, std::vector<int64_t>,
+                    std::vector<double>, std::vector<long double>,
+                    std::vector<std::string>, std::string,
+                    python::object> prop_value_types;
+
+extern const char* prop_type_names[];
+
+
 /////////////////////////////////////////////////////////////////////////////
 // Graph reader exceptions
 /////////////////////////////////////////////////////////////////////////////
@@ -86,9 +102,9 @@ class mutate_graph_impl : public mutate_graph
 public:
     mutate_graph_impl(MutableGraph& g, dynamic_properties& dp,
                       bool ignore_directedness,
-                      std::set<std::string> ignore_vp,
-                      std::set<std::string> ignore_ep,
-                      std::set<std::string> ignore_gp)
+                      std::unordered_set<std::string> ignore_vp,
+                      std::unordered_set<std::string> ignore_ep,
+                      std::unordered_set<std::string> ignore_gp)
         : m_g(g), m_dp(dp), m_ignore_directedness(ignore_directedness),
           m_is_directed(false), m_ignore_vp(ignore_vp),
           m_ignore_ep(ignore_ep), m_ignore_gp(ignore_gp) { }
@@ -141,10 +157,10 @@ public:
         bool type_found = false;
         try
         {
-            mpl::for_each<value_types>
-                (put_property<graph_property_tag,value_types>
+            mpl::for_each<prop_value_types>
+                (put_property<graph_property_tag,prop_value_types>
                  (name, m_dp, graph_property_tag(), value, value_type,
-                  m_type_names, type_found));
+                  type_found));
         }
         catch (bad_lexical_cast)
         {
@@ -167,10 +183,10 @@ public:
         bool type_found = false;
         try
         {
-            mpl::for_each<value_types>
-                (put_property<vertex_descriptor,value_types>
+            mpl::for_each<prop_value_types>
+                (put_property<vertex_descriptor,prop_value_types>
                  (name, m_dp, any_cast<vertex_descriptor>(vertex),
-                  value, value_type, m_type_names, type_found));
+                  value, value_type, type_found));
         }
         catch (bad_lexical_cast)
         {
@@ -193,10 +209,10 @@ public:
         bool type_found = false;
         try
         {
-            mpl::for_each<value_types>
-                (put_property<edge_descriptor,value_types>
+            mpl::for_each<prop_value_types>
+                (put_property<edge_descriptor,prop_value_types>
                  (name, m_dp, any_cast<edge_descriptor>(edge),
-                  value, value_type, m_type_names, type_found));
+                  value, value_type, type_found));
         }
         catch (bad_lexical_cast)
         {
@@ -213,26 +229,37 @@ public:
     {
     public:
         put_property(const std::string& name, dynamic_properties& dp,
-                     const Key& key,
-                     const std::string& value, const std::string& value_type,
-                     const char** type_names, bool& type_found)
-            : m_name(name), m_dp(dp), m_key(key), m_value(value),
-              m_value_type(value_type), m_type_names(type_names),
-              m_type_found(type_found) {}
+                     const Key& key, const std::string& value,
+                     const std::string& value_type,
+                     bool& type_found) : m_name(name), m_dp(dp), m_key(key),
+            m_value(value), m_value_type(value_type),
+            m_type_found(type_found) {}
         template <class Value>
         void operator()(Value)
         {
             if (m_value_type ==
-                m_type_names[mpl::find<ValueVector,Value>::type::pos::value])
+                prop_type_names[mpl::find<ValueVector,Value>::type::pos::value])
             {
+                std::string val = m_value;
+                if (m_value_type == "boolean")
+                {
+                    if (val == "true" || val == "True")
+                        val = "1";
+                    if (val == "false" || val == "False")
+                        val = "0";
+                }
+
                 if (is_same<Value,uint8_t>::value) // chars are stored as ints
                 {
-                    int v = lexical_cast<int>(m_value);
+                    int v = lexical_cast<int>(val);
                     put(m_name, m_dp, m_key, uint8_t(v));
                 }
                 else
                 {
-                    put(m_name, m_dp, m_key, lexical_cast<Value>(m_value));
+                    if (is_same<Value, boost::python::object>::value)
+                        val = base64_decode(m_value);
+
+                    put(m_name, m_dp, m_key, lexical_cast<Value>(val));
                 }
                 m_type_found = true;
             }
@@ -243,7 +270,6 @@ public:
         const Key& m_key;
         const std::string& m_value;
         const std::string& m_value_type;
-        const  char** m_type_names;
         bool& m_type_found;
     };
 
@@ -252,24 +278,11 @@ protected:
     dynamic_properties& m_dp;
     bool m_ignore_directedness;
     bool m_is_directed;
-    std::set<std::string> m_ignore_vp;
-    std::set<std::string> m_ignore_ep;
-    std::set<std::string> m_ignore_gp;
-    typedef mpl::vector<uint8_t, int16_t, int32_t, int64_t, double, long double,
-                        std::vector<uint8_t>, std::vector<int32_t>,
-                        std::vector<int64_t>, std::vector<double>,
-                        std::vector<long double>, std::vector<std::string>,
-                        std::string, python::object>
-        value_types;
-    static const char* m_type_names[];
+    std::unordered_set<std::string> m_ignore_vp;
+    std::unordered_set<std::string> m_ignore_ep;
+    std::unordered_set<std::string> m_ignore_gp;
 };
 
-template<typename MutableGraph>
-const char* mutate_graph_impl<MutableGraph>::m_type_names[] =
-{"boolean", "short", "int", "long", "float", "double", "vector_boolean", "vector_int",
- "vector_long", "vector_float", "vector_double", "vector_string", "string",
- "python_object"};
-
 void
 read_graphml(std::istream& in, mutate_graph& g, bool integer_vertices,
              bool store_ids);
@@ -278,9 +291,9 @@ template<typename MutableGraph>
 bool
 read_graphml(std::istream& in, MutableGraph& g, dynamic_properties& dp,
              bool store_ids, bool integer_vertices, bool ignore_directedness,
-             std::set<std::string> ignore_vp = std::set<std::string>(),
-             std::set<std::string> ignore_ep = std::set<std::string>(),
-             std::set<std::string> ignore_gp = std::set<std::string>())
+             std::unordered_set<std::string> ignore_vp = std::unordered_set<std::string>(),
+             std::unordered_set<std::string> ignore_ep = std::unordered_set<std::string>(),
+             std::unordered_set<std::string> ignore_gp = std::unordered_set<std::string>())
 {
     mutate_graph_impl<MutableGraph> mg(g, dp, ignore_directedness, ignore_vp,
                                        ignore_ep, ignore_gp);
@@ -292,18 +305,16 @@ template <typename Types>
 class get_type_name
 {
 public:
-    get_type_name(const std::type_info& type, const char** type_names,
-                  std::string& type_name)
-        : m_type(type), m_type_names(type_names), m_type_name(type_name) {}
+    get_type_name(const std::type_info& type, std::string& type_name)
+        : m_type(type), m_type_name(type_name) {}
     template <typename Type>
     void operator()(Type)
     {
         if (typeid(Type) == m_type)
-            m_type_name = m_type_names[mpl::find<Types,Type>::type::pos::value];
+            m_type_name = prop_type_names[mpl::find<Types,Type>::type::pos::value];
     }
 private:
     const std::type_info &m_type;
-    const char** m_type_names;
     std::string &m_type_name;
 };
 
@@ -319,6 +330,8 @@ struct get_string
         if (v != 0)
         {
             sval = lexical_cast<std::string>(*v);
+            if (is_same<ValueType, boost::python::object>::value)
+                sval = base64_encode(sval);
         }
     }
 };
@@ -328,8 +341,8 @@ std::string print_value(dynamic_property_map& pmap, Descriptor v)
 {
     std::string val;
     boost::any oval = pmap.get(v);
-    mpl::for_each<ValueTypes>(bind<void>(get_string(), ref(oval),
-                                         ref(val), _1));
+    mpl::for_each<ValueTypes>(boost::bind<void>(get_string(), boost::ref(oval),
+                                                boost::ref(val), _1));
     return val;
 }
 
@@ -354,19 +367,6 @@ write_graphml(std::ostream& out, const Graph& g, VertexIndexMap vertex_index,
            "         xsi:schemaLocation=\"http://graphml.graphdrawing.org/xmlns"
            " http://graphml.graphdrawing.org/xmlns/1.0/graphml.xsd\">\n\n";
 
-    typedef mpl::vector<bool, uint8_t, int8_t, uint16_t, int16_t, uint32_t,
-                        int32_t, uint64_t, int64_t, float, double, long double,
-                        std::vector<uint8_t>, std::vector<int32_t>,
-                        std::vector<int64_t>, std::vector<double>,
-                        std::vector<long double>, std::vector<std::string>,
-                        std::string, python::object> value_types;
-    const char* type_names[] = {"boolean", "boolean", "boolean", "short",
-                                "short", "int", "int", "long", "long", "float",
-                                "float", "double", "vector_boolean",
-                                "vector_int", "vector_long", "vector_float",
-                                "vector_double", "vector_string", "string",
-                                "python_object"};
-
     std::map<std::string, std::string> graph_key_ids;
     std::map<std::string, std::string> vertex_key_ids;
     std::map<std::string, std::string> edge_key_ids;
@@ -402,9 +402,8 @@ write_graphml(std::ostream& out, const Graph& g, VertexIndexMap vertex_index,
         else
             continue;
         std::string type_name = "string";
-        mpl::for_each<value_types>
-            (get_type_name<value_types>(i->second->value(), type_names,
-                                        type_name));
+        mpl::for_each<prop_value_types>
+            (get_type_name<prop_value_types>(i->second->value(), type_name));
         out << "  <key id=\"" << protect_xml_string(key_id) << "\" for=\""
             << (i->second->key() == typeid(graph_property_tag) ? "graph" :
                 (i->second->key() == typeid(vertex_descriptor) ? "node" :
@@ -431,7 +430,7 @@ write_graphml(std::ostream& out, const Graph& g, VertexIndexMap vertex_index,
         if (i->second->key() == typeid(graph_property_tag))
         {
             std::string val = protect_xml_string
-                (print_value<value_types>(*i->second, graph_property_tag()));
+                (print_value<prop_value_types>(*i->second, graph_property_tag()));
             if (val.empty())
                 continue;
             out << "   <data key=\""
@@ -463,7 +462,7 @@ write_graphml(std::ostream& out, const Graph& g, VertexIndexMap vertex_index,
             if (i->second->key() == typeid(vertex_descriptor))
             {
                 std::string val = protect_xml_string
-                    (print_value<value_types>(*i->second, *v));
+                    (print_value<prop_value_types>(*i->second, *v));
                 if (val.empty())
                     continue;
                 out << "      <data key=\""
@@ -511,7 +510,7 @@ write_graphml(std::ostream& out, const Graph& g, VertexIndexMap vertex_index,
             if (i->second->key() == typeid(edge_descriptor))
             {
                 std::string val = protect_xml_string
-                    (print_value<value_types>(*i->second, *e));
+                    (print_value<prop_value_types>(*i->second, *e));
                 if (val.empty())
                     continue;
                 out << "      <data key=\""
