@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -38,27 +38,28 @@ size_t pagerank(GraphInterface& g, boost::any rank, boost::any pers,
         throw ValueException("personalization vertex property must have a scalar value type");
 
     typedef ConstantPropertyMap<double, GraphInterface::vertex_t> pers_map_t;
-    typedef mpl::push_back<vertex_scalar_properties, pers_map_t>::type
+    typedef boost::mpl::push_back<vertex_scalar_properties, pers_map_t>::type
         pers_props_t;
 
     if(pers.empty())
-        pers = pers_map_t(1.0 / g.GetNumberOfVertices());
+        pers = pers_map_t(1.0 / g.get_num_vertices());
 
-    typedef ConstantPropertyMap<double, GraphInterface::edge_t> weight_map_t;
-    typedef mpl::push_back<edge_scalar_properties, weight_map_t>::type
+    typedef UnityPropertyMap<int,GraphInterface::edge_t> weight_map_t;
+    typedef boost::mpl::push_back<edge_scalar_properties, weight_map_t>::type
         weight_props_t;
 
     if (!weight.empty() && !belongs<edge_scalar_properties>()(weight))
         throw ValueException("weight edge property must have a scalar value type");
 
     if(weight.empty())
-        weight = weight_map_t(1.0);
+        weight = weight_map_t();
 
     size_t iter;
     run_action<>()
-        (g, bind<void>(get_pagerank(),
-                       _1, g.GetVertexIndex(), _2, _3, _4, d,
-                       epsilon, max_iter, ref(iter)),
+        (g, std::bind(get_pagerank(),
+                      std::placeholders::_1, g.get_vertex_index(), std::placeholders::_2,
+                      std::placeholders::_3, std::placeholders::_4, d,
+                      epsilon, max_iter, std::ref(iter)),
          vertex_floating_properties(),
          pers_props_t(), weight_props_t())(rank, pers, weight);
     return iter;
