@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -29,9 +29,11 @@ pair<double,double>
 assortativity_coefficient(GraphInterface& gi,
                           GraphInterface::deg_t deg)
 {
-    double a, a_err;
-    run_action<>()(gi,bind<void>(get_assortativity_coefficient(), _1, _2,
-                                 ref(a), ref(a_err)), scalar_selectors())
+    double a = 0, a_err = 0;
+    run_action<>()(gi,std::bind(get_assortativity_coefficient(),
+                                std::placeholders::_1, std::placeholders::_2,
+                                std::ref(a), std::ref(a_err)),
+                   scalar_selectors())
         (degree_selector(deg));
     return make_pair(a, a_err);
 }
@@ -40,9 +42,10 @@ pair<double,double>
 scalar_assortativity_coefficient(GraphInterface& gi,
                                  GraphInterface::deg_t deg)
 {
-    double a, a_err;
-    run_action<>()(gi, bind<void>(get_scalar_assortativity_coefficient(),
-                                  _1, _2, ref(a), ref(a_err)),
+    double a = 0, a_err = 0;
+    run_action<>()(gi, std::bind(get_scalar_assortativity_coefficient(),
+                                 std::placeholders::_1, std::placeholders::_2,
+                                 std::ref(a), std::ref(a_err)),
                    scalar_selectors())
         (degree_selector(deg));
     return make_pair(a, a_err);
