@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -33,24 +33,24 @@ using namespace graph_tool;
 
 // implementations spread across different compile units to minimize memory
 // usage during compilation
-void graph_avg_corr_imp1(GraphInterface& g, python::object& avg,
-                         python::object& dev, python::object& ret_bins,
+void graph_avg_corr_imp1(GraphInterface& g, boost::python::object& avg,
+                         boost::python::object& dev, boost::python::object& ret_bins,
                          boost::any deg1, boost::any deg2,
                          boost::any weight,
                          const vector<long double>& bins);
 
 
-typedef ConstantPropertyMap<int,GraphInterface::edge_t> cweight_map_t;
+typedef UnityPropertyMap<int,GraphInterface::edge_t> cweight_map_t;
 
-python::object
+boost::python::object
 get_vertex_avg_correlation(GraphInterface& gi,
                            GraphInterface::deg_t deg1,
                            GraphInterface::deg_t deg2,
                            boost::any weight,
                            const vector<long double>& bins)
 {
-    python::object avg, dev;
-    python::object ret_bins;
+    boost::python::object avg, dev;
+    boost::python::object ret_bins;
 
     any weight_prop;
     typedef DynamicPropertyMapWrap<long double, GraphInterface::edge_t>
@@ -61,14 +61,14 @@ get_vertex_avg_correlation(GraphInterface& gi,
         weight_prop = wrapped_weight_t(weight, edge_scalar_properties());
     }
     else
-        weight_prop = cweight_map_t(1);
+        weight_prop = cweight_map_t();
 
     try
     {
         run_action<>()(gi, get_avg_correlation<GetNeighboursPairs>
                        (avg, dev, bins, ret_bins),
                        scalar_selectors(), scalar_selectors(),
-                       mpl::vector<cweight_map_t>())
+                       boost::mpl::vector<cweight_map_t>())
             (degree_selector(deg1), degree_selector(deg2), weight_prop);
     }
     catch (ActionNotFound&)
@@ -78,7 +78,7 @@ get_vertex_avg_correlation(GraphInterface& gi,
                             degree_selector(deg2), weight_prop, bins);
     }
 
-    return python::make_tuple(avg, dev, ret_bins);
+    return boost::python::make_tuple(avg, dev, ret_bins);
 }
 
 using namespace boost::python;
