@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -30,30 +30,30 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-typedef ConstantPropertyMap<int,GraphInterface::edge_t> dummy_weight;
+typedef UnityPropertyMap<int,GraphInterface::edge_t> dummy_weight;
 
-python::object
+boost::python::object
 get_vertex_combined_correlation_histogram(GraphInterface& gi,
                                           GraphInterface::deg_t deg1,
                                           GraphInterface::deg_t deg2,
                                           const vector<long double>& xbin,
                                           const vector<long double>& ybin)
 {
-    python::object hist;
-    python::object ret_bins;
+    boost::python::object hist;
+    boost::python::object ret_bins;
 
-    array<vector<long double>,2> bins;
+    std::array<vector<long double>,2> bins;
     bins[0] = xbin;
     bins[1] = ybin;
 
     run_action<>()(gi, get_correlation_histogram<GetCombinedPair>(hist, bins,
                                                                   ret_bins),
                    scalar_selectors(), scalar_selectors(),
-                   mpl::vector<dummy_weight>())
-        (degree_selector(deg1), degree_selector(deg2),
+                   boost::mpl::vector<dummy_weight>())
+        (any(degree_selector(deg1)), any(degree_selector(deg2)),
          boost::any(dummy_weight()));
 
-    return python::make_tuple(hist, ret_bins);
+    return boost::python::make_tuple(hist, ret_bins);
 }
 
 using namespace boost::python;
