@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -29,11 +29,11 @@ using namespace graph_tool;
 void complete(GraphInterface& gi, size_t N, bool directed,
               bool self_loops)
 {
-    get_complete()(gi.GetGraph(), N, directed, self_loops);
+    get_complete()(gi.get_graph(), N, directed, self_loops);
 }
 
 void circular(GraphInterface& gi, size_t N, size_t k, bool directed,
               bool self_loops)
 {
-    get_circular()(gi.GetGraph(), N, k, directed, self_loops);
+    get_circular()(gi.get_graph(), N, k, directed, self_loops);
 }
