@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -26,10 +26,10 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-void lattice(GraphInterface& gi, python::object oshape, bool periodic)
+void lattice(GraphInterface& gi, boost::python::object oshape, bool periodic)
 {
-    vector<size_t> shape(python::len(oshape));
+    vector<size_t> shape(boost::python::len(oshape));
     for(size_t i = 0; i < shape.size(); ++i)
         shape[i] = python::extract<size_t>(oshape[i]);
-    get_lattice()(gi.GetGraph(), shape, periodic);
+    get_lattice()(gi.get_graph(), shape, periodic);
 }
