@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -26,8 +26,8 @@ using namespace boost;
 void predecessor_graph(GraphInterface& gi, GraphInterface& gpi,
                        boost::any pred_map)
 {
-     run_action<>()(gi, bind<void>(get_predecessor_graph(), _1,
-                                   gi.GetVertexIndex(), ref(gpi.GetGraph()),
-                                   _2),
-                    vertex_scalar_properties())(pred_map);
+    run_action<>()(gi, std::bind(get_predecessor_graph(), std::placeholders::_1,
+                                 std::ref(gpi.get_graph()),
+                                 std::placeholders::_2),
+                   vertex_scalar_properties())(pred_map);
 }
