@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -28,5 +28,6 @@ using namespace graph_tool;
 void price(GraphInterface& gi, size_t N, double gamma, double c, size_t m,
            rng_t& rng)
 {
-    run_action<>()(gi, bind<void>(get_price(), _1, N, gamma, c, m, ref(rng)))();
+    run_action<>()(gi, std::bind(get_price(), std::placeholders::_1, N, gamma, c, m,
+                                 std::ref(rng)))();
 }
