@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -21,6 +21,8 @@
 //  software in the executable aside from CGAL.
 
 #include "graph.hh"
+#include "graph_util.hh"
+#include "graph_filtering.hh"
 
 #if (GCC_VERSION < 40400 || defined __clang__)
 #   define CGAL_CFG_NO_TR1_ARRAY
@@ -66,18 +68,16 @@ bool operator==(const PeriodicDelaunayTriangulation::Vertex& a,
 #endif
 
 #include "graph_triangulation.hh"
-#include "graph_util.hh"
-#include "graph_filtering.hh"
 #include "numpy_bind.hh"
 
 using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-void triangulation(GraphInterface& gi, python::object points, boost::any pos,
-                   string type, bool periodic)
+void triangulation(GraphInterface& gi, boost::python::object points,
+                   boost::any pos, string type, bool periodic)
 {
-    UndirectedAdaptor<GraphInterface::multigraph_t> g(gi.GetGraph());
+    UndirectedAdaptor<GraphInterface::multigraph_t> g(gi.get_graph());
     multi_array_ref<double,2> points_array = get_array<double,2>(points);
     typedef property_map_type::apply
         <vector<double>, GraphInterface::vertex_index_map_t>::type pos_type_t;
@@ -85,20 +85,20 @@ void triangulation(GraphInterface& gi, python::object points, boost::any pos,
 
     if (type == "simple")
     {
-        get_triangulation<SimpleTriangulation>()(g, points_array, pos_map);
+        get_triangulation<SimpleTriangulation, std::false_type>()(g, points_array, pos_map);
     }
     else if (type == "delaunay")
     {
         if (!periodic)
         {
-            get_triangulation<DelaunayTriangulation>()(g, points_array,
-                                                       pos_map);
+            get_triangulation<DelaunayTriangulation, std::false_type>()
+                (g, points_array, pos_map);
         }
         else
         {
 #if (CGAL_VERSION_NR >= 1030500000)
-            get_triangulation<PeriodicDelaunayTriangulation>()(g, points_array,
-                                                               pos_map);
+            get_triangulation<PeriodicDelaunayTriangulation, std::true_type>()
+                (g, points_array, pos_map);
 #else
             throw ValueException("Periodic Delaunay triangulation is only "
                                  "available with versions of CGAL newer than "
