@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -27,23 +27,9 @@
 using namespace graph_tool;
 using namespace boost;
 
-typedef property_map_type::apply<int32_t,
-                                 GraphInterface::vertex_index_map_t>::type
-    vprop_t;
+typedef vprop_map_t<int64_t>::type vprop_t;
 
-typedef property_map_type::apply<GraphInterface::edge_t,
-                                 GraphInterface::edge_index_map_t>::type
-    eprop_t;
-
-struct get_pointers
-{
-    template <class List>
-    struct apply
-    {
-        typedef typename mpl::transform<List,
-                                        mpl::quote1<add_pointer> >::type type;
-    };
-};
+typedef eprop_map_t<GraphInterface::edge_t>::type eprop_t;
 
 void vertex_property_union(GraphInterface& ugi, GraphInterface& gi,
                            boost::any p_vprop, boost::any p_eprop,
@@ -52,10 +38,11 @@ void vertex_property_union(GraphInterface& ugi, GraphInterface& gi,
     vprop_t vprop = any_cast<vprop_t>(p_vprop);
     eprop_t eprop = any_cast<eprop_t>(p_eprop);
 
-    run_action<graph_tool::detail::always_directed>()
-        (ugi, bind<void>(graph_tool::property_union(),
-                         _1, _2, vprop, eprop, _3, prop),
-         get_pointers::apply<graph_tool::detail::always_directed>::type(),
+    gt_dispatch<>()
+        (std::bind(graph_tool::property_union(), std::placeholders::_1,
+                   std::placeholders::_2, vprop, eprop, std::placeholders::_3,
+                   prop),
+         always_directed(), always_directed(),
          writable_vertex_properties())
-        (gi.GetGraphView(), uprop);
+        (ugi.get_graph_view(), gi.get_graph_view(), uprop);
 }
