@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -15,16 +15,13 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-#include <boost/python.hpp>
-#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
-#include "bytesobject.h"
+#include "graph.hh"
+#include "graph_util.hh"
 
 #define NUMPY_EXPORT
 #include "numpy_bind.hh"
 
-#include "graph.hh"
 #include "graph_python_interface.hh"
-#include "graph_util.hh"
 
 #include "random.hh"
 
@@ -36,6 +33,11 @@
 #include "weave/scxx/str.h"
 #endif
 
+#include <boost/python.hpp>
+#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
+#include <boost/python/stl_iterator.hpp>
+#include "bytesobject.h"
+
 using namespace std;
 using namespace graph_tool;
 using namespace boost;
@@ -60,39 +62,32 @@ struct LibInfo
     }
 };
 
+
 template <class ValueType>
 struct vector_from_list
 {
     vector_from_list()
     {
-        converter::registry::push_back
+         boost::python::converter::registry::push_back
             (&convertible, &construct,
              boost::python::type_id<vector<ValueType> >());
     }
 
     static void* convertible(PyObject* obj_ptr)
     {
-        handle<> x(borrowed(obj_ptr));
-        object o(x);
-        size_t N = len(o);
-        for (size_t i = 0; i < N; ++i)
-        {
-            extract<ValueType> elem(o[i]);
-            if (!elem.check())
-                return 0;
-        }
+        // can't verify without potentially exhausting an iterator
         return obj_ptr;
     }
 
     static void construct(PyObject* obj_ptr,
-                          converter::rvalue_from_python_stage1_data* data)
+                          boost::python::converter::rvalue_from_python_stage1_data* data)
     {
-        handle<> x(borrowed(obj_ptr));
-        object o(x);
+        python::handle<> x(python::borrowed(obj_ptr));
+        python::object o(x);
         vector<ValueType> value;
-        size_t N = len(o);
-        for (size_t i = 0; i < N; ++i)
-            value.push_back(extract<ValueType>(o[i]));
+        python::stl_input_iterator<ValueType> iter(o), end;
+        for (; iter != end; ++iter)
+            value.push_back(*iter);
         void* storage =
             ( (boost::python::converter::rvalue_from_python_storage
                <vector<ValueType> >*) data)->storage.bytes;
@@ -125,28 +120,43 @@ bool vector_nequal_compare(const vector<ValueType>& v1,
 struct export_vector_types
 {
     template <class ValueType>
-    void operator()(ValueType) const
+    void operator()(ValueType, std::string type_name = "") const
     {
-        string type_name = get_type_name<>()(typeid(ValueType));
-        if (type_name == "long double")
-            type_name = "long_double";
+        if (type_name.empty())
+            type_name = get_type_name<>()(typeid(ValueType));
+        std::replace(type_name.begin(), type_name.end(), ' ', '_');
         string name = "Vector_" + type_name;
         class_<vector<ValueType> > vc(name.c_str());
+        std::function<size_t(const vector<ValueType>&)> hasher =
+            [] (const vector<ValueType>& v) -> size_t
+            { return std::hash<vector<ValueType>>()(v); };
+        std::function<void(vector<ValueType>&, size_t size)> resize =
+            [] (vector<ValueType>& v, size_t n) { v.resize(n); };
+        std::function<void(vector<ValueType>&, size_t n)> reserve =
+            [] (vector<ValueType>& v, size_t n) { v.reserve(n); };
+        std::function<void(vector<ValueType>&)> shrink_to_fit =
+            [] (vector<ValueType>& v) { v.shrink_to_fit(); };
+        std::function<bool(vector<ValueType>&)> empty =
+            [] (vector<ValueType>& v) -> bool { return v.empty(); };
         vc.def(vector_indexing_suite<vector<ValueType> >())
             .def("__eq__", &vector_equal_compare<ValueType>)
-            .def("__ne__", &vector_nequal_compare<ValueType>);
-        wrap_array(vc, typename mpl::has_key<numpy_types,ValueType>::type());
+            .def("__ne__", &vector_nequal_compare<ValueType>)
+            .def("__hash__", hasher)
+            .def("resize", resize)
+            .def("shrink_to_fit", shrink_to_fit)
+            .def("empty", empty);
+        wrap_array(vc, typename boost::mpl::has_key<numpy_types,ValueType>::type());
         vector_from_list<ValueType>();
     }
 
     template <class ValueType>
-    void wrap_array(class_<vector<ValueType> >& vc, mpl::true_) const
+    void wrap_array(class_<vector<ValueType> >& vc, boost::mpl::true_) const
     {
         vc.def("get_array", &wrap_vector_not_owned<ValueType>);
     }
 
     template <class ValueType>
-    void wrap_array(class_<vector<ValueType> >& vc, mpl::false_) const
+    void wrap_array(class_<vector<ValueType> >&, boost::mpl::false_) const
     {
     }
 };
@@ -156,15 +166,12 @@ template <class Exception>
 void graph_exception_translator(const Exception& e)
 {
     PyObject* error;
-    if (is_same<Exception, GraphException>::value)
+    if (std::is_same<Exception, GraphException>::value)
         error = PyExc_RuntimeError;
-    if (is_same<Exception, IOException>::value)
+    if (std::is_same<Exception, IOException>::value)
         error = PyExc_IOError;
-    if (is_same<Exception, ValueException>::value)
+    if (std::is_same<Exception, ValueException>::value)
         error = PyExc_ValueError;
-
-    PyObject* message = PyUnicode_FromString(e.what());
-    PyObject_SetAttrString(error, "message", message);
     PyErr_SetString(error, e.what());
 }
 
@@ -196,7 +203,7 @@ struct pair_from_tuple
     {
         handle<> x(borrowed(obj_ptr));
         object o(x);
-        if (python::len(o) < 2)
+        if (boost::python::len(o) < 2)
             return 0;
         extract<T1> first(o[0]);
         extract<T2> second(o[1]);
@@ -211,10 +218,8 @@ struct pair_from_tuple
         handle<> x(borrowed(obj_ptr));
         object o(x);
         pair<T1,T2> value;
-        if (python::len(o) < 2)
-            throw ValueException("Invalid conversion to pair... Sequence is too short.");
-        value.first = extract<T1>(o[0]);
-        value.second = extract<T2>(o[1]);
+        value.first = extract<T1>(o[0])();
+        value.second = extract<T2>(o[1])();
         void* storage =
             ( (boost::python::converter::rvalue_from_python_storage
                <pair<T1,T2> >*) data)->storage.bytes;
@@ -248,7 +253,7 @@ struct variant_from_python
     {
         handle<> x(borrowed(obj_ptr));
         object o(x);
-        ValueType value = extract<ValueType>(o);
+        ValueType value = extract<ValueType>(o)();
         GraphInterface::deg_t deg = value;
         void* storage =
             ( (boost::python::converter::rvalue_from_python_storage
@@ -291,7 +296,7 @@ struct integer_from_convertible
     {
         handle<> x(borrowed(obj_ptr));
         object o(x);
-        T value = extract<T>(o.attr("__int__")());
+        T value = extract<T>(o.attr("__int__")())();
         void* storage =
             ( (boost::python::converter::rvalue_from_python_storage<T>*) data)->storage.bytes;
         new (storage) T(value);
@@ -320,7 +325,7 @@ struct float_from_convertible
     {
         handle<> x(borrowed(obj_ptr));
         object o(x);
-        T value = extract<T>(o.attr("__float__")());
+        T value = extract<T>(o.attr("__float__")())();
         void* storage =
             ( (boost::python::converter::rvalue_from_python_storage<T>*) data)->storage.bytes;
         new (storage) T(value);
@@ -336,30 +341,31 @@ extern python::object object_pickler;
 extern python::object object_unpickler;
 }
 
-void set_pickler(python::object o)
+void set_pickler(boost::python::object o)
 {
     graph_tool::object_pickler = o;
 }
 
-void set_unpickler(python::object o)
+void set_unpickler(boost::python::object o)
 {
     graph_tool::object_unpickler = o;
 }
 
-python::list get_property_types()
+boost::python::list get_property_types()
 {
-    python::list plist;
-    for (int i = 0; i < mpl::size<value_types>::value; ++i)
+    boost::python::list plist;
+    for (int i = 0; i < boost::mpl::size<value_types>::value; ++i)
         plist.append(string(type_names[i]));
     return plist;
 }
 
 struct graph_type_name
 {
+    typedef void result_type;
     template <class Graph>
-    void operator()(const Graph& g, string& name) const
+    void operator()(const Graph&, string& name) const
     {
-        using python::detail::gcc_demangle;
+        using boost::python::detail::gcc_demangle;
         name = string(gcc_demangle(typeid(Graph).name()));
     }
 };
@@ -367,18 +373,16 @@ struct graph_type_name
 string get_graph_type(GraphInterface& g)
 {
     string name;
-    run_action<>()(g, bind<void>(graph_type_name(), _1, ref(name)))();
+    run_action<>()(g, std::bind(graph_type_name(), std::placeholders::_1,
+                                std::ref(name)))();
     return name;
 }
 
-bool openmp_enabled()
+size_t get_ptr(std::shared_ptr<GraphInterface::multigraph_t>& p)
 {
-#ifdef USING_OPENMP
-    return true;
-#else
-    return false;
-#endif
-}
+    return size_t(p.get());
+};
+
 
 // numpy array interface weirdness
 void* do_import_array()
@@ -391,16 +395,35 @@ void ungroup_vector_property(GraphInterface& g, boost::any vector_prop,
                              boost::any prop, size_t pos, bool edge);
 void group_vector_property(GraphInterface& g, boost::any vector_prop,
                            boost::any prop, size_t pos, bool edge);
+void property_map_values(GraphInterface& g, boost::any src_prop,
+                         boost::any tgt_prop, boost::python::object mapper,
+                         bool edge);
 void infect_vertex_property(GraphInterface& gi, boost::any prop,
-                            python::object val);
-void edge_difference(GraphInterface& gi, boost::any prop,
-                     boost::any eprop);
+                            boost::python::object val);
+void edge_endpoint(GraphInterface& gi, boost::any prop,
+                   boost::any eprop, std::string endpoint);
+void out_edges_op(GraphInterface& gi, boost::any eprop, boost::any vprop,
+                  std::string op);
 void mark_edges(GraphInterface& gi, boost::any prop);
 
+void perfect_ehash(GraphInterface& gi, boost::any prop, boost::any hprop,
+                   boost::any& dict);
+void perfect_vhash(GraphInterface& gi, boost::any prop, boost::any hprop,
+                   boost::any& dict);
+void set_vertex_property(GraphInterface& gi, boost::any prop,
+                         boost::python::object val);
+void set_edge_property(GraphInterface& gi, boost::any prop,
+                       boost::python::object val);
+
+
 void export_python_interface();
 
+void export_openmp();
+
 BOOST_PYTHON_MODULE(libgraph_tool_core)
 {
+    using namespace boost::python;
+
     // numpy
     do_import_array();
     export_python_interface();
@@ -419,46 +442,57 @@ BOOST_PYTHON_MODULE(libgraph_tool_core)
     def("raise_error", &raise_error);
     def("get_property_types", &get_property_types);
     class_<boost::any>("any")
-        .def("empty", &boost::any::empty);
+        .def("empty", &boost::any::empty)
+        .def("type",  &boost::any::type,
+             return_value_policy<reference_existing_object>());
+    class_<std::type_info, boost::noncopyable>("type_info", no_init)
+        .def("name", &std::type_info::name)
+        .def("hash_code", &std::type_info::hash_code);
+    def("gcc_demangle", &python::detail::gcc_demangle);
 
     def("graph_filtering_enabled", &graph_filtering_enabled);
-    def("openmp_enabled", &openmp_enabled);
+    export_openmp();
 
-    mpl::for_each<mpl::push_back<scalar_types,string>::type>(export_vector_types());
+    boost::mpl::for_each<boost::mpl::push_back<scalar_types,string>::type>(export_vector_types());
+    export_vector_types()(size_t(), "size_t");
 
     class_<GraphInterface>("GraphInterface", init<>())
-        .def(init<GraphInterface,bool,python::object,python::object,python::object>())
-        .def("GetNumberOfVertices", &GraphInterface::GetNumberOfVertices)
-        .def("GetNumberOfEdges", &GraphInterface::GetNumberOfEdges)
-        .def("SetDirected", &GraphInterface::SetDirected)
-        .def("GetDirected", &GraphInterface::GetDirected)
-        .def("SetReversed", &GraphInterface::SetReversed)
-        .def("GetReversed", &GraphInterface::GetReversed)
-        .def("SetKeepEpos", &GraphInterface::SetKeepEpos)
-        .def("GetKeepEpos", &GraphInterface::GetKeepEpos)
-        .def("SetVertexFilterProperty",
-             &GraphInterface::SetVertexFilterProperty)
-        .def("IsVertexFilterActive", &GraphInterface::IsVertexFilterActive)
-        .def("SetEdgeFilterProperty",
-             &GraphInterface::SetEdgeFilterProperty)
-        .def("IsEdgeFilterActive", &GraphInterface::IsEdgeFilterActive)
-        .def("PurgeVertices",  &GraphInterface::PurgeVertices)
-        .def("PurgeEdges",  &GraphInterface::PurgeEdges)
-        .def("ShiftVertexProperty",  &GraphInterface::ShiftVertexProperty)
-        .def("MoveVertexProperty",  &GraphInterface::MoveVertexProperty)
-        .def("ReIndexVertexProperty",  &GraphInterface::ReIndexVertexProperty)
-        .def("WriteToFile", &GraphInterface::WriteToFile)
-        .def("ReadFromFile",&GraphInterface::ReadFromFile)
-        .def("DegreeMap", &GraphInterface::DegreeMap)
-        .def("Clear", &GraphInterface::Clear)
-        .def("ClearEdges", &GraphInterface::ClearEdges)
-        .def("GetVertexIndex", &GraphInterface::GetVertexIndex)
-        .def("GetEdgeIndex", &GraphInterface::GetEdgeIndex)
-        .def("GetMaxEdgeIndex", &GraphInterface::GetMaxEdgeIndex)
-        .def("ReIndexEdges", &GraphInterface::ReIndexEdges)
-        .def("GetGraphIndex", &GraphInterface::GetGraphIndex)
-        .def("CopyVertexProperty", &GraphInterface::CopyVertexProperty)
-        .def("CopyEdgeProperty", &GraphInterface::CopyEdgeProperty);
+        .def(init<GraphInterface,bool,boost::python::object,
+                  boost::python::object, boost::python::object>())
+        .def("get_num_vertices", &GraphInterface::get_num_vertices)
+        .def("get_num_edges", &GraphInterface::get_num_edges)
+        .def("set_directed", &GraphInterface::set_directed)
+        .def("get_directed", &GraphInterface::get_directed)
+        .def("set_reversed", &GraphInterface::set_reversed)
+        .def("get_reversed", &GraphInterface::get_reversed)
+        .def("set_keep_epos", &GraphInterface::set_keep_epos)
+        .def("get_keep_epos", &GraphInterface::get_keep_epos)
+        .def("set_vertex_filter_property",
+             &GraphInterface::set_vertex_filter_property)
+        .def("is_vertex_filter_active", &GraphInterface::is_vertex_filter_active)
+        .def("set_edge_filter_property",
+             &GraphInterface::set_edge_filter_property)
+        .def("is_edge_filter_active", &GraphInterface::is_edge_filter_active)
+        .def("purge_vertices",  &GraphInterface::purge_vertices)
+        .def("purge_edges",  &GraphInterface::purge_edges)
+        .def("shift_vertex_property",  &GraphInterface::shift_vertex_property)
+        .def("move_vertex_property",  &GraphInterface::move_vertex_property)
+        .def("re_index_vertex_property",  &GraphInterface::re_index_vertex_property)
+        .def("write_to_file", &GraphInterface::write_to_file)
+        .def("read_from_file",&GraphInterface::read_from_file)
+        .def("degree_map", &GraphInterface::degree_map)
+        .def("clear", &GraphInterface::clear)
+        .def("clear_edges", &GraphInterface::clear_edges)
+        .def("get_vertex_index", &GraphInterface::get_vertex_index)
+        .def("get_edge_index", &GraphInterface::get_edge_index)
+        .def("get_edge_index_range", &GraphInterface::get_edge_index_range)
+        .def("re_index_edges", &GraphInterface::re_index_edges)
+        .def("shrink_to_fit", &GraphInterface::shrink_to_fit)
+        .def("get_graph_index", &GraphInterface::get_graph_index)
+        .def("copy_vertex_property", &GraphInterface::copy_vertex_property)
+        .def("copy_edge_property", &GraphInterface::copy_edge_property)
+        .def("get_graph_ptr", &GraphInterface::get_graph_ptr)
+        .def("get_graph_view", &GraphInterface::get_graph_view);
 
     class_<GraphInterface::vertex_index_map_t>("vertex_index_map", no_init);
     class_<GraphInterface::edge_index_map_t>("edge_index_map", no_init);
@@ -487,6 +521,10 @@ BOOST_PYTHON_MODULE(libgraph_tool_core)
     float_from_convertible<double>();
     float_from_convertible<long double>();
 
+    class_<std::shared_ptr<GraphInterface::multigraph_t>>
+        ("shared_ptr<multigraph_t>", no_init)
+        .def("get", &get_ptr);
+
 #ifdef HAVE_SCIPY
     to_python_converter<py::object, scxx_to_python<py::object> >();
     to_python_converter<py::tuple, scxx_to_python<py::tuple> >();
@@ -495,17 +533,23 @@ BOOST_PYTHON_MODULE(libgraph_tool_core)
     to_python_converter<py::str, scxx_to_python<py::str> >();
 #endif
 
-    class_<IStream>("IStream", no_init).def("Read", &IStream::Read);
-    class_<OStream>("OStream", no_init).def("Write", &OStream::Write).
-        def("Flush", &OStream::Flush);
+    class_<IStream>("IStream", no_init).def("read", &IStream::read);
+    class_<OStream>("OStream", no_init).def("write", &OStream::write).
+        def("flush", &OStream::flush);
     def("set_pickler", &set_pickler);
     def("set_unpickler", &set_unpickler);
 
     def("group_vector_property", &group_vector_property);
     def("ungroup_vector_property", &ungroup_vector_property);
+    def("property_map_values", &property_map_values);
     def("infect_vertex_property", &infect_vertex_property);
-    def("edge_difference", &edge_difference);
+    def("edge_endpoint", &edge_endpoint);
+    def("out_edges_op", &out_edges_op);
     def("mark_edges", &mark_edges);
+    def("perfect_ehash", &perfect_ehash);
+    def("perfect_vhash", &perfect_vhash);
+    def("set_vertex_property", &set_vertex_property);
+    def("set_edge_property", &set_edge_property);
 
     class_<LibInfo>("mod_info")
         .add_property("name", &LibInfo::GetName)
