@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -28,57 +28,6 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-template <class Descriptor, class GraphTgt, class GraphSrc, class IndexMap>
-struct copy_property_dispatch
-{
-    copy_property_dispatch(const Descriptor& src_d, const Descriptor& tgt_d,
-                           const GraphTgt& tgt, const GraphSrc& src,
-                           boost::any& prop_src, boost::any& prop_tgt,
-                           IndexMap& index_map, bool& found)
-        : src_d(src_d), tgt_d(tgt_d), tgt(tgt), src(src), prop_src(prop_src),
-          prop_tgt(prop_tgt), index_map(index_map), found(found) {}
-
-
-    const Descriptor& src_d;
-    const Descriptor& tgt_d;
-    const GraphTgt& tgt;
-    const GraphSrc& src;
-    boost::any& prop_src;
-    boost::any& prop_tgt;
-    IndexMap& index_map;
-    bool& found;
-
-    template <class PropertyMap>
-    void operator()(PropertyMap) const
-    {
-        PropertyMap* psrc = any_cast<PropertyMap>(&prop_src);
-        if (psrc == NULL)
-            return;
-        if (prop_tgt.empty())
-            prop_tgt = PropertyMap(index_map);
-        PropertyMap* ptgt = any_cast<PropertyMap>(&prop_tgt);
-        if (ptgt == NULL)
-            return;
-        (*ptgt)[tgt_d] = (*psrc)[src_d];
-        found = true;
-    }
-};
-
-template <class PropertyMaps, class Descriptor, class GraphTgt, class GraphSrc,
-          class IndexMap>
-void copy_property(const Descriptor& src_d, const Descriptor& tgt_d,
-                   boost::any& prop_src, boost::any& prop_tgt,
-                   const GraphTgt& tgt, const GraphSrc& src,
-                   IndexMap& index_map)
-{
-    bool found = false;
-    mpl::for_each<PropertyMaps>(copy_property_dispatch<Descriptor, GraphTgt, GraphSrc, IndexMap>
-                                (src_d, tgt_d, tgt, src, prop_src, prop_tgt, index_map,
-                                 found));
-    if (!found)
-        throw ValueException("Cannot find property map type.");
-}
-
 template <class GraphSrc, class GraphTgt, class IndexMap, class SrcIndexMap,
           class TgtIndexMap>
 struct copy_vertex_property_dispatch
@@ -117,20 +66,18 @@ struct copy_vertex_property_dispatch
             return;
         found = true;
 
-        typename PropertyMap::unchecked_t p_src =
-            psrc->get_unchecked(num_vertices(src));
-        typename PropertyMap::unchecked_t p_tgt =
-            ptgt->get_unchecked(num_vertices(tgt));
+        auto p_src = psrc->get_unchecked(num_vertices(src));
+        auto p_tgt = ptgt->get_unchecked(num_vertices(tgt));
 
         int i, N = num_vertices(src);
-        #pragma omp parallel for default(shared) private(i) schedule(static) if (N > 100)
+        #pragma omp parallel for default(shared) private(i) \
+            schedule(runtime) if (N > 100)
         for (i = 0; i < N; ++i)
         {
-            typename graph_traits<GraphSrc>::vertex_descriptor v = vertex(i, src);
-            if (v == graph_traits<GraphSrc>::null_vertex())
+            auto v = vertex(i, src);
+            if (!is_valid_vertex(v, src))
                 continue;
-            typename graph_traits<GraphTgt>::vertex_descriptor new_v =
-                vertex(index_map[i], tgt);
+            auto new_v = vertex(index_map[i], tgt);
             p_tgt[new_v] = p_src[v];
         }
     }
@@ -144,9 +91,9 @@ void copy_vertex_property(boost::any& prop_src, boost::any& prop_tgt,
                           TgtIndexMap& tgt_vertex_index)
 {
     bool found = false;
-    mpl::for_each<PropertyMaps>(copy_vertex_property_dispatch<GraphSrc, GraphTgt,
-                                                              IndexMap, SrcIndexMap,
-                                                              TgtIndexMap>
+    boost::mpl::for_each<PropertyMaps>(copy_vertex_property_dispatch<GraphSrc, GraphTgt,
+                                                                     IndexMap, SrcIndexMap,
+                                                                     TgtIndexMap>
             (src, tgt, prop_src, prop_tgt, index_map, src_vertex_index,
              tgt_vertex_index, found));
     if (!found)
@@ -191,31 +138,24 @@ struct copy_edge_property_dispatch
             return;
         found = true;
 
-        typename PropertyMap::unchecked_t p_src =
-            psrc->get_unchecked(max_src_edge_index + 1);
-        typename PropertyMap::unchecked_t p_tgt =
-            ptgt->get_unchecked(num_edges(tgt));
+        auto p_src = psrc->get_unchecked(max_src_edge_index + 1);
+        auto p_tgt = ptgt->get_unchecked(num_edges(tgt));
 
-        int i, N = num_vertices(src);
-        #pragma omp parallel for default(shared) private(i) schedule(static) if (N > 100)
-        for (i = 0; i < N; ++i)
-        {
-            typename graph_traits<GraphSrc>::vertex_descriptor v = vertex(i, src);
-            if (v == graph_traits<GraphSrc>::null_vertex())
-                continue;
-
-            typename graph_traits<GraphSrc>::out_edge_iterator e, e_end;
-            for (tie(e, e_end) = out_edges(v, src); e != e_end; ++e)
-            {
-                typename graph_traits<GraphSrc>::vertex_descriptor s = source(*e, src);
-                typename graph_traits<GraphSrc>::vertex_descriptor t = target(*e, src);
-                if (!is_directed::apply<GraphSrc>::type::value && s > t)
-                    continue;
-                size_t ei = src_edge_index[*e];
-                typename graph_traits<GraphTgt>::edge_descriptor new_e = index_map[ei];
-                p_tgt[new_e] = p_src[*e];
-            }
-        }
+        parallel_vertex_loop
+            (src,
+             [&](auto v)
+             {
+                 for (auto e : out_edges_range(v, src))
+                 {
+                     auto s = source(e, src);
+                     auto t = target(e, src);
+                     if (!is_directed::apply<GraphSrc>::type::value && s > t)
+                         continue;
+                     size_t ei = src_edge_index[e];
+                     const auto& new_e = index_map[ei];
+                     p_tgt[new_e] = p_src[e];
+                 }
+             });
     }
 };
 
@@ -227,8 +167,8 @@ void copy_edge_property(boost::any& prop_src, boost::any& prop_tgt,
                         size_t max_src_edge_index)
 {
     bool found = false;
-    mpl::for_each<PropertyMaps>(copy_edge_property_dispatch<GraphSrc, GraphTgt,
-                                                            IndexMap, SrcIndexMap>
+    boost::mpl::for_each<PropertyMaps>(copy_edge_property_dispatch<GraphSrc, GraphTgt,
+                                                                   IndexMap, SrcIndexMap>
             (src, tgt, prop_src, prop_tgt, index_map, src_vertex_index,
              max_src_edge_index, found));
     if (!found)
@@ -251,20 +191,18 @@ struct do_graph_copy
                     TgtEdgeIndexMap,
                     SrcEdgeIndexMap src_edge_index,
                     OrderMap vertex_order,
-                    vector<pair<reference_wrapper<boost::any>,reference_wrapper<boost::any> > >& vprops,
-                    vector<pair<reference_wrapper<boost::any>,reference_wrapper<boost::any> > >& eprops) const
+                    vector<pair<std::reference_wrapper<boost::any>,std::reference_wrapper<boost::any>>>& vprops,
+                    vector<pair<std::reference_wrapper<boost::any>,std::reference_wrapper<boost::any>>>& eprops) const
     {
         vector<size_t> index_map(num_vertices(src));
-        typename graph_traits<GraphSrc>::vertex_iterator v, v_end;
-        for (tie(v, v_end) = vertices(src); v != v_end; ++v)
+        for (auto v : vertices_range(src))
         {
-            if (src_vertex_index[*v] >= index_map.size())
-                index_map.resize(src_vertex_index[*v]+1);
-            typename graph_traits<GraphTgt>::vertex_descriptor new_v =
-                get(vertex_order, *v);
-            while (new_v >= num_vertices(tgt))
+            if (src_vertex_index[v] >= index_map.size())
+                index_map.resize(src_vertex_index[v] + 1);
+            auto new_v = get(vertex_order, v);
+            while (size_t(new_v) >= num_vertices(tgt))
                 add_vertex(tgt);
-            index_map[src_vertex_index[*v]] = tgt_vertex_index[new_v];
+            index_map[src_vertex_index[v]] = tgt_vertex_index[new_v];
         }
 
         for (size_t i = 0; i < vprops.size(); ++i)
@@ -274,15 +212,13 @@ struct do_graph_copy
 
 
         vector<typename graph_traits<GraphTgt>::edge_descriptor> edge_map(num_edges(src));
-        typename graph_traits<GraphSrc>::edge_iterator e, e_end;
-        for (tie(e, e_end) = edges(src); e != e_end; ++e)
+        for (auto e : edges_range(src))
         {
-            size_t s = index_map[src_vertex_index[source(*e, src)]];
-            size_t t = index_map[src_vertex_index[target(*e, src)]];
-            typedef typename graph_traits<GraphTgt>::edge_descriptor edge_t;
-            edge_t new_e = add_edge(vertex(s,tgt), vertex(t,tgt), tgt).first;
+            size_t s = index_map[src_vertex_index[source(e, src)]];
+            size_t t = index_map[src_vertex_index[target(e, src)]];
+            auto new_e = add_edge(vertex(s,tgt), vertex(t,tgt), tgt).first;
 
-            size_t ei = src_edge_index[*e];
+            size_t ei = src_edge_index[e];
             if (ei >= edge_map.size())
                 edge_map.resize(ei + 1);
 
@@ -301,7 +237,7 @@ struct do_graph_copy
 GraphInterface::GraphInterface(const GraphInterface& gi, bool keep_ref,
                                python::object ovprops, python::object oeprops,
                                python::object vorder)
-    :_mg(keep_ref ? gi._mg : shared_ptr<multigraph_t>(new multigraph_t())),
+    :_mg(keep_ref ? gi._mg : std::make_shared<multigraph_t>()),
      _vertex_index(get(vertex_index, *_mg)),
      _edge_index(get(edge_index_t(), *_mg)),
      _reversed(gi._reversed),
@@ -323,25 +259,27 @@ GraphInterface::GraphInterface(const GraphInterface& gi, bool keep_ref,
         return;
     }
 
-    vector<pair<reference_wrapper<boost::any>,reference_wrapper<boost::any> > > vprops;
+    vector<pair<std::reference_wrapper<boost::any>,std::reference_wrapper<boost::any>>> vprops;
     for (int i = 0; i < python::len(ovprops); ++i)
     {
-        vprops.push_back(make_pair(ref(python::extract<boost::any&>(ovprops[i][0])()),
-                                   ref(python::extract<boost::any&>(ovprops[i][1])())));
+        vprops.push_back(make_pair(std::ref(python::extract<boost::any&>(ovprops[i][0])()),
+                                   std::ref(python::extract<boost::any&>(ovprops[i][1])())));
     }
-    vector<pair<reference_wrapper<boost::any>,reference_wrapper<boost::any> > > eprops;
+    vector<pair<std::reference_wrapper<boost::any>,std::reference_wrapper<boost::any>>> eprops;
     for (int i = 0; i < python::len(oeprops); ++i)
     {
-        eprops.push_back(make_pair(ref(python::extract<boost::any&>(oeprops[i][0])()),
-                                   ref(python::extract<boost::any&>(oeprops[i][1])())));
+        eprops.push_back(make_pair(std::ref(python::extract<boost::any&>(oeprops[i][0])()),
+                                   std::ref(python::extract<boost::any&>(oeprops[i][1])())));
     }
 
-    boost::any avorder = python::extract<boost::any>(vorder);
+    boost::any avorder = python::extract<boost::any>(vorder)();
     run_action<>()
         (const_cast<GraphInterface&>(gi),
-         bind<void>(do_graph_copy(gi._mg->get_last_index()), _1, ref(*_mg),
-                    gi._vertex_index, _vertex_index, gi._edge_index,
-                    _edge_index, _2, ref(vprops), ref(eprops)),
+         std::bind(do_graph_copy(gi._mg->get_edge_index_range()),
+                   std::placeholders::_1, std::ref(*_mg),
+                   gi._vertex_index, _vertex_index, gi._edge_index,
+                   _edge_index, std::placeholders::_2, std::ref(vprops),
+                   std::ref(eprops)),
          vertex_scalar_properties())(avorder);
     // filters will be copied in python
 }
