@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -18,7 +18,6 @@
 #include <boost/python/extract.hpp>
 
 #include <iostream>
-#include <boost/algorithm/string.hpp>
 #include <boost/iostreams/categories.hpp>
 #include <boost/iostreams/filtering_stream.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
@@ -26,11 +25,9 @@
 #include <boost/iostreams/device/file_descriptor.hpp>
 #include <boost/iostreams/device/file.hpp>
 #include <boost/graph/graphml.hpp>
-#include <boost/graph/graphviz.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/xpressive/xpressive.hpp>
 
-#include "gml.hh"
 #include "graph.hh"
 #include "graph_filtering.hh"
 #include "graph_properties.hh"
@@ -39,6 +36,8 @@
 #include "graph_python_interface.hh"
 #include "str_repr.hh"
 
+#include "gml.hh"
+
 using namespace std;
 using namespace boost;
 using namespace graph_tool;
@@ -79,7 +78,20 @@ python::object lexical_cast<python::object,string>(const string& ps)
     o = object_unpickler(IStream(s));
     return o;
 }
+
+namespace python
+{
+std::ostringstream& operator<<(std::ostringstream& stream, const boost::python::object& o)
+{
+    stream << base64_encode(lexical_cast<string>(o));
+    return stream;
 }
+}
+}
+
+#include <boost/graph/graphviz.hpp>
+
+#include "graph_io_binary.hh"
 
 // the following source & sink provide iostream access to python file-like
 // objects
@@ -90,15 +102,11 @@ public:
     typedef char                 char_type;
     typedef iostreams::seekable_device_tag  category;
 
-    python_file_device(python::object file): _file(file) {}
+    python_file_device(boost::python::object file): _file(file) {}
     std::streamsize read(char* s, std::streamsize n)
     {
-        python::object pbuf = _file.attr("read")(n);
-#if (PY_MAJOR_VERSION >= 3)
-        string buf = python::extract<string>(pbuf.attr("decode")("utf-8"));
-#else
-        string buf = python::extract<string>(pbuf);
-#endif
+        boost::python::object pbuf = _file.attr("read")(n);
+        string buf = boost::python::extract<string>(pbuf);
         for (size_t i = 0; i < buf.size(); ++i)
             s[i] = buf[i];
         return buf.size();
@@ -107,8 +115,15 @@ public:
     std::streamsize write(const char* s, std::streamsize n)
     {
         string buf(s, s+n);
-        python::object pbuf(buf);
-        _file.attr("write")(pbuf.attr("encode")("utf-8"));
+#if (PY_MAJOR_VERSION >= 3)
+        // in python 3 we need to construct a 'bytes' instance
+        PyObject* bytes = PyBytes_FromStringAndSize(s, n);
+        boost::python::handle<> x(bytes);
+        boost::python::object pbuf(x);
+#else
+        boost::python::str pbuf(buf);
+#endif
+        _file.attr("write")(pbuf);
         return n;
     }
 
@@ -116,11 +131,11 @@ public:
                                   std::ios_base::seekdir way)
     {
         _file.attr("seek")(off, int(way));
-        return python::extract<iostreams::stream_offset>(_file.attr("tell")());
+        return boost::python::extract<iostreams::stream_offset>(_file.attr("tell")());
     }
 
 private:
-    python::object _file;
+    boost::python::object _file;
 };
 
 // Property Maps
@@ -130,28 +145,53 @@ struct get_python_property
 {
     template <class ValueType, class IndexMap>
     void operator()(ValueType, IndexMap, dynamic_property_map& map,
-                    python::object& pmap) const
+                    boost::python::object& pmap) const
     {
         typedef typename property_map_type::apply<ValueType, IndexMap>::type
             map_t;
         try
         {
-            pmap = python::object
+            pmap = boost::python::object
                 (PythonPropertyMap<map_t>
                  (dynamic_cast
                   <boost::detail::dynamic_property_map_adaptor<map_t>&>(map)
                   .base()));
         } catch (bad_cast&) {}
     }
+
+    template <class ValueType, class IndexMap>
+    void operator()(ValueType, IndexMap, boost::any& map,
+                    boost::python::object& pmap) const
+    {
+        typedef typename property_map_type::apply<ValueType, IndexMap>::type
+            map_t;
+        try
+        {
+            PythonPropertyMap<map_t> opmap(any_cast<map_t>(map));
+            pmap = boost::python::object(opmap);
+        }
+        catch (bad_any_cast&) {}
+    }
+
 };
 
 template <class IndexMap>
-python::object find_property_map(dynamic_property_map& map, IndexMap)
+boost::python::object find_property_map(dynamic_property_map& map, IndexMap)
 {
-    python::object pmap;
-    mpl::for_each<value_types>(boost::bind<void>(get_python_property(),
-                                                 _1, IndexMap(), ref(map),
-                                                 boost::ref(pmap)));
+    boost::python::object pmap;
+    boost::mpl::for_each<value_types>(std::bind(get_python_property(),
+                                                std::placeholders::_1, IndexMap(), std::ref(map),
+                                                std::ref(pmap)));
+    return pmap;
+}
+
+template <class IndexMap>
+boost::python::object find_property_map(boost::any& map, IndexMap)
+{
+    boost::python::object pmap;
+    boost::mpl::for_each<value_types>(std::bind(get_python_property(),
+                                                std::placeholders::_1, IndexMap(), std::ref(map),
+                                                std::ref(pmap)));
     return pmap;
 }
 
@@ -204,7 +244,7 @@ struct create_dynamic_map
         dynamic_property_map* map;
         try
         {
-            mpl::for_each<value_types>
+            boost::mpl::for_each<value_types>
                 (check_value_type<VertexIndexMap>(_vertex_map,
                                                   any_cast<vertex_t>(key),
                                                   value, map));
@@ -213,7 +253,7 @@ struct create_dynamic_map
         {
             try
             {
-                mpl::for_each<value_types>
+                boost::mpl::for_each<value_types>
                     (check_value_type<EdgeIndexMap>(_edge_map,
                                                     any_cast<edge_t>(key),
                                                     value, map));
@@ -221,7 +261,7 @@ struct create_dynamic_map
             catch (bad_any_cast)
             {
                 ConstantPropertyMap<size_t,graph_property_tag> graph_index(0);
-                mpl::for_each<value_types>
+                boost::mpl::for_each<value_types>
                     (check_value_type<ConstantPropertyMap<size_t,
                                                           graph_property_tag> >
                      (graph_index, any_cast<graph_property_tag>(key),
@@ -274,23 +314,33 @@ struct graph_traits<FakeUndirGraph<Graph> >
     typedef typename graph_traits<Graph>::edge_descriptor edge_descriptor;
     typedef FakeEdgeIterator<Graph> edge_iterator;
 };
+
+template <class Graph>
+std::pair<FakeEdgeIterator<Graph>,
+          FakeEdgeIterator<Graph>> edges(const FakeUndirGraph<Graph>& g)
+{
+    auto e = edges(UndirectedAdaptor<Graph>(g));
+    return std::make_pair(FakeEdgeIterator<Graph>(e.first),
+                          FakeEdgeIterator<Graph>(e.second));
+}
+
 }
 
 
 //==============================================================================
-// ReadFromFile(file, pfile, format)
+// read_from_file(file, pfile, format)
 //==============================================================================
 
-void build_stream
-    (boost::iostreams::filtering_stream<boost::iostreams::input>& stream,
-     const string& file,  python::object& pfile, std::ifstream& file_stream)
+void build_stream(boost::iostreams::filtering_stream<boost::iostreams::input>& stream,
+                  const string& file, boost::python::object& pfile,
+                  std::ifstream& file_stream)
 {
     stream.reset();
     if (file == "-")
         stream.push(std::cin);
     else
     {
-        if (pfile == python::object())
+        if (pfile == boost::python::object())
         {
             file_stream.open(file.c_str(), std::ios_base::in |
                              std::ios_base::binary);
@@ -311,13 +361,14 @@ void build_stream
 }
 
 
-python::tuple GraphInterface::ReadFromFile(string file, python::object pfile,
-                                           string format,
-                                           python::list ignore_vp,
-                                           python::list ignore_ep,
-                                           python::list ignore_gp)
+boost::python::tuple GraphInterface::read_from_file(string file,
+                                                    boost::python::object pfile,
+                                                    string format,
+                                                    boost::python::list ignore_vp,
+                                                    boost::python::list ignore_ep,
+                                                    boost::python::list ignore_gp)
 {
-    if (format != "dot" && format != "xml" && format != "gml")
+    if (format != "gt" && format != "dot" && format != "xml" && format != "gml")
         throw ValueException("error reading from file '" + file +
                              "': requested invalid format '" + format + "'");
     try
@@ -327,13 +378,13 @@ python::tuple GraphInterface::ReadFromFile(string file, python::object pfile,
         std::ifstream file_stream;
         build_stream(stream, file, pfile, file_stream);
 
-        set<string> ivp, iep, igp;
+        std::unordered_set<std::string> ivp, iep, igp;
         for (int i = 0; i < len(ignore_vp); ++i)
-            ivp.insert(python::extract<string>(ignore_vp[i]));
+            ivp.insert(boost::python::extract<string>(ignore_vp[i]));
         for (int i = 0; i < len(ignore_ep); ++i)
-            iep.insert(python::extract<string>(ignore_ep[i]));
+            iep.insert(boost::python::extract<string>(ignore_ep[i]));
         for (int i = 0; i < len(ignore_gp); ++i)
-            igp.insert(python::extract<string>(ignore_gp[i]));
+            igp.insert(boost::python::extract<string>(ignore_gp[i]));
 
         create_dynamic_map<vertex_index_map_t,edge_index_map_t>
             map_creator(_vertex_index, _edge_index);
@@ -342,27 +393,44 @@ python::tuple GraphInterface::ReadFromFile(string file, python::object pfile,
 
         if (format == "dot")
             _directed = read_graphviz(stream, *_mg, dp, "vertex_name", true,
-                                     ivp, iep, igp);
+                                      ivp, iep, igp);
         else if (format == "xml")
             _directed = read_graphml(stream, *_mg, dp, true, true, true,
                                      ivp, iep, igp);
         else if (format == "gml")
             _directed = read_gml(stream, *_mg, dp, ivp, iep, igp);
 
-        python::dict vprops, eprops, gprops;
-        for(typeof(dp.begin()) iter = dp.begin(); iter != dp.end(); ++iter)
+
+        boost::python::dict vprops, eprops, gprops;
+        if (format == "gt")
         {
-            if (iter->second->key() == typeid(vertex_t))
-                vprops[iter->first] = find_property_map(*iter->second,
-                                                       _vertex_index);
-            else if (iter->second->key() == typeid(edge_t))
-                eprops[iter->first] = find_property_map(*iter->second,
-                                                       _edge_index);
-            else
-                gprops[iter->first] = find_property_map(*iter->second,
-                                                        _graph_index);
+            vector<pair<string, boost::any>> agprops, avprops, aeprops;
+            stream.exceptions(ios_base::badbit | ios_base::failbit | ios_base::eofbit);
+            _directed = read_graph(stream, *_mg, agprops, avprops, aeprops, igp,
+                                   ivp, iep);
+            for (auto& p : agprops)
+                gprops[p.first] = find_property_map(p.second, _graph_index);
+            for (auto& p : avprops)
+                vprops[p.first] = find_property_map(p.second, _vertex_index);
+            for (auto& p : aeprops)
+                eprops[p.first] = find_property_map(p.second, _edge_index);
         }
-        return python::make_tuple(vprops, eprops, gprops);
+        else
+        {
+            for(auto iter = dp.begin(); iter != dp.end(); ++iter)
+            {
+                if (iter->second->key() == typeid(vertex_t))
+                    vprops[iter->first] = find_property_map(*iter->second,
+                                                            _vertex_index);
+                else if (iter->second->key() == typeid(edge_t))
+                    eprops[iter->first] = find_property_map(*iter->second,
+                                                            _edge_index);
+                else
+                    gprops[iter->first] = find_property_map(*iter->second,
+                                                            _graph_index);
+            }
+        }
+        return boost::python::make_tuple(vprops, eprops, gprops);
     }
     catch (ios_base::failure &e)
     {
@@ -384,8 +452,7 @@ string graphviz_insert_index(dynamic_properties& dp, IndexMap index_map,
 {
     typedef GraphInterface::vertex_t vertex_t;
     bool found = false;
-    for(typeof(dp.begin()) iter = dp.begin(); iter != dp.end();
-        ++iter)
+    for(auto iter = dp.begin(); iter != dp.end(); ++iter)
         if (iter->first == "vertex_name" &&
             iter->second->key() == typeid(vertex_t))
             found = true;
@@ -399,14 +466,12 @@ string graphviz_insert_index(dynamic_properties& dp, IndexMap index_map,
 
 // writes a graph to a file
 
-struct write_to_file
+struct do_write_to_file
 {
     template <class Graph, class IndexMap>
     void operator()(ostream& stream, Graph& g, IndexMap index_map,
                     dynamic_properties& dp, const string& format) const
     {
-        typedef typename graph_traits<Graph>::vertex_descriptor vertex_t;
-
         if (format == "dot")
         {
             string name = graphviz_insert_index(dp, index_map, false);
@@ -423,7 +488,7 @@ struct write_to_file
     }
 };
 
-struct write_to_file_fake_undir: public write_to_file
+struct do_write_to_file_fake_undir: public do_write_to_file
 {
     template <class Graph, class IndexMap>
     void operator()(ostream& stream, Graph& g, IndexMap index_map,
@@ -431,7 +496,19 @@ struct write_to_file_fake_undir: public write_to_file
     {
         typedef typename Graph::original_graph_t graph_t;
         FakeUndirGraph<graph_t> ug(g);
-        write_to_file(*this)(stream, ug, index_map, dp, format);
+        do_write_to_file(*this)(stream, ug, index_map, dp, format);
+    }
+};
+
+struct do_write_to_binary_file
+{
+    template <class Graph, class IndexMap>
+    void operator()(ostream& stream, Graph& g, IndexMap index_map, size_t N,
+                    bool directed, vector<pair<string, boost::any >> & gprops,
+                    vector<pair<string, boost::any >> & vprops,
+                    vector<pair<string, boost::any >> & eprops) const
+    {
+        write_graph(g, index_map, N, directed, gprops, vprops, eprops, stream);
     }
 };
 
@@ -447,10 +524,10 @@ struct generate_index
     }
 };
 
-void GraphInterface::WriteToFile(string file, python::object pfile,
-                                 string format, python::list props)
+void GraphInterface::write_to_file(string file, boost::python::object pfile,
+                                   string format, boost::python::list props)
 {
-    if (format != "xml" && format != "dot" && format != "gml")
+    if (format != "gt" && format != "xml" && format != "dot" && format != "gml")
         throw ValueException("error writing to file '" + file +
                              "': requested invalid format '" + format + "'");
     try
@@ -461,7 +538,7 @@ void GraphInterface::WriteToFile(string file, python::object pfile,
             stream.push(std::cout);
         else
         {
-            if (pfile == python::object())
+            if (pfile == boost::python::object())
             {
                 file_stream.open(file.c_str(), std::ios_base::out |
                                  std::ios_base::binary);
@@ -480,57 +557,112 @@ void GraphInterface::WriteToFile(string file, python::object pfile,
         }
         stream.exceptions(ios_base::badbit | ios_base::failbit);
 
-        dynamic_properties dp;
-        for (int i = 0; i < len(props); ++i)
+        if (format == "gt")
         {
-            dynamic_property_map* pmap =
-                any_cast<dynamic_property_map*>
-                (python::extract<boost::any>
-                 (props[i][1].attr("get_dynamic_map")()));
-            dp.insert(python::extract<string>(props[i][0]),
-                      DP_SMART_PTR<dynamic_property_map>(pmap));
-        }
+            typedef property_map_types::apply<value_types,
+                                              GraphInterface::graph_index_map_t>::type
+                graph_properties;
 
-        if (IsVertexFilterActive())
-        {
-            // vertex indexes must be between the [0, HardNumVertices(g)] range
-            typedef tr1::unordered_map<vertex_t, size_t>  map_t;
-            map_t vertex_to_index;
-            associative_property_map<map_t> index_map(vertex_to_index);
-            run_action<>()(*this, boost::bind<void>(generate_index(),
-                                                    _1, index_map))();
-            if (format == "dot")
-                graphviz_insert_index(dp, index_map);
+            vector<pair<string, boost::any>> agprops, avprops, aeprops;
+            for (int i = 0; i < python::len(props); ++i)
+            {
+                string name = python::extract<string>(props[i][0])();
+                boost::any p = python::extract<boost::any>(props[i][1].attr("get_map")())();
+                if (belongs<graph_properties>()(p))
+                    agprops.push_back(make_pair(name, p));
+                if (belongs<vertex_properties>()(p))
+                    avprops.push_back(make_pair(name, p));
+                if (belongs<edge_properties>()(p))
+                    aeprops.push_back(make_pair(name, p));
+            }
 
-            if (GetDirected())
-                run_action<detail::always_directed>()
-                    (*this, boost::bind<void>(write_to_file(),
-                                              boost::ref(stream), _1,
-                                              index_map, boost::ref(dp),
-                                              format))();
+            bool directed = _directed;
+            _directed = true;
+
+            if (is_vertex_filter_active())
+            {
+                // vertex indexes must be between the [0, HardNumVertices(g)] range
+                vector_property_map<size_t> index_map(_vertex_index);
+                run_action<>()(*this, std::bind(generate_index(),
+                                                std::placeholders::_1,
+                                                index_map))();
+                run_action<>()(*this, std::bind(do_write_to_binary_file(),
+                                                std::ref(stream),
+                                                std::placeholders::_1,
+                                                index_map,
+                                                get_num_vertices(),
+                                                directed,
+                                                std::ref(agprops),
+                                                std::ref(avprops),
+                                                std::ref(aeprops)))();
+            }
             else
-                run_action<detail::never_directed>()
-                    (*this,boost::bind<void>(write_to_file_fake_undir(),
-                                             boost::ref(stream), _1, index_map,
-                                             boost::ref(dp), format))();
+            {
+                run_action<>()(*this, std::bind(do_write_to_binary_file(),
+                                                std::ref(stream),
+                                                std::placeholders::_1,
+                                                _vertex_index,
+                                                get_num_vertices(),
+                                                directed,
+                                                std::ref(agprops),
+                                                std::ref(avprops),
+                                                std::ref(aeprops)))();
+            }
+
+            _directed = directed;
         }
         else
         {
-            if (format == "dot")
-                graphviz_insert_index(dp, _vertex_index);
+            dynamic_properties dp;
+            for (int i = 0; i < len(props); ++i)
+            {
+                dynamic_property_map* pmap =
+                    any_cast<dynamic_property_map*>
+                    (boost::python::extract<boost::any>
+                     (props[i][1].attr("get_dynamic_map")()));
+                dp.insert(boost::python::extract<string>(props[i][0]),
+                          DP_SMART_PTR<dynamic_property_map>(pmap));
+            }
 
-            if (GetDirected())
-                run_action<detail::always_directed>()
-                    (*this, boost::bind<void>(write_to_file(),
-                                              boost::ref(stream), _1,
-                                              _vertex_index,  boost::ref(dp),
-                                              format))();
+            if (is_vertex_filter_active())
+            {
+                // vertex indexes must be between the [0, HardNumVertices(g)] range
+                vector_property_map<size_t> index_map(_vertex_index);
+                run_action<>()(*this, boost::bind<void>(generate_index(),
+                                                        _1, index_map))();
+                if (format == "dot")
+                    graphviz_insert_index(dp, index_map);
+
+                if (get_directed())
+                    run_action<detail::always_directed>()
+                        (*this, boost::bind<void>(do_write_to_file(),
+                                                  boost::ref(stream), _1,
+                                                  index_map, boost::ref(dp),
+                                                  format))();
+                else
+                    run_action<detail::never_directed>()
+                        (*this,boost::bind<void>(do_write_to_file_fake_undir(),
+                                                 boost::ref(stream), _1, index_map,
+                                                 boost::ref(dp), format))();
+            }
             else
-                run_action<detail::never_directed>()
-                    (*this,boost::bind<void>(write_to_file_fake_undir(),
-                                             boost::ref(stream), _1,
-                                             _vertex_index, boost::ref(dp),
-                                             format))();
+            {
+                if (format == "dot")
+                    graphviz_insert_index(dp, _vertex_index);
+
+                if (get_directed())
+                    run_action<detail::always_directed>()
+                        (*this, boost::bind<void>(do_write_to_file(),
+                                                  boost::ref(stream), _1,
+                                                  _vertex_index,  boost::ref(dp),
+                                                  format))();
+                else
+                    run_action<detail::never_directed>()
+                        (*this,boost::bind<void>(do_write_to_file_fake_undir(),
+                                                 boost::ref(stream), _1,
+                                                 _vertex_index, boost::ref(dp),
+                                                 format))();
+            }
         }
         stream.reset();
     }
