@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -31,44 +31,42 @@ using namespace graph_tool;
 
 struct do_get_radial
 {
-    template <class Graph, class PosProp, class LevelMap>
-    void operator()(Graph& g, PosProp tpos, LevelMap level, size_t root,
-                    bool weighted, double r) const
+    template <class Graph, class PosProp, class LevelMap, class OrderMap,
+              class WeightMap>
+    void operator()(Graph& g, PosProp tpos, LevelMap level, OrderMap order,
+                    WeightMap weight, size_t root, bool weighted, double r) const
     {
         typedef typename graph_traits<Graph>::vertex_descriptor vertex_t;
-        typedef property_map_type::apply<int, GraphInterface::vertex_index_map_t>::type
-            vcount_t;
-        vcount_t::unchecked_t count(get(vertex_index, g), num_vertices(g));
+        typedef typename vprop_map_t<typename property_traits<WeightMap>::value_type>::type vcount_t;
+        typename vcount_t::unchecked_t count(get(vertex_index, g), num_vertices(g));
 
         if (!weighted)
         {
-            typename graph_traits<Graph>::vertex_iterator v, v_end;
-            for(tie(v, v_end) = vertices(g); v != v_end; ++v)
-                count[*v] = 1;
+            for (auto v : vertices_range(g))
+                count[v] = weight[v];
         }
         else
         {
             deque<vertex_t> q;
-            typename graph_traits<Graph>::vertex_iterator v, v_end;
-            for(tie(v, v_end) = vertices(g); v != v_end; ++v)
-                if (out_degree(*v, g) == 0)
+            for (auto v : vertices_range(g))
+            {
+                if (out_degree(v, g) == 0)
                 {
-                    q.push_back(*v);
-                    count[*v] = 1;
+                    q.push_back(v);
+                    count[v] = weight[v];
                 }
+            }
 
-            typedef property_map_type::apply<uint8_t, GraphInterface::vertex_index_map_t>::type
-                vmark_t;
+            typedef vprop_map_t<uint8_t>::type vmark_t;
             vmark_t::unchecked_t mark(get(vertex_index, g), num_vertices(g));
 
             while (!q.empty())
             {
                 vertex_t v = q.front();
                 q.pop_front();
-                typename graph_traits<Graph>::in_edge_iterator e, e_end;
-                for(tie(e, e_end) = in_edges(v, g); e != e_end; ++e)
+                for (auto e : in_edges_range(v, g))
                 {
-                    vertex_t w = source(*e, g);
+                    vertex_t w = source(e, g);
                     count[w] += count[v];
                     if (!mark[w])
                     {
@@ -79,7 +77,7 @@ struct do_get_radial
             }
         }
 
-        vector<vector<vertex_t> > layers(1);
+        vector<vector<vertex_t>> layers(1);
         layers[0].push_back(root);
 
         bool last = false;
@@ -93,33 +91,39 @@ struct do_get_radial
             for (size_t i = 0; i < last_layer.size(); ++i)
             {
                 vertex_t v = last_layer[i];
-                typename graph_traits<Graph>::out_edge_iterator e, e_end;
-                for(tie(e, e_end) = out_edges(v, g); e != e_end; ++e)
+                for (auto e : out_edges_range(v, g))
                 {
-                    vertex_t w = target(*e, g);
+                    vertex_t w = target(e, g);
                     new_layer.push_back(w);
 
-                    if (layers.size() - 1 == level[w])
+                    if (int(layers.size()) - 1 == int(level[w]))
                         last = false;
                 }
+
+                std::sort(new_layer.end() - out_degree(v, g), new_layer.end(),
+                          [&] (vertex_t u, vertex_t v) -> bool { return order[u] < order[v]; });
+
                 if (out_degree(v, g) == 0)
                     new_layer.push_back(v);
             }
+
             if (last)
                 layers.pop_back();
         }
 
 
-        typedef property_map_type::apply<double, GraphInterface::vertex_index_map_t>::type
-            vangle_t;
+        typedef vprop_map_t<double>::type vangle_t;
         vangle_t::unchecked_t angle(get(vertex_index, g), num_vertices(g));
 
         double d_sum = 0;
         vector<vertex_t>& outer_layer = layers.back();
         for (size_t i = 0; i < outer_layer.size(); ++i)
             d_sum += count[outer_layer[i]];
+        angle[outer_layer[0]] = (2 * M_PI * count[outer_layer[0]]) / d_sum;
+        for (size_t i = 1; i < outer_layer.size(); ++i)
+            angle[outer_layer[i]] = angle[outer_layer[i-1]] + (2 * M_PI * count[outer_layer[i]]) / d_sum;
         for (size_t i = 0; i < outer_layer.size(); ++i)
-            angle[outer_layer[i]] = (i * 2 * M_PI * count[outer_layer[i]]) / d_sum;
+            angle[outer_layer[i]] -= (2 * M_PI * count[outer_layer[i]]) / (2 * d_sum);
 
         for (size_t i = 0; i < layers.size(); ++i)
         {
@@ -128,15 +132,14 @@ struct do_get_radial
             {
                 vertex_t v = vs[j];
                 d_sum = 0;
-                typename graph_traits<Graph>::out_edge_iterator e, e_end;
-                for(tie(e, e_end) = out_edges(v, g); e != e_end; ++e)
+                for (auto e : out_edges_range(v, g))
                 {
-                    vertex_t w = target(*e, g);
+                    vertex_t w = target(e, g);
                     d_sum += count[w];
                 }
-                for(tie(e, e_end) = out_edges(v, g); e != e_end; ++e)
+                for (auto e : out_edges_range(v, g))
                 {
-                    vertex_t w = target(*e, g);
+                    vertex_t w = target(e, g);
                     angle[v] += angle[w] * count[w] / d_sum;
                 }
                 double d = level[v] * r;
@@ -149,12 +152,22 @@ struct do_get_radial
 };
 
 void get_radial(GraphInterface& gi, boost::any otpos, boost::any olevels,
-                size_t root, bool weighted, double r)
+                boost::any oorder, boost::any oweight, size_t root,
+                bool weighted, double r)
 {
+    typedef vprop_map_t<int32_t>::type vmap_t;
+
+    vmap_t levels = boost::any_cast<vmap_t>(olevels);
+
+    typedef vprop_map_t<double>::type wmap_t;
+
+    wmap_t weight = boost::any_cast<wmap_t>(oweight);
+
     run_action<graph_tool::detail::always_directed>()
-        (gi, bind<void>(do_get_radial(), _1, _2, _3, root, weighted, r),
+        (gi, std::bind(do_get_radial(), std::placeholders::_1, std::placeholders::_2,
+                       levels, std::placeholders::_3, weight, root, weighted,  r),
          vertex_scalar_vector_properties(),
-         vertex_scalar_properties())(otpos, olevels);
+         vertex_properties())(otpos, oorder);
 }
 
 #include <boost/python.hpp>
