@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -24,28 +24,26 @@
 
 #include "graph_sfdp.hh"
 #include "random.hh"
+#include "hash_map_wrap.hh"
 
 using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-
 void sfdp_layout(GraphInterface& g, boost::any pos, boost::any vweight,
                  boost::any eweight, boost::any pin, python::object spring_parms,
                  double theta, double init_step, double step_schedule,
                  size_t max_level, double epsilon, size_t max_iter,
-                 bool adaptive, bool verbose)
+                 bool adaptive, bool verbose, rng_t& rng)
 {
-    typedef ConstantPropertyMap<int32_t,GraphInterface::vertex_t> vweight_map_t;
-    typedef ConstantPropertyMap<int32_t,GraphInterface::edge_t> eweight_map_t;
+    typedef UnityPropertyMap<int,GraphInterface::vertex_t> vweight_map_t;
+    typedef UnityPropertyMap<int,GraphInterface::edge_t> eweight_map_t;
     typedef mpl::push_back<vertex_scalar_properties, vweight_map_t>::type
         vertex_props_t;
     typedef mpl::push_back<edge_scalar_properties, eweight_map_t>::type
         edge_props_t;
 
-    typedef property_map_type::apply<int32_t,
-                                     GraphInterface::vertex_index_map_t>::type
-        group_map_t;
+    typedef vprop_map_t<int32_t>::type group_map_t;
 
     double C = python::extract<double>(spring_parms[0]);
     double K = python::extract<double>(spring_parms[1]);
@@ -57,23 +55,23 @@ void sfdp_layout(GraphInterface& g, boost::any pos, boost::any vweight,
         any_cast<group_map_t>(python::extract<any>(spring_parms[6]));
 
     if(vweight.empty())
-        vweight = vweight_map_t(1);
+        vweight = vweight_map_t();
     if(eweight.empty())
-        eweight = eweight_map_t(1);
+        eweight = eweight_map_t();
 
-    typedef property_map_type::apply<uint8_t,
-                                     GraphInterface::vertex_index_map_t>::type
-        pin_map_t;
+    typedef vprop_map_t<uint8_t>::type pin_map_t;
     pin_map_t pin_map = any_cast<pin_map_t>(pin);
 
     run_action<graph_tool::detail::never_directed>()
         (g,
-         bind<void>(get_sfdp_layout(C, K, p, theta, gamma, mu, mu_p, init_step,
-                                    step_schedule, max_level, epsilon,
-                                    max_iter, adaptive),
-                    _1, g.GetVertexIndex(), _2, _3, _4,
-                    pin_map.get_unchecked(num_vertices(g.GetGraph())),
-                    groups.get_unchecked(num_vertices(g.GetGraph())), verbose),
+         std::bind(get_sfdp_layout(C, K, p, theta, gamma, mu, mu_p, init_step,
+                                   step_schedule, max_level, epsilon,
+                                   max_iter, adaptive),
+                   std::placeholders::_1, std::placeholders::_2,
+                   std::placeholders::_3, std::placeholders::_4,
+                   pin_map.get_unchecked(num_vertices(g.get_graph())),
+                   groups.get_unchecked(num_vertices(g.get_graph())), verbose,
+                   std::ref(rng)),
          vertex_floating_vector_properties(), vertex_props_t(), edge_props_t())
         (pos, vweight, eweight);
 }
@@ -82,7 +80,7 @@ struct do_propagate_pos
 {
     template <class Graph, class CoarseGraph, class VertexMap, class PosMap,
               class RNG>
-    void operator()(Graph& g, CoarseGraph* cg, VertexMap& vmap,
+    void operator()(Graph& g, CoarseGraph& cg, VertexMap vmap,
                     boost::any acvmap, PosMap pos, boost::any acpos,
                     double delta, RNG& rng) const
     {
@@ -94,53 +92,25 @@ struct do_propagate_pos
         typedef typename property_traits<PosMap>::value_type pos_t;
         typedef typename pos_t::value_type val_t;
 
-        tr1::variate_generator<RNG&, tr1::uniform_real<val_t> >
-            noise(rng, tr1::uniform_real<val_t>(-delta, delta));
+        uniform_real_distribution<val_t> noise(-delta, delta);
+        gt_hash_map<c_t, pos_t> cmap;
 
-        tr1::unordered_map<c_t, pos_t, boost::hash<c_t> >
-            cmap(num_vertices(*cg));
-        int i, N = num_vertices(*cg);
-        #pragma omp parallel for default(shared) private(i)
-        for (i = 0; i < N; ++i)
-        {
-            typename graph_traits<CoarseGraph>::vertex_descriptor v =
-                vertex(i, *cg);
-            if (v == graph_traits<CoarseGraph>::null_vertex())
-                continue;
+        for (auto v : vertices_range(cg))
             cmap[cvmap[v]] = cpos[v];
-        }
 
-        N = num_vertices(g);
-        #pragma omp parallel for default(shared) private(i)
-        for (i = 0; i < N; ++i)
+        for (auto v : vertices_range(g))
         {
-            typename graph_traits<Graph>::vertex_descriptor v =
-                vertex(i, g);
-            if (v == graph_traits<Graph>::null_vertex())
-                continue;
             pos[v] = cmap[vmap[v]];
+
+            if (delta > 0)
             {
-                if (delta > 0)
-                {
-                    #pragma omp critical
-                    for (size_t j = 0; j < pos[v].size(); ++j)
-                        pos[v][j] += noise();
-                }
+                for (size_t j = 0; j < pos[v].size(); ++j)
+                    pos[v][j] += noise(rng);
             }
         }
     }
 };
 
-struct get_pointers
-{
-    template <class List>
-    struct apply
-    {
-        typedef typename mpl::transform<List,
-                                        mpl::quote1<add_pointer> >::type type;
-    };
-};
-
 void propagate_pos(GraphInterface& gi, GraphInterface& cgi, boost::any vmap,
                    boost::any cvmap, boost::any pos, boost::any cpos,
                    double delta, rng_t& rng)
@@ -150,13 +120,13 @@ void propagate_pos(GraphInterface& gi, GraphInterface& cgi, boost::any vmap,
                              GraphInterface::vertex_index_map_t>::type>::type
         vmaps_t;
 
-
-    run_action<>()
-        (gi, bind<void>(do_propagate_pos(),
-                        _1, _2, _3, cvmap, _4, cpos, delta, ref(rng)),
-         get_pointers::apply<graph_tool::detail::all_graph_views>::type(),
+    gt_dispatch<>()
+        (std::bind(do_propagate_pos(),
+                       std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+                       cvmap, std::placeholders::_4, cpos, delta, std::ref(rng)),
+         all_graph_views(), all_graph_views(),
          vmaps_t(), vertex_floating_vector_properties())
-        (cgi.GetGraphView(), vmap, pos);
+        (gi.get_graph_view(), cgi.get_graph_view(), vmap, pos);
 }
 
 struct do_propagate_pos_mivs
@@ -168,29 +138,20 @@ struct do_propagate_pos_mivs
         typedef typename property_traits<PosMap>::value_type pos_t;
         typedef typename pos_t::value_type val_t;
 
-        tr1::variate_generator<RNG&, tr1::uniform_real<val_t> >
-            noise(rng, tr1::uniform_real<val_t>(-delta, delta));
+        uniform_real_distribution<val_t> noise(-delta, delta);
 
-        int i, N = num_vertices(g);
-        #pragma omp parallel for default(shared) private(i)
-        for (i = 0; i < N; ++i)
+        for (auto v : vertices_range(g))
         {
-            typename graph_traits<Graph>::vertex_descriptor v =
-                vertex(i, g);
-            if (v == graph_traits<Graph>::null_vertex())
-                continue;
             if (mivs[v])
                 continue;
-            pos[v].resize(2);
-            pos[v][0] = pos[v][1] = 0;
             size_t count = 0;
-            typename graph_traits<Graph>::adjacency_iterator a, a_end;
-            for(tie(a, a_end) = adjacent_vertices(v, g); a != a_end; ++a)
+            for (auto a : adjacent_vertices_range(v, g))
             {
-                if (!mivs[*a])
+                if (!mivs[a])
                     continue;
-                for (size_t j = 0; j < pos[v].size(); ++j)
-                    pos[v][j] += pos[*a][j];
+                pos[v].resize(pos[a].size(), 0);
+                for (size_t j = 0; j < pos[a].size(); ++j)
+                    pos[v][j] += pos[a][j];
                 ++count;
             }
 
@@ -202,9 +163,8 @@ struct do_propagate_pos_mivs
             {
                 if (delta > 0)
                 {
-                    #pragma omp critical
                     for (size_t j = 0; j < pos[v].size(); ++j)
-                        pos[v][j] += noise();
+                        pos[v][j] += noise(rng);
                 }
             }
             else
@@ -221,8 +181,9 @@ void propagate_pos_mivs(GraphInterface& gi, boost::any mivs, boost::any pos,
                         double delta, rng_t& rng)
 {
     run_action<>()
-        (gi, bind<void>(do_propagate_pos_mivs(),
-                        _1, _2, _3, delta, ref(rng)),
+        (gi, std::bind(do_propagate_pos_mivs(),
+                       std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+                       delta, std::ref(rng)),
          vertex_scalar_properties(), vertex_floating_vector_properties())
         (mivs, pos);
 }
@@ -242,16 +203,16 @@ struct do_avg_dist
         {
             typename graph_traits<Graph>::vertex_descriptor v =
                 vertex(i, g);
-            if (v == graph_traits<Graph>::null_vertex())
+            if (!is_valid_vertex(v, g))
                 continue;
-            typename graph_traits<Graph>::adjacency_iterator a, a_end;
-            for(tie(a, a_end) = adjacent_vertices(v, g); a != a_end; ++a)
+            for (auto a : adjacent_vertices_range(v, g))
             {
-                d += dist(pos[v], pos[*a]);
+                d += dist(pos[v], pos[a]);
                 count++;
             }
         }
-        d /= count;
+        if (count > 0)
+            d /= count;
         ad = d;
     }
 };
@@ -261,7 +222,8 @@ double avg_dist(GraphInterface& gi, boost::any pos)
 {
     double d;
     run_action<>()
-        (gi, bind<void>(do_avg_dist(), _1, _2, ref(d)),
+        (gi, std::bind(do_avg_dist(), std::placeholders::_1,
+                       std::placeholders::_2, std::ref(d)),
          vertex_scalar_vector_properties()) (pos);
     return d;
 }
@@ -273,12 +235,12 @@ struct do_sanitize_pos
     void operator()(Graph& g, PosMap pos) const
     {
         int i, N = num_vertices(g);
-        #pragma omp parallel for default(shared) private(i) schedule(static) if (N > 100)
+        #pragma omp parallel for default(shared) private(i) schedule(runtime) if (N > 100)
         for (i = 0; i < N; ++i)
         {
             typename graph_traits<Graph>::vertex_descriptor v =
                 vertex(i, g);
-            if (v == graph_traits<Graph>::null_vertex())
+            if (!is_valid_vertex(v, g))
                 continue;
             pos[v].resize(2);
         }
@@ -289,11 +251,10 @@ struct do_sanitize_pos
 void sanitize_pos(GraphInterface& gi, boost::any pos)
 {
     run_action<>()
-        (gi, bind<void>(do_sanitize_pos(), _1, _2),
+        (gi, std::bind(do_sanitize_pos(), std::placeholders::_1, std::placeholders::_2),
          vertex_scalar_vector_properties()) (pos);
 }
 
-
 #include <boost/python.hpp>
 
 void export_sfdp()
