@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -19,5 +19,6 @@
 
 rng_t get_rng(size_t seed)
 {
-    return rng_t(static_cast<rng_t::result_type>(seed));
+    std::seed_seq seq{seed, seed + 1, seed + 2, seed + 3, seed + 4};
+    return rng_t(seq);
 }
