@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -33,58 +33,65 @@ using namespace graph_tool;
 class BFVisitorWrapper
 {
 public:
-    BFVisitorWrapper(python::object gi, python::object vis)
+    BFVisitorWrapper(GraphInterface& gi, python::object vis)
         : _gi(gi), _vis(vis) {}
 
     template <class Edge, class Graph>
-    void examine_edge(Edge e, const Graph& g)
+    void examine_edge(Edge e, Graph& g)
     {
+        auto gp = retrieve_graph_view<Graph>(_gi, g);
         _vis.attr("examine_edge")
-            (PythonEdge<Graph>(_gi, e));
+            (PythonEdge<Graph>(gp, e));
     }
 
     template <class Edge, class Graph>
-    void edge_relaxed(Edge e, const Graph& g)
+    void edge_relaxed(Edge e, Graph& g)
     {
+        auto gp = retrieve_graph_view<Graph>(_gi, g);
         _vis.attr("edge_relaxed")
-            (PythonEdge<Graph>(_gi, e));
+            (PythonEdge<Graph>(gp, e));
     }
 
     template <class Edge, class Graph>
-    void edge_not_relaxed(Edge e, const Graph& g)
+    void edge_not_relaxed(Edge e, Graph& g)
     {
+        auto gp = retrieve_graph_view<Graph>(_gi, g);
         _vis.attr("edge_not_relaxed")
-            (PythonEdge<Graph>(_gi, e));
+            (PythonEdge<Graph>(gp, e));
     }
 
     template <class Edge, class Graph>
-    void edge_minimized(Edge e, const Graph& g)
+    void edge_minimized(Edge e, Graph& g)
     {
+        auto gp = retrieve_graph_view<Graph>(_gi, g);
         _vis.attr("edge_minimized")
-            (PythonEdge<Graph>(_gi, e));
+            (PythonEdge<Graph>(gp, e));
     }
 
     template <class Edge, class Graph>
-    void edge_not_minimized(Edge e, const Graph& g)
+    void edge_not_minimized(Edge e, Graph& g)
     {
+        auto gp = retrieve_graph_view<Graph>(_gi, g);
         _vis.attr("edge_not_minimized")
-            (PythonEdge<Graph>(_gi, e));
+            (PythonEdge<Graph>(gp, e));
     }
 
 private:
-    python::object _gi, _vis;
+    GraphInterface& _gi;
+    python::object _vis;
 };
 
 
 class BFCmp
 {
 public:
+    BFCmp() {}
     BFCmp(python::object cmp): _cmp(cmp) {}
 
     template <class Value1, class Value2>
     bool operator()(const Value1& v1, const Value2& v2) const
     {
-        return extract<bool>(_cmp(v1, v2));
+        return python::extract<bool>(_cmp(v1, v2));
     }
 
 private:
@@ -94,6 +101,7 @@ private:
 class BFCmb
 {
 public:
+    BFCmb() {}
     BFCmb(python::object cmb): _cmb(cmb) {}
 
     template <class Value1, class Value2 >
@@ -109,17 +117,17 @@ private:
 struct do_bf_search
 {
     template <class Graph, class DistanceMap>
-    void operator()(const Graph& g, size_t s, DistanceMap dist,
-                    boost::any pred_map, boost::any aweight,
-                    BFVisitorWrapper vis, pair<BFCmp, BFCmb> cm,
-                    pair<python::object, python::object> range, bool& ret) const
+    void operator()(Graph& g, size_t s, DistanceMap dist, boost::any pred_map,
+                    boost::any aweight, BFVisitorWrapper vis, pair<BFCmp,
+                    BFCmb> cm, pair<python::object, python::object> range,
+                    bool& ret) const
     {
         typedef typename property_traits<DistanceMap>::value_type dtype_t;
         dtype_t z = python::extract<dtype_t>(range.first);
         dtype_t i = python::extract<dtype_t>(range.second);
 
         typedef typename property_map_type::
-            apply<int32_t, typeof(get(vertex_index, g))>::type pred_t;
+            apply<int64_t, decltype(get(vertex_index, g))>::type pred_t;
         pred_t pred = any_cast<pred_t>(pred_map);
         typedef typename graph_traits<Graph>::edge_descriptor edge_t;
         DynamicPropertyMapWrap<dtype_t, edge_t> weight(aweight,
@@ -136,18 +144,19 @@ struct do_bf_search
 };
 
 
-bool bellman_ford_search(GraphInterface& g, python::object gi, size_t source,
-                         boost::any dist_map, boost::any pred_map,
-                         boost::any weight, python::object vis,
-                         python::object cmp, python::object cmb,
-                         python::object zero, python::object inf)
+bool bellman_ford_search(GraphInterface& g, size_t source, boost::any dist_map,
+                         boost::any pred_map, boost::any weight,
+                         python::object vis, python::object cmp,
+                         python::object cmb, python::object zero,
+                         python::object inf)
 {
     bool ret = false;
-    run_action<graph_tool::detail::all_graph_views,mpl::true_>()
-        (g, bind<void>(do_bf_search(), _1, source, _2, pred_map, weight,
-                       BFVisitorWrapper(gi, vis),
-                       make_pair(BFCmp(cmp), BFCmb(cmb)), make_pair(zero, inf),
-                       ref(ret)),
+    run_action<graph_tool::all_graph_views,mpl::true_>()
+        (g, std::bind(do_bf_search(),  std::placeholders::_1, source,
+                      std::placeholders::_2, pred_map, weight,
+                      BFVisitorWrapper(g, vis),
+                      make_pair(BFCmp(cmp), BFCmb(cmb)), make_pair(zero, inf),
+                      std::ref(ret)),
          writable_vertex_properties())
         (dist_map);
     return ret;
