@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -38,7 +38,7 @@ void laplacian(GraphInterface& g, boost::any index, boost::any weight,
     if (!belongs<vertex_scalar_properties>()(index))
         throw ValueException("index vertex property must have a scalar value type");
 
-    typedef ConstantPropertyMap<double, GraphInterface::edge_t> weight_map_t;
+    typedef UnityPropertyMap<double, GraphInterface::edge_t> weight_map_t;
     typedef mpl::push_back<edge_scalar_properties, weight_map_t>::type
         weight_props_t;
 
@@ -46,7 +46,7 @@ void laplacian(GraphInterface& g, boost::any index, boost::any weight,
         throw ValueException("weight edge property must have a scalar value type");
 
     if(weight.empty())
-        weight = weight_map_t(1.0);
+        weight = weight_map_t();
 
     deg_t deg;
     if (sdeg == "in")
@@ -60,8 +60,9 @@ void laplacian(GraphInterface& g, boost::any index, boost::any weight,
     multi_array_ref<int32_t,1> i = get_array<int32_t,1>(oi);
     multi_array_ref<int32_t,1> j = get_array<int32_t,1>(oj);
     run_action<>()
-        (g, bind<void>(get_laplacian(),
-                       _1, _2, _3, deg, ref(data), ref(i), ref(j)),
+        (g, std::bind(get_laplacian(),
+                      std::placeholders::_1,  std::placeholders::_2,  std::placeholders::_3,
+                      deg, std::ref(data), std::ref(i), std::ref(j)),
          vertex_scalar_properties(),
          weight_props_t())(index, weight);
 
