@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -42,6 +42,10 @@ void incidence(GraphInterface& g, boost::any vindex, boost::any eindex,
                python::object odata, python::object oi,
                python::object oj);
 
+void transition(GraphInterface& g, boost::any index, boost::any weight,
+                python::object odata, python::object oi,
+                python::object oj);
+
 BOOST_PYTHON_MODULE(libgraph_tool_spectral)
 {
     using namespace boost::python;
@@ -49,4 +53,5 @@ BOOST_PYTHON_MODULE(libgraph_tool_spectral)
     def("laplacian", &laplacian);
     def("norm_laplacian", &norm_laplacian);
     def("incidence", &incidence);
+    def("transition", &transition);
 }
