@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -36,17 +36,17 @@ python::object distance_histogram(GraphInterface& gi, boost::any weight,
     if (weight.empty())
     {
         run_action<>()(gi,
-                       bind<void>(get_distance_histogram(), _1,
-                                  gi.GetVertexIndex(), no_weightS(),
-                                  ref(bins), ref(ret)))();
+                       std::bind(get_distance_histogram(), std::placeholders::_1,
+                                 gi.get_vertex_index(), no_weightS(),
+                                 std::ref(bins), std::ref(ret)))();
     }
     else
     {
         run_action<>()(gi,
-                       bind<void>(get_distance_histogram(), _1,
-                                  gi.GetVertexIndex(), _2,
-                                  ref(bins), ref(ret)),
-                           edge_scalar_properties())(weight);
+                       std::bind(get_distance_histogram(), std::placeholders::_1,
+                                 gi.get_vertex_index(), std::placeholders::_2,
+                                 std::ref(bins), std::ref(ret)),
+                       edge_scalar_properties())(weight);
     }
     return ret;
 }
