@@ -1,4 +1,6 @@
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// graph-tool -- a general graph modification and manipulation thingy
+//
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -28,23 +30,26 @@ using namespace boost;
 using namespace graph_tool;
 
 void do_label_parallel_edges(GraphInterface& gi, boost::any property,
-                             bool mark_only, bool count_all)
+                             bool mark_only)
 {
-    run_action<>()(gi, bind<void>(label_parallel_edges(), _1, _2, mark_only, count_all),
+    run_action<>()(gi, std::bind(label_parallel_edges(), std::placeholders::_1,
+                                 std::placeholders::_2, mark_only),
                    writable_edge_scalar_properties())(property);
 }
 
 void do_label_self_loops(GraphInterface& gi, boost::any property,
                          bool mark_only)
 {
-    run_action<>()(gi, bind<void>(label_self_loops(), _1, _2, mark_only),
+    run_action<>()(gi, std::bind(label_self_loops(),
+                                 std::placeholders::_1, std::placeholders::_2, mark_only),
                    writable_edge_scalar_properties())(property);
 }
 
 void do_remove_labeled_edges(GraphInterface& gi, boost::any property)
 {
     run_action<graph_tool::detail::always_directed_never_reversed, mpl::true_>()
-        (gi, bind<void>(remove_labeled_edges(), _1, _2),
+        (gi, std::bind(remove_labeled_edges(), std::placeholders::_1,
+                       std::placeholders::_2),
          edge_scalar_properties())(property);
 }
 
