@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -37,11 +37,11 @@ struct get_bipartite
         is_bip = is_bipartite(g, vertex_index, part);
 
         int i, N = num_vertices(g);
-        #pragma omp parallel for default(shared) private(i) schedule(static) if (N > 100)
+        #pragma omp parallel for default(shared) private(i) schedule(runtime) if (N > 100)
         for (i = 0; i < N; ++i)
         {
-            typename graph_traits<Graph>::vertex_descriptor v = vertex(i, g);
-            if (v == graph_traits<Graph>::null_vertex())
+            auto v = vertex(i, g);
+            if (!is_valid_vertex(v, g))
                 continue;
             part_map[v] = (part[v] == color_traits<default_color_type>::white());
         }
@@ -66,8 +66,8 @@ bool is_bipartite(GraphInterface& gi, boost::any part_map)
                            dummy_property_map>::type vertex_map_types;
 
     run_action<graph_tool::detail::never_directed>()
-        (gi, bind<void>(get_bipartite(), _1, gi.GetVertexIndex(),
-                        _2, ref(is_bip)),
+        (gi, std::bind(get_bipartite(), std::placeholders::_1, gi.get_vertex_index(),
+                       std::placeholders::_2, std::ref(is_bip)),
          vertex_map_types())(part_map);
     return is_bip;
 }
