@@ -1,4 +1,6 @@
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// graph-tool -- a general graph modification and manipulation thingy
+//
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -101,11 +103,11 @@ struct do_bfs_search
         typedef unchecked_vector_property_map<size_t, VertexIndexMap> dist_map_t;
         dist_map_t dist_map(vertex_index, num_vertices(g));
         int i, N = num_vertices(g);
-        #pragma omp parallel for default(shared) private(i) schedule(static) if (N > 100)
+        #pragma omp parallel for default(shared) private(i) schedule(runtime) if (N > 100)
         for (i = 0; i < N; ++i)
         {
-            typename graph_traits<Graph>::vertex_descriptor v = vertex(i, g);
-            if (v == graph_traits<Graph>::null_vertex())
+            auto v = vertex(i, g);
+            if (!is_valid_vertex(v, g))
                 continue;
             dist_map[v] = numeric_limits<size_t>::max();
         }
@@ -116,7 +118,7 @@ struct do_bfs_search
         target = source;
         breadth_first_search(g, vertex(source, g),
                              visitor(bfs_diam_visitor<dist_map_t>
-                                     (dist_map, ref(target))).
+                                      (dist_map, std::ref(target))).
                              vertex_index_map(vertex_index).
                              color_map(color_map));
         max_dist = dist_map[vertex(target, g)];
@@ -138,7 +140,7 @@ struct do_djk_search
                                 distance_map(dist_map).
                                 vertex_index_map(vertex_index).
                                 visitor(djk_diam_visitor<dist_map_t>
-                                        (dist_map, ref(target))));
+                                        (dist_map, std::ref(target))));
         max_dist = dist_map[vertex(target, g)];
     }
 };
@@ -150,14 +152,14 @@ python::object get_diam(GraphInterface& gi, size_t source, boost::any weight)
     if (weight.empty())
     {
         run_action<>()
-            (gi, bind<void>(do_bfs_search(), _1, source, gi.GetVertexIndex(),
-                            ref(target), ref(max_dist)))();
+            (gi, std::bind(do_bfs_search(), std::placeholders::_1, source, gi.get_vertex_index(),
+                           std::ref(target), std::ref(max_dist)))();
     }
     else
     {
         run_action<>()
-            (gi, bind<void>(do_djk_search(), _1, source, gi.GetVertexIndex(),
-                            _2, ref(target), ref(max_dist)),
+            (gi, std::bind(do_djk_search(), std::placeholders::_1, source, gi.get_vertex_index(),
+                           std::placeholders::_2, std::ref(target), std::ref(max_dist)),
              edge_scalar_properties())(weight);
 
     }
