@@ -1,4 +1,6 @@
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// graph-tool -- a general graph modification and manipulation thingy
+//
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -29,10 +31,14 @@ using namespace graph_tool;
 void do_kcore_decomposition(GraphInterface& gi, boost::any prop,
                             GraphInterface::deg_t deg)
 {
-    run_action<>()(gi, bind<void>(kcore_decomposition(), _1,
-                                  gi.GetVertexIndex(), _2, _3),
-                   writable_vertex_scalar_properties(),
-                   degree_selectors())(prop, degree_selector(deg));
+    gt_dispatch<>()
+        ([](auto& g, auto core, auto d)
+         {
+             kcore_decomposition(g, core, d);
+         },
+         all_graph_views(), writable_vertex_scalar_properties(),
+         degree_selectors())(gi.get_graph_view(), prop,
+                             degree_selector(deg));
 }
 
 void export_kcore()
