@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -69,6 +69,6 @@ struct do_maximal_planar
 void maximal_planar(GraphInterface& gi)
 {
     run_action<graph_tool::detail::never_directed, mpl::true_>()
-        (gi, bind<void>(do_maximal_planar(), _1, gi.GetVertexIndex(),
-                        gi.GetEdgeIndex()))();
+        (gi, std::bind(do_maximal_planar(), std::placeholders::_1, gi.get_vertex_index(),
+                       gi.get_edge_index()))();
 }
