@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -40,7 +40,8 @@ bool topological_sort(GraphInterface& gi, vector<int32_t>& sort)
     try
     {
         run_action<>()
-            (gi, bind<void>(get_topological_sort(), _1, ref(sort)))();
+            (gi, std::bind(get_topological_sort(), std::placeholders::_1,
+                           std::ref(sort)))();
         return true;
     }
     catch (not_a_dag& e)
