@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -40,6 +40,6 @@ struct get_transitive_closure
 void transitive_closure(GraphInterface& gi, GraphInterface& tcgi)
 {
     run_action<graph_tool::detail::always_directed>()
-        (gi, bind<void>(get_transitive_closure(), _1,
-                        ref(tcgi.GetGraph())))();
+        (gi, std::bind(get_transitive_closure(), std::placeholders::_1,
+                       std::ref(tcgi.get_graph())))();
 }
