@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@ Utility module which includes all the sub-modules in graph_tool
 """
 
 from __future__ import division, absolute_import, print_function
+import warnings
 
 from graph_tool import *
 import graph_tool
@@ -33,8 +34,10 @@ import graph_tool.centrality
 try:
     from graph_tool.draw import *
     import graph_tool.draw
-except ImportError:
+except ImportError as e:
     # Proceed despite errors with cairo, matplotlib, etc.
+    msg = "Error importing draw module, proceeding nevertheless: " + str(e)
+    warnings.warn(msg, RuntimeWarning)
     pass
 from graph_tool.stats import *
 import graph_tool.stats
@@ -44,10 +47,8 @@ from graph_tool.stats import *
 import graph_tool.stats
 from graph_tool.clustering import *
 import graph_tool.clustering
-from graph_tool.community import *
-import graph_tool.community
-from graph_tool.run_action import *
-import graph_tool.run_action
+from graph_tool.inference import *
+import graph_tool.inference
 from graph_tool.topology import *
 import graph_tool.topology
 from graph_tool.flow import *
