@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@ from .. import _prop, ungroup_vector_property
 from .. topology import shortest_distance
 import sys
 import numpy
+import numpy.linalg
 
 __all__ = ["pagerank", "betweenness", "central_point_dominance", "closeness",
            "eigentrust", "eigenvector", "katz", "hits", "trust_transitivity"]
@@ -137,6 +138,10 @@ def pagerank(g, damping=0.85, pers=None, weight=None, prop=None, epsilon=1e-6,
     Examples
     --------
 
+    .. testsetup:: pagerank
+
+       import matplotlib
+
     .. doctest:: pagerank
 
        >>> g = gt.collection.data["polblogs"]
@@ -144,7 +149,8 @@ def pagerank(g, damping=0.85, pers=None, weight=None, prop=None, epsilon=1e-6,
        >>> pr = gt.pagerank(g)
        >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=pr,
        ...               vertex_size=gt.prop_to_size(pr, mi=5, ma=15),
-       ...               vorder=pr, output="polblogs_pr.pdf")
+       ...               vorder=pr, vcmap=matplotlib.cm.gist_heat,
+       ...               output="polblogs_pr.pdf")
        <...>
 
     .. testcode:: pagerank
@@ -152,7 +158,8 @@ def pagerank(g, damping=0.85, pers=None, weight=None, prop=None, epsilon=1e-6,
 
        gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=pr,
                      vertex_size=gt.prop_to_size(pr, mi=5, ma=15),
-                     vorder=pr, output="polblogs_pr.png")
+                     vorder=pr, vcmap=matplotlib.cm.gist_heat,
+                     output="polblogs_pr.png")
 
 
     .. figure:: polblogs_pr.*
@@ -171,7 +178,8 @@ def pagerank(g, damping=0.85, pers=None, weight=None, prop=None, epsilon=1e-6,
        >>> pr = gt.pagerank(g, pers=p)
        >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=pr,
        ...               vertex_size=gt.prop_to_size(pr, mi=5, ma=15),
-       ...               vorder=pr, output="polblogs_pr_pers.pdf")
+       ...               vorder=pr, vcmap=matplotlib.cm.gist_heat,
+       ...               output="polblogs_pr_pers.pdf")
        <...>
 
     .. testcode:: pagerank
@@ -179,6 +187,7 @@ def pagerank(g, damping=0.85, pers=None, weight=None, prop=None, epsilon=1e-6,
 
        gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=pr,
                      vertex_size=gt.prop_to_size(pr, mi=5, ma=15),
+                     vcmap=matplotlib.cm.gist_heat,
                      vorder=pr, output="polblogs_pr_pers.png")
 
 
@@ -207,8 +216,8 @@ def pagerank(g, damping=0.85, pers=None, weight=None, prop=None, epsilon=1e-6,
         max_iter = 0
     if prop == None:
         prop = g.new_vertex_property("double")
-        N = len(prop.a)
-        prop.a = pers.a[:N] if pers is not None else 1. / g.num_vertices()
+        N = len(prop.fa)
+        prop.fa = pers.fa[:N] if pers is not None else 1. / g.num_vertices()
     ic = libgraph_tool_centrality.\
             get_pagerank(g._Graph__graph, _prop("v", g, prop),
                          _prop("v", g, pers), _prop("e", g, weight),
@@ -274,6 +283,10 @@ def betweenness(g, vprop=None, eprop=None, weight=None, norm=True):
     Examples
     --------
 
+    .. testsetup:: betweenness
+
+       import matplotlib
+
     .. doctest:: betweenness
 
        >>> g = gt.collection.data["polblogs"]
@@ -282,6 +295,7 @@ def betweenness(g, vprop=None, eprop=None, weight=None, norm=True):
        >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=vp,
        ...               vertex_size=gt.prop_to_size(vp, mi=5, ma=15),
        ...               edge_pen_width=gt.prop_to_size(ep, mi=0.5, ma=5),
+       ...               vcmap=matplotlib.cm.gist_heat,
        ...               vorder=vp, output="polblogs_betweenness.pdf")
        <...>
 
@@ -291,6 +305,7 @@ def betweenness(g, vprop=None, eprop=None, weight=None, norm=True):
        gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=vp,
                      vertex_size=gt.prop_to_size(vp, mi=5, ma=15),
                      edge_pen_width=gt.prop_to_size(ep, mi=0.5, ma=5),
+                     vcmap=matplotlib.cm.gist_heat,
                      vorder=vp, output="polblogs_betweenness.png")
 
 
@@ -379,11 +394,11 @@ def closeness(g, weight=None, source=None, vprop=None, norm=True, harmonic=False
     If ``norm == True``, the values of :math:`c_i` are normalized by
     :math:`n_i-1` where :math:`n_i` is the size of the (out-) component of
     :math:`i`. If ``harmonic == True``, they are instead simply normalized by
-    :math:`N-1`.
+    :math:`V-1`.
 
-    The algorithm complexity of :math:`O(N(N + E))` for unweighted graphs and
-    :math:`O(N(N+E) \log N)` for weighted graphs. If the option ``source`` is
-    specified, this drops to :math:`O(N + E)` and :math:`O((N+E)\log N)`
+    The algorithm complexity of :math:`O(V(V + E))` for unweighted graphs and
+    :math:`O(V(v+E) \log V)` for weighted graphs. If the option ``source`` is
+    specified, this drops to :math:`O(V + E)` and :math:`O((V+E)\log V)`
     respectively.
 
     If enabled during compilation, this algorithm runs in parallel.
@@ -391,6 +406,10 @@ def closeness(g, weight=None, source=None, vprop=None, norm=True, harmonic=False
     Examples
     --------
 
+    .. testsetup:: closeness
+
+       import matplotlib
+
     .. doctest:: closeness
 
        >>> g = gt.collection.data["polblogs"]
@@ -398,6 +417,7 @@ def closeness(g, weight=None, source=None, vprop=None, norm=True, harmonic=False
        >>> c = gt.closeness(g)
        >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=c,
        ...               vertex_size=gt.prop_to_size(c, mi=5, ma=15),
+       ...               vcmap=matplotlib.cm.gist_heat,
        ...               vorder=c, output="polblogs_closeness.pdf")
        <...>
 
@@ -406,6 +426,7 @@ def closeness(g, weight=None, source=None, vprop=None, norm=True, harmonic=False
 
        gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=c,
                      vertex_size=gt.prop_to_size(c, mi=5, ma=15),
+                     vcmap=matplotlib.cm.gist_heat,
                      vorder=c, output="polblogs_closeness.png")
 
 
@@ -492,7 +513,7 @@ def central_point_dominance(g, betweenness):
     >>> g = gt.GraphView(g, vfilt=gt.label_largest_component(g))
     >>> vp, ep = gt.betweenness(g)
     >>> print(gt.central_point_dominance(g, vp))
-    0.11610685614353008
+    0.11610685614...
 
     References
     ----------
@@ -565,9 +586,11 @@ def eigenvector(g, weight=None, vprop=None, epsilon=1e-6, max_iter=None):
 
     Examples
     --------
+
     .. testsetup:: eigenvector
 
        np.random.seed(42)
+       import matplotlib
 
     .. doctest:: eigenvector
 
@@ -577,9 +600,10 @@ def eigenvector(g, weight=None, vprop=None, epsilon=1e-6, max_iter=None):
        >>> w.a = np.random.random(len(w.a)) * 42
        >>> ee, x = gt.eigenvector(g, w)
        >>> print(ee)
-       0.0013713102236792602
+       724.302745922...
        >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=x,
        ...               vertex_size=gt.prop_to_size(x, mi=5, ma=15),
+       ...               vcmap=matplotlib.cm.gist_heat,
        ...               vorder=x, output="polblogs_eigenvector.pdf")
        <...>
 
@@ -588,6 +612,7 @@ def eigenvector(g, weight=None, vprop=None, epsilon=1e-6, max_iter=None):
 
        gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=x,
                      vertex_size=gt.prop_to_size(x, mi=5, ma=15),
+                     vcmap=matplotlib.cm.gist_heat,
                      vorder=x, output="polblogs_eigenvector.png")
 
 
@@ -613,7 +638,7 @@ def eigenvector(g, weight=None, vprop=None, epsilon=1e-6, max_iter=None):
 
     if vprop is None:
         vprop = g.new_vertex_property("double")
-        vprop.a = 1. / g.num_vertices()
+        vprop.fa = 1. / g.num_vertices()
     if max_iter is None:
         max_iter = 0
     ee = libgraph_tool_centrality.\
@@ -622,7 +647,8 @@ def eigenvector(g, weight=None, vprop=None, epsilon=1e-6, max_iter=None):
     return ee, vprop
 
 
-def katz(g, alpha=0.01, beta=None, weight=None, vprop=None, epsilon=1e-6, max_iter=None):
+def katz(g, alpha=0.01, beta=None, weight=None, vprop=None, epsilon=1e-6,
+         max_iter=None, norm=True):
     r"""
     Calculate the Katz centrality of each vertex in the graph.
 
@@ -646,6 +672,8 @@ def katz(g, alpha=0.01, beta=None, weight=None, vprop=None, epsilon=1e-6, max_it
         vertices are below this value.
     max_iter : int, optional (default: ``None``)
         If supplied, this will limit the total number of iterations.
+    norm : bool, optional (default: ``True``)
+        Whether or not the centrality values should be normalized.
 
     Returns
     -------
@@ -685,16 +713,18 @@ def katz(g, alpha=0.01, beta=None, weight=None, vprop=None, epsilon=1e-6, max_it
     .. testsetup:: katz
 
        np.random.seed(42)
+       import matplotlib
 
     .. doctest:: katz
 
        >>> g = gt.collection.data["polblogs"]
        >>> g = gt.GraphView(g, vfilt=gt.label_largest_component(g))
        >>> w = g.new_edge_property("double")
-       >>> w.a = np.random.random(len(w.a)) * 42
+       >>> w.a = np.random.random(len(w.a))
        >>> x = gt.katz(g, weight=w)
        >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=x,
        ...               vertex_size=gt.prop_to_size(x, mi=5, ma=15),
+       ...               vcmap=matplotlib.cm.gist_heat,
        ...               vorder=x, output="polblogs_katz.pdf")
        <...>
 
@@ -703,6 +733,7 @@ def katz(g, alpha=0.01, beta=None, weight=None, vprop=None, epsilon=1e-6, max_it
 
        gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=x,
                      vertex_size=gt.prop_to_size(x, mi=5, ma=15),
+                     vcmap=matplotlib.cm.gist_heat,
                      vorder=x, output="polblogs_katz.png")
 
 
@@ -723,15 +754,15 @@ def katz(g, alpha=0.01, beta=None, weight=None, vprop=None, epsilon=1e-6, max_it
        Weblogging Ecosystem (2005). :DOI:`10.1145/1134271.1134277`
     """
 
-    if vprop == None:
+    if vprop is None:
         vprop = g.new_vertex_property("double")
-        N = len(vprop.a)
-        vprop.a = beta.a[:N] if beta is not None else 1.
     if max_iter is None:
         max_iter = 0
-    ee = libgraph_tool_centrality.\
+    libgraph_tool_centrality.\
          get_katz(g._Graph__graph, _prop("e", g, weight), _prop("v", g, vprop),
-         _prop("v", beta, vprop), float(alpha), epsilon, max_iter)
+                  _prop("v", g, beta), float(alpha), epsilon, max_iter)
+    if norm:
+        vprop.fa = vprop.fa / numpy.linalg.norm(vprop.fa)
     return vprop
 
 
@@ -802,6 +833,10 @@ def hits(g, weight=None, xprop=None, yprop=None, epsilon=1e-6, max_iter=None):
     Examples
     --------
 
+    .. testsetup:: hits
+
+       import matplotlib
+
     .. doctest:: hits
 
        >>> g = gt.collection.data["polblogs"]
@@ -809,10 +844,12 @@ def hits(g, weight=None, xprop=None, yprop=None, epsilon=1e-6, max_iter=None):
        >>> ee, x, y = gt.hits(g)
        >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=x,
        ...               vertex_size=gt.prop_to_size(x, mi=5, ma=15),
+       ...               vcmap=matplotlib.cm.gist_heat,
        ...               vorder=x, output="polblogs_hits_auths.pdf")
        <...>
        >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=y,
        ...               vertex_size=gt.prop_to_size(y, mi=5, ma=15),
+       ...               vcmap=matplotlib.cm.gist_heat,
        ...               vorder=y, output="polblogs_hits_hubs.pdf")
        <...>
 
@@ -821,20 +858,22 @@ def hits(g, weight=None, xprop=None, yprop=None, epsilon=1e-6, max_iter=None):
 
        gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=x,
                      vertex_size=gt.prop_to_size(x, mi=5, ma=15),
+                     vcmap=matplotlib.cm.gist_heat,
                      vorder=x, output="polblogs_hits_auths.png")
        gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=y,
                      vertex_size=gt.prop_to_size(y, mi=5, ma=15),
+                     vcmap=matplotlib.cm.gist_heat,
                      vorder=y, output="polblogs_hits_hubs.png")
 
 
     .. figure:: polblogs_hits_auths.*
-       :align: left
+       :align: center
 
        HITS authority values of the a political blogs network of
        [adamic-polblogs]_.
 
     .. figure:: polblogs_hits_hubs.*
-       :align: right
+       :align: center
 
        HITS hub values of the a political blogs network of [adamic-polblogs]_.
 
@@ -924,6 +963,7 @@ def eigentrust(g, trust_map, vprop=None, norm=False, epsilon=1e-6, max_iter=0,
     .. testsetup:: eigentrust
 
        np.random.seed(42)
+       import matplotlib
 
     .. doctest:: eigentrust
 
@@ -934,6 +974,7 @@ def eigentrust(g, trust_map, vprop=None, norm=False, epsilon=1e-6, max_iter=0,
        >>> t = gt.eigentrust(g, w)
        >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=t,
        ...               vertex_size=gt.prop_to_size(t, mi=5, ma=15),
+       ...               vcmap=matplotlib.cm.gist_heat,
        ...               vorder=t, output="polblogs_eigentrust.pdf")
        <...>
 
@@ -942,6 +983,7 @@ def eigentrust(g, trust_map, vprop=None, norm=False, epsilon=1e-6, max_iter=0,
 
        gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=t,
                      vertex_size=gt.prop_to_size(t, mi=5, ma=15),
+                     vcmap=matplotlib.cm.gist_heat,
                      vorder=t, output="polblogs_eigentrust.png")
 
 
@@ -1037,11 +1079,11 @@ def trust_transitivity(g, trust_map, source=None, target=None, vprop=None):
     maximum weight, using Dijkstra's algorithm, to all in-neighbours of a given
     target. This search needs to be performed repeatedly for every target, since
     it needs to be removed from the graph first. For each given source, the
-    resulting complexity is therefore :math:`O(N^2\log N)` for all targets, and
-    :math:`O(N\log N)` for a single target. For a given target, the complexity
-    for obtaining the trust from all given sources is :math:`O(kN\log N)`, where
+    resulting complexity is therefore :math:`O(V^2\log V)` for all targets, and
+    :math:`O(V\log V)` for a single target. For a given target, the complexity
+    for obtaining the trust from all given sources is :math:`O(kV\log V)`, where
     :math:`k` is the in-degree of the target. Thus, the complexity for obtaining
-    the complete trust matrix is :math:`O(EN\log N)`, where :math:`E` is the
+    the complete trust matrix is :math:`O(EV\log V)`, where :math:`E` is the
     number of edges in the network.
 
     If enabled during compilation, this algorithm runs in parallel.
@@ -1051,6 +1093,7 @@ def trust_transitivity(g, trust_map, source=None, target=None, vprop=None):
     .. testsetup:: trust_transitivity
 
        np.random.seed(42)
+       import matplotlib
 
     .. doctest:: trust_transitivity
 
@@ -1064,6 +1107,7 @@ def trust_transitivity(g, trust_map, source=None, target=None, vprop=None):
        >>> t = gt.trust_transitivity(g, w, source=g.vertex(42))
        >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=t,
        ...               vertex_size=gt.prop_to_size(t, mi=5, ma=15),
+       ...               vcmap=matplotlib.cm.gist_heat,
        ...               vorder=t, output="polblogs_trust_transitivity.pdf")
        <...>
 
@@ -1072,6 +1116,7 @@ def trust_transitivity(g, trust_map, source=None, target=None, vprop=None):
 
        gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=t,
                      vertex_size=gt.prop_to_size(t, mi=5, ma=15),
+                     vcmap=matplotlib.cm.gist_heat,
                      vorder=t, output="polblogs_trust_transitivity.png")
 
 
