@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -105,7 +105,7 @@ def local_clustering(g, prop=None, undirected=True):
     .. math::
        c'_i = 2c_i.
 
-    The implemented algorithm runs in :math:`O(|V|\left< k\right>^3)` time,
+    The implemented algorithm runs in :math:`O(|V|\left< k\right>^2)` time,
     where :math:`\left< k\right>` is the average out-degree.
 
     If enabled during compilation, this algorithm runs in parallel.
@@ -121,7 +121,7 @@ def local_clustering(g, prop=None, undirected=True):
     >>> g = gt.random_graph(1000, lambda: (5,5))
     >>> clust = gt.local_clustering(g)
     >>> print(gt.vertex_average(g, clust))
-    (0.0072, 0.00039746045691969764)
+    (0.006177777777777778, 0.00036966184079941211)
 
     References
     ----------
@@ -134,7 +134,7 @@ def local_clustering(g, prop=None, undirected=True):
         prop = g.new_vertex_property("double")
     if g.is_directed() and undirected:
         g = GraphView(g, directed=False)
-    _gt.extended_clustering(g._Graph__graph, [_prop("v", g, prop)])
+    _gt.local_clustering(g._Graph__graph, _prop("v", g, prop))
     return prop
 
 
@@ -167,7 +167,7 @@ def global_clustering(g):
        c = 3 \times \frac{\text{number of triangles}}
                           {\text{number of connected triples}}
 
-    The implemented algorithm runs in :math:`O(|V|\left< k\right>^3)` time,
+    The implemented algorithm runs in :math:`O(|V|\left< k\right>^2)` time,
     where :math:`\left< k\right>` is the average (total) degree.
 
     If enabled during compilation, this algorithm runs in parallel.
@@ -182,7 +182,7 @@ def global_clustering(g):
 
     >>> g = gt.random_graph(1000, lambda: (5,5))
     >>> print(gt.global_clustering(g))
-    (0.007182172103638073, 0.0003970213508956326)
+    (0.006177777777777778, 0.0003700318726720...)
 
     References
     ----------
@@ -261,11 +261,11 @@ def extended_clustering(g, props=None, max_depth=3, undirected=False):
     >>> for i in range(0, 5):
     ...    print(gt.vertex_average(g, clusts[i]))
     ...
-    (0.005535, 0.00046266726404860573)
-    (0.02351, 0.0009395843702876762)
-    (0.11651833333333333, 0.0019837472110181336)
-    (0.391765, 0.0030221072114043567)
-    (0.4439983333333333, 0.0030393922085216675)
+    (0.00421, 0.00041685103920811...)
+    (0.027226666666666666, 0.00100735228307788...)
+    (0.11549166666666667, 0.00208139908541773...)
+    (0.4128066666666666, 0.00294792212319872...)
+    (0.4205716666666667, 0.00315646574114149...)
 
     References
     ----------
@@ -286,9 +286,9 @@ def extended_clustering(g, props=None, max_depth=3, undirected=False):
 
 def motifs(g, k, p=1.0, motif_list=None, return_maps=False):
     r"""
-    Count the occurrence of k-size subgraphs (motifs). A tuple with two lists is
-    returned: the list of motifs found, and the list with their respective
-    counts.
+    Count the occurrence of k-size node-induced subgraphs (motifs). A tuple with
+    two lists is returned: the list of motifs found, and the list with their
+    respective counts.
 
     Parameters
     ----------
@@ -347,9 +347,9 @@ def motifs(g, k, p=1.0, motif_list=None, return_maps=False):
     >>> g = gt.random_graph(1000, lambda: (5,5))
     >>> motifs, counts = gt.motifs(gt.GraphView(g, directed=False), 4)
     >>> print(len(motifs))
-    63
+    13
     >>> print(counts)
-    [28448, 28343, 28929, 29600, 159144, 98111, 98187, 33446, 284, 176, 70, 139, 210, 37, 526, 280, 85, 156, 553, 620, 469, 585, 443, 380, 203, 362, 210, 313, 241, 100, 295, 64, 253, 278, 250, 7, 5, 3, 4, 4, 3, 2, 6, 1, 7, 2, 1, 2, 3, 1, 1, 3, 2, 3, 3, 1, 1, 2, 2, 1, 1, 1, 1]
+    [115408, 388542, 1031, 1182, 2662, 2138, 833, 28, 16, 5, 5, 3, 4]
 
 
     References
@@ -358,6 +358,7 @@ def motifs(g, k, p=1.0, motif_list=None, return_maps=False):
        motifs", IEEE/ACM Transactions on Computational Biology and
        Bioinformatics (TCBB), Volume 3, Issue 4, Pages 347-359, 2006.
        :doi:`10.1109/TCBB.2006.51`
+    .. [induced-subgraph-isomorphism] http://en.wikipedia.org/wiki/Induced_subgraph_isomorphism_problem
     """
 
     sub_list = []
@@ -397,7 +398,10 @@ def motifs(g, k, p=1.0, motif_list=None, return_maps=False):
         temp.append(mg)
     sub_list = temp
 
-    list_hist = list(zip(sub_list, hist))
+    if return_maps:
+        list_hist = list(zip(sub_list, hist, vertex_maps))
+    else:
+        list_hist = list(zip(sub_list, hist))
     # sort according to in-degree sequence
     list_hist.sort(key=lambda x: sorted([v.in_degree() for v in x[0].vertices()]))
 
@@ -411,6 +415,7 @@ def motifs(g, k, p=1.0, motif_list=None, return_maps=False):
     hist = [x[1] for x in list_hist]
 
     if return_maps:
+        vertex_maps = [x[2] for x in list_hist]
         for i, vlist in enumerate(vertex_maps):
             sub = sub_list[i]
             vertex_maps[i] = [PropertyMap(vm, sub, "v") for vm in vlist]
@@ -516,8 +521,7 @@ def motif_significance(g, k, n_shuffles=100, p=1.0, motif_list=None,
     >>> print(len(motifs))
     11
     >>> print(zscores)
-    [1.0170744045563587, 0.90350945788797254, 0.93486676613275743, -0.84952469609377301, -0.95933706030135635, -0.68770434820213655, -0.24381350447066388, 0.88, -0.12, -0.35, -0.21]
-
+    [0.04162670293683441, 0.046769781223679897, 0.55888261369689918, 0.82906624302416043, -0.41384527710386287, -0.42070845824900477, -1.3262411347517733, 1.82, -0.13, -0.37, -0.22]
     """
 
     s_ms, counts = motifs(g, k, p, motif_list)
