@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@ Graph drawing
    :nosignatures:
 
    graph_draw
+   draw_hierarchy
    graphviz_draw
    prop_to_size
 
@@ -70,9 +71,8 @@ from .. import GraphView, _check_prop_vector, group_vector_property, \
      ungroup_vector_property, infect_vertex_property, _prop, _get_rng
 from .. topology import max_cardinality_matching, max_independent_vertex_set, \
     label_components, pseudo_diameter, shortest_distance
-from .. community import condensation_graph
 from .. stats import label_parallel_edges
-from .. generation import predecessor_tree
+from .. generation import predecessor_tree, condensation_graph
 import numpy.random
 from numpy import sqrt
 import sys
@@ -81,11 +81,10 @@ from .. dl_import import dl_import
 dl_import("from . import libgraph_tool_layout")
 
 
-__all__ = ["graph_draw", "graphviz_draw",
-           "fruchterman_reingold_layout",
-           "arf_layout", "sfdp_layout", "random_layout",
-           "radial_tree_layout",
-           "cairo_draw", "prop_to_size", "get_hierarchy_control_points"]
+__all__ = ["graph_draw", "graphviz_draw", "fruchterman_reingold_layout",
+           "arf_layout", "sfdp_layout", "random_layout", "radial_tree_layout",
+           "cairo_draw", "prop_to_size", "get_hierarchy_control_points",
+           "default_cm"]
 
 
 def random_layout(g, shape=None, pos=None, dim=2):
@@ -150,8 +149,8 @@ def random_layout(g, shape=None, pos=None, dim=2):
         d = r[1] - r[0]
 
         # deal with filtering
-        p = pos[i].ma
-        p[:] = numpy.random.random(len(p)) * d + r[0]
+        p = pos[i].fa
+        pos[i].fa = numpy.random.random(len(p)) * d + r[0]
 
     pos = group_vector_property(pos)
     return pos
@@ -375,6 +374,7 @@ def _propagate_pos(g, cg, c, cc, cpos, delta, mivs):
                                        _prop("v", cg, cpos),
                                        delta if mivs is None else 0,
                                        _get_rng())
+
     if mivs is not None:
         g = g.base
         u = GraphView(g, directed=False)
@@ -391,7 +391,7 @@ def _propagate_pos(g, cg, c, cc, cpos, delta, mivs):
 def _avg_edge_distance(g, pos):
     libgraph_tool_layout.sanitize_pos(g._Graph__graph, _prop("v", g, pos))
     ad = libgraph_tool_layout.avg_dist(g._Graph__graph, _prop("v", g, pos))
-    if numpy.isnan(ad):
+    if numpy.isnan(ad) or ad == 0:
         ad = 1.
     return ad
 
@@ -506,9 +506,9 @@ def coarse_graph_stack(g, c, coarse_stack, eweight=None, vweight=None,
 
 
 def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
-                K=None, p=2., theta=0.6, max_level=11, gamma=1., mu=0., mu_p=1.,
-                init_step=None, cooling_step=0.9, adaptive_cooling=True,
-                epsilon=1e-1, max_iter=0, pos=None, multilevel=None,
+                K=None, p=2., theta=0.6, max_level=15, gamma=1., mu=0., mu_p=1.,
+                init_step=None, cooling_step=0.95, adaptive_cooling=True,
+                epsilon=1e-2, max_iter=0, pos=None, multilevel=None,
                 coarse_method="hybrid", mivs_thres=0.9, ec_thres=0.75,
                 coarse_stack=None, weighted_coarse=False, verbose=False):
     r"""Obtain the SFDP spring-block layout of the graph.
@@ -536,7 +536,7 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
         Repulsive force exponent.
     theta : float (optional, default: ``0.6``)
         Quadtree opening parameter, a.k.a. Barnes-Hut opening criterion.
-    max_level : int (optional, default: ``11``)
+    max_level : int (optional, default: ``15``)
         Maximum quadtree level.
     gamma : float (optional, default: ``1.0``)
         Strength of the attractive force between connected components, or group
@@ -549,11 +549,11 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
         connected component, or group assignment.
     init_step : float (optional, default: ``None``)
         Initial update step. If not provided, it will be chosen automatically.
-    cooling_step : float (optional, default: ``0.9``)
+    cooling_step : float (optional, default: ``0.95``)
         Cooling update step.
     adaptive_cooling : bool (optional, default: ``True``)
         Use an adaptive cooling scheme.
-    epsilon : float (optional, default: ``0.1``)
+    epsilon : float (optional, default: ``0.01``)
         Relative convergence criterion.
     max_iter : int (optional, default: ``0``)
         Maximum number of iterations. If this value is ``0``, it runs until
@@ -634,7 +634,7 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
         K = _avg_edge_distance(g, pos)
 
     if init_step is None:
-        init_step = 10 * max(_avg_edge_distance(g, pos), K)
+        init_step = 2 * max(_avg_edge_distance(g, pos), K)
 
     if multilevel is None:
         multilevel = g.num_vertices() > 1000
@@ -671,8 +671,8 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
                               max_iter=max_iter,
                               cooling_step=cooling_step,
                               adaptive_cooling=False,
-                              init_step=max(2 * K,
-                                            _avg_edge_distance(u, pos) / 10),
+                              # init_step=max(2 * K,
+                              #               _avg_edge_distance(u, pos)),
                               multilevel=False,
                               verbose=False)
             #graph_draw(u, pos)
@@ -699,10 +699,11 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
                                      (C, K, p, gamma, mu, mu_p, _prop("v", g, groups)),
                                      theta, init_step, cooling_step, max_level,
                                      epsilon, max_iter, not adaptive_cooling,
-                                     verbose)
+                                     verbose, _get_rng())
     return pos
 
-def radial_tree_layout(g, root, weighted=False, r=1.):
+def radial_tree_layout(g, root, rel_order=None, weighted=False,
+                       node_weight=None, r=1.):
     r"""Computes a radial layout of the graph according to the minimum spanning
     tree centered at the ``root`` vertex.
 
@@ -712,9 +713,14 @@ def radial_tree_layout(g, root, weighted=False, r=1.):
         Graph to be used.
     root : :class:`~graph_tool.Vertex` or ``int``
         The root of the radial tree.
+    rel_order : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
+        Relative order of the nodes at each respective branch.
     weighted : ``bool`` (optional, default: ``False``)
         If true, the angle between the child branches will be computed according
         to weight of the entire sub-branches.
+    node_weight : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
+        If given, the relative spacing between leafs will correspond to the node
+        weights.
     r : ``float`` (optional, default: ``1.``)
         Layer spacing.
 
@@ -758,30 +764,20 @@ def radial_tree_layout(g, root, weighted=False, r=1.):
     t = predecessor_tree(g, pred_map)
     pos = t.new_vertex_property("vector<double>")
     levels = t.own_property(levels)
-
+    if rel_order is None:
+        rel_order = g.vertex_index.copy("int")
+    if node_weight is None:
+        node_weight = g.new_vertex_property("double", 1)
+    elif node_weight.value_type() != "double":
+        node_weight = node_weight.copy("double")
     libgraph_tool_layout.get_radial(t._Graph__graph,
                                     _prop("v", g, pos),
                                     _prop("v", g, levels),
+                                    _prop("v", g, rel_order),
+                                    _prop("v", g, node_weight),
                                     int(root), weighted, r)
     return g.own_property(pos)
 
-try:
-    from .cairo_draw import graph_draw, cairo_draw, get_hierarchy_control_points
-except ImportError:
-    pass
-
-try:
-    from .cairo_draw import GraphWidget, GraphWindow, \
-        interactive_window
-    __all__ += ["interactive_window", "GraphWidget", "GraphWindow"]
-except ImportError:
-    pass
-
-try:
-   from .graphviz_draw import graphviz_draw
-except ImportError:
-   pass
-
 def prop_to_size(prop, mi=0, ma=5, log=False, power=0.5):
     r"""Convert property map values to be more useful as a vertex size, or edge
     width. The new values are taken to be
@@ -790,7 +786,7 @@ def prop_to_size(prop, mi=0, ma=5, log=False, power=0.5):
 
         y = mi + (ma - mi) \left(\frac{x_i - min(x)} {max(x) - min(x)}\right)^\text{power}
 
-    If `log=True`, the natural logarithm of the property values are used instead.
+    If `log=True`, the natural logarithm of the property values is used instead.
 
     """
     prop = prop.copy(value_type="double")
@@ -804,3 +800,20 @@ def prop_to_size(prop, mi=0, ma=5, log=False, power=0.5):
         delta = 1
     prop.fa = mi + (ma - mi) * ((vals - vals.min()) / delta) ** power
     return prop
+
+try:
+    from .cairo_draw import graph_draw, cairo_draw, get_hierarchy_control_points, default_cm
+except ImportError:
+    pass
+
+try:
+    from .cairo_draw import GraphWidget, GraphWindow, \
+        interactive_window, draw_hierarchy
+    __all__ += ["interactive_window", "GraphWidget", "GraphWindow", "draw_hierarchy"]
+except ImportError:
+    pass
+
+try:
+   from .graphviz_draw import graphviz_draw
+except ImportError:
+   pass
