@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,10 @@
 from __future__ import division, absolute_import, print_function
 
 import sys
+if sys.version_info < (3,):
+    range = xrange
+else:
+    unicode = str
 import os
 import os.path
 import time
@@ -28,7 +32,8 @@ import warnings
 import ctypes
 import ctypes.util
 import tempfile
-from .. import PropertyMap, group_vector_property, ungroup_vector_property
+from .. import PropertyMap, group_vector_property, ungroup_vector_property, _c_str
+import numpy
 import numpy.random
 import copy
 
@@ -71,6 +76,16 @@ try:
     # create a context to use the whole time (if we keep freeing and recreating
     # it, we will hit a memory leak in graphviz)
     gvc = libgv.gvContext()
+
+    try:
+        gv_new_api = True
+        libgv_directed = libgv.Agdirected
+        libgv_undirected = libgv.Agundirected
+    except AttributeError:
+        gv_new_api = False
+        libgv_directed = 1
+        libgv_undirected = 0
+
 except OSError:
     msg = "Error importing graphviz C library (libgvc)... graphviz_draw() will not work."
     warnings.filterwarnings("always", msg, ImportWarning)
@@ -302,7 +317,13 @@ def graphviz_draw(g, pos=None, size=(15, 15), pin=False, layout=None,
 
     has_layout = False
     try:
-        gvg = libgv.agopen("G".encode("utf8"), 1 if g.is_directed() else 0)
+        if gv_new_api:
+            gvg = libgv.agopen("G".encode("utf8"),
+                               libgv_directed if g.is_directed() else libgv_undirected,
+                               0)
+        else:
+            gvg = libgv.agopen("G".encode("utf8"),
+                               libgv_directed if g.is_directed() else libgv_undirected)
 
         if layout is None:
             if pin == False:
@@ -365,7 +386,7 @@ def graphviz_draw(g, pos=None, size=(15, 15), pin=False, layout=None,
         # normalize color properties
         if (isinstance(vcolor, PropertyMap) and
             vcolor.value_type() != "string"):
-            minmax = [float("inf"), -float("inf")]
+            minmax = [numpy.inf, -numpy.inf]
             for v in g.vertices():
                 c = vcolor[v]
                 minmax[0] = min(c, minmax[0])
@@ -379,7 +400,7 @@ def graphviz_draw(g, pos=None, size=(15, 15), pin=False, layout=None,
 
         if (isinstance(ecolor, PropertyMap) and
             ecolor.value_type() != "string"):
-            minmax = [float("inf"), -float("inf")]
+            minmax = [numpy.inf, -numpy.inf]
             for e in g.edges():
                 c = ecolor[e]
                 minmax[0] = min(c, minmax[0])
@@ -404,7 +425,10 @@ def graphviz_draw(g, pos=None, size=(15, 15), pin=False, layout=None,
         else:
             vertices = g.vertices()
         for v in vertices:
-            n = libgv.agnode(gvg, str(int(v)).encode("utf8"))
+            if gv_new_api:
+                n = libgv.agnode(gvg, str(int(v)).encode("utf8"), 1)
+            else:
+                n = libgv.agnode(gvg, str(int(v)).encode("utf8"))
 
             if type(vsize) == PropertyMap:
                 vw = vh = vsize[v]
@@ -417,12 +441,12 @@ def graphviz_draw(g, pos=None, size=(15, 15), pin=False, layout=None,
             aset(n, "style", "filled")
             aset(n, "color", "#2e3436")
             # apply color
-            if isinstance(vcolor, str):
-                aset(n, "fillcolor", vcolor)
+            if isinstance(vcolor, (str, unicode)):
+                aset(n, "fillcolor", _c_str(vcolor))
             else:
                 color = vcolor[v]
-                if isinstance(color, str):
-                    aset(n, "fillcolor", color)
+                if isinstance(color, (str, unicode)):
+                    aset(n, "fillcolor", _c_str(color))
                 else:
                     color = tuple([int(c * 255.0) for c in vcmap(vnorm(color))])
                     aset(n, "fillcolor", "#%.2x%.2x%.2x%.2x" % color)
@@ -451,20 +475,26 @@ def graphviz_draw(g, pos=None, size=(15, 15), pin=False, layout=None,
         else:
             edges = g.edges()
         for e in edges:
-            ge = libgv.agedge(gvg,
-                              libgv.agnode(gvg, str(int(e.source())).encode("utf8")),
-                              libgv.agnode(gvg, str(int(e.target())).encode("utf8")))
+            if gv_new_api:
+                ge = libgv.agedge(gvg,
+                                  libgv.agnode(gvg, str(int(e.source())).encode("utf8"), False),
+                                  libgv.agnode(gvg, str(int(e.target())).encode("utf8"), False),
+                                  str(g.edge_index[e]).encode("utf8"), 1)
+            else:
+                ge = libgv.agedge(gvg,
+                                  libgv.agnode(gvg, str(int(e.source())).encode("utf8")),
+                                  libgv.agnode(gvg, str(int(e.target())).encode("utf8")))
             aset(ge, "arrowsize", "0.3")
             if g.is_directed():
                 aset(ge, "arrowhead", "vee")
 
             # apply color
-            if isinstance(ecolor, str):
-                aset(ge, "color", ecolor)
+            if isinstance(ecolor, (str, unicode)):
+                aset(ge, "color", _c_str(ecolor))
             else:
                 color = ecolor[e]
-                if isinstance(color, str):
-                    aset(ge, "color", color)
+                if isinstance(color, (str, unicode)):
+                    aset(ge, "color", _c_str(color))
                 else:
                     color = tuple([int(c * 255.0) for c in ecmap(enorm(color))])
                     aset(ge, "color", "#%.2x%.2x%.2x%.2x" % color)
