@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,25 +39,32 @@ Summary
    price_network
    complete_graph
    circular_graph
+   condensation_graph
 
 Contents
 ++++++++
 """
 
 from __future__ import division, absolute_import, print_function
+import sys
+if sys.version_info < (3,):
+    range = xrange
 
 from .. dl_import import dl_import
 dl_import("from . import libgraph_tool_generation")
 
-from .. import Graph, GraphView, _check_prop_scalar, _prop, _limit_args, _gt_type, _get_rng
+from .. import Graph, GraphView, _check_prop_scalar, _prop, _limit_args, \
+    _gt_type, _get_rng, _c_str, libcore
 from .. stats import label_parallel_edges, label_self_loops
 import inspect
 import types
-import sys, numpy, numpy.random
+import numpy
+import numpy.random
 
 __all__ = ["random_graph", "random_rewire", "predecessor_tree", "line_graph",
            "graph_union", "triangulation", "lattice", "geometric_graph",
-           "price_network", "complete_graph", "circular_graph"]
+           "price_network", "complete_graph", "circular_graph",
+           "condensation_graph"]
 
 
 def random_graph(N, deg_sampler, directed=True,
@@ -109,6 +116,10 @@ def random_graph(N, deg_sampler, directed=True,
         types. It must be callable either with no arguments or with a single
         argument which will be the vertex index. In either case it must return
         a type compatible with the ``block_type`` parameter.
+
+        See the documentation for the ``vertex_corr`` parameter of the
+        :func:`~graph_tool.generation.random_rewire` function which specifies
+        the correlation matrix.
     block_type : string (optional, default: ``"int"``)
         Value type of block labels. Valid only if ``block_membership != None``.
     degree_block : bool (optional, default: ``False``)
@@ -161,9 +172,9 @@ def random_graph(N, deg_sampler, directed=True,
     .. testcode::
        :hide:
 
-       from numpy.random import randint, random, seed, poisson
+       import numpy.random
        from pylab import *
-       seed(43)
+       np.random.seed(43)
        gt.seed_rng(42)
 
     This is a degree sampler which uses rejection sampling to sample from the
@@ -172,8 +183,8 @@ def random_graph(N, deg_sampler, directed=True,
     >>> def sample_k(max):
     ...     accept = False
     ...     while not accept:
-    ...         k = randint(1,max+1)
-    ...         accept = random() < 1.0/k
+    ...         k = np.random.randint(1,max+1)
+    ...         accept = np.random.random() < 1.0/k
     ...     return k
     ...
 
@@ -188,8 +199,6 @@ def random_graph(N, deg_sampler, directed=True,
     >>> g = gt.random_graph(1000, lambda: sample_k(40), model="probabilistic",
     ...                     vertex_corr=lambda i, k: 1.0 / (1 + abs(i - k)), directed=False,
     ...                     n_iter=100)
-    >>> gt.scalar_assortativity(g, "out")
-    (0.6285094791115295, 0.010745128857935755)
 
     The following samples an in,out-degree pair from the joint distribution:
 
@@ -202,9 +211,9 @@ def random_graph(N, deg_sampler, directed=True,
 
     >>> def deg_sample():
     ...    if random() > 0.5:
-    ...        return poisson(4), poisson(4)
+    ...        return np.random.poisson(4), np.random.poisson(4)
     ...    else:
-    ...        return poisson(20), poisson(20)
+    ...        return np.random.poisson(20), np.random.poisson(20)
     ...
 
     The following generates a random directed graph with this distribution, and
@@ -214,7 +223,8 @@ def random_graph(N, deg_sampler, directed=True,
     >>>
     >>> hist = gt.combined_corr_hist(g, "in", "out")
     >>>
-    >>> clf()
+    >>> figure()
+    <...>
     >>> imshow(hist[0].T, interpolation="nearest", origin="lower")
     <...>
     >>> colorbar()
@@ -223,6 +233,7 @@ def random_graph(N, deg_sampler, directed=True,
     <...>
     >>> ylabel("out-degree")
     <...>
+    >>> tight_layout()
     >>> savefig("combined-deg-hist.pdf")
 
     .. testcode::
@@ -256,8 +267,7 @@ def random_graph(N, deg_sampler, directed=True,
 
     Lets plot the average degree correlations to check.
 
-    >>> clf()
-    >>> axes([0.1,0.15,0.63,0.8])
+    >>> figure()
     <...>
     >>> corr = gt.avg_neighbour_corr(g, "in", "in")
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-",
@@ -275,12 +285,13 @@ def random_graph(N, deg_sampler, directed=True,
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-",
     ...          label=r"$\left<\text{out}\right>$ vs out")
     <...>
-    >>> legend(bbox_to_anchor=(1.01, 0.5), loc="center left", borderaxespad=0.)
+    >>> legend(loc="lower right", borderaxespad=0., framealpha=0.8)
     <...>
     >>> xlabel("Source degree")
     <...>
     >>> ylabel("Average target degree")
     <...>
+    >>> tight_layout()
     >>> savefig("deg-corr-dir.pdf")
 
     .. testcode::
@@ -410,9 +421,11 @@ def random_graph(N, deg_sampler, directed=True,
         bm = None
 
     if random:
+        g.set_fast_edge_removal(True)
         random_rewire(g, parallel_edges=parallel_edges,
                       self_loops=self_loops, verbose=verbose,
                       block_membership=bm, **kwargs)
+        g.set_fast_edge_removal(False)
 
     if bm is None:
         return g
@@ -426,7 +439,7 @@ def random_graph(N, deg_sampler, directed=True,
 def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
                   parallel_edges=False, self_loops=False, vertex_corr=None,
                   block_membership=None, alias=True, cache_probs=True,
-                  persist=False, ret_fail=False, verbose=False):
+                  persist=False, pin=None, ret_fail=False, verbose=False):
     r"""
 
     Shuffle the graph in-place, following a variety of possible statistical
@@ -449,8 +462,11 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
            graph will remain unmodified.
         ``correlated``
            The edges will be rewired randomly, but both the degree sequence of
-           the graph and the *vertex-vertex degree correlations* will remain
-           unmodified.
+           the graph and the *vertex-vertex (in,out)-degree correlations* will
+           remain exactly preserved. If the ``block_membership`` parameter is
+           passed, the block variables at the endpoints of the edges will be
+           preserved (instead of the degrees), in addition to the degree
+           sequence.
         ``probabilistic``
            This is similar to the ``correlated`` option, but the vertex-vertex
            correlations are not kept unmodified, but instead are sampled from an
@@ -473,25 +489,43 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
         If ``True``, each iteration will perform an entire "sweep" over the
         edges, where each edge is visited once in random order, and a edge swap
         is attempted.
-    parallel : bool (optional, default: ``False``)
+    parallel_edges : bool (optional, default: ``False``)
         If ``True``, parallel edges are allowed.
     self_loops : bool (optional, default: ``False``)
         If ``True``, self-loops are allowed.
-    vertex_corr : function (optional, default: ``None``)
-        A function which gives the vertex-vertex correlation of the graph.
+    vertex_corr : function or sequence of triples (optional, default: ``None``)
 
-        If ``model == probabilistic`` it should be callable with two parameters:
-        the (in, out)-degree pair of the source vertex an edge, and the
-        (in,out)-degree pair of the target of the same edge (for undirected
-        graphs, both parameters are single values). The function should return a
-        number proportional to the probability of such an edge existing in the
-        generated graph.
+        A function which gives the vertex-vertex correlation of the edges in the
+        graph. In general it should have the following signature:
+
+        .. code::
+
+            def vertex_corr(r, s):
+                ...
+                return p
+
+        where the return value should be a scalar.
+
+        Alternatively, this parameter can be a list of triples of the form
+        ``(r, s, p)``, with the same meaning as the ``r``, ``s`` and ``p``
+        values above. If a given ``(r, s)`` combination is not present in this
+        list, the corresponding value of ``p`` is assumed to be zero. If the same
+        ``(r, s)`` combination appears more than once, their ``p`` values will
+        be summed together. This is useful when the correlation matrix is sparse,
+        i.e. most entries are zero.
+
+        If ``model == probabilistic`` the parameters ``r`` and ``s`` correspond
+        respectively to the (in, out)-degree pair of the source vertex an edge,
+        and the (in,out)-degree pair of the target of the same edge (for
+        undirected graphs, both parameters are scalars instead). The value of
+        ``p`` should be a number proportional to the probability of such an
+        edge existing in the generated graph.
 
         If ``model == blockmodel`` or ``model == blockmodel-traditional``, the
-        values passed to the function will be the block value of the respective
-        vertices, as specified via the ``block_membership``. The function should
-        also return a number proportional to the probability of such an edge
-        existing in the generated graph.
+        ``r`` and ``s`` values passed to the function will be the block values
+        of the respective vertices, as specified via the ``block_membership``
+        parameter. The value of  ``p`` should be a number proportional to the
+        probability of such an edge existing in the generated graph.
     block_membership : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         If supplied, the graph will be rewired to conform to a blockmodel
         ensemble. The value must be a vertex property map which defines the
@@ -517,6 +551,10 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
         some probabilistic models, and should be sufficiently fast for sparse
         graphs, but otherwise it may result in many repeated attempts for
         certain corner-cases in which edges are difficult to swap.
+    pin : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
+        Edge property map which, if provided, specifies which edges are allowed
+        to be rewired. Edges for which the property value is ``1`` (or ``True``)
+        will be left unmodified in the graph.
     verbose : bool (optional, default: ``False``)
         If ``True``, verbose information is displayed.
 
@@ -573,7 +611,7 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
        seed(43)
        gt.seed_rng(42)
 
-    >>> g, pos = gt.triangulation(random((1000,2)))
+    >>> g, pos = gt.triangulation(np.random.random((1000,2)))
     >>> pos = gt.arf_layout(g)
     >>> gt.graph_draw(g, pos=pos, output="rewire_orig.pdf", output_size=(300, 300))
     <...>
@@ -583,8 +621,7 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
 
        gt.graph_draw(g, pos=pos, output="rewire_orig.png", output_size=(300, 300))
 
-    >>> gt.random_rewire(g, "correlated")
-    189
+    >>> ret = gt.random_rewire(g, "correlated")
     >>> pos = gt.arf_layout(g)
     >>> gt.graph_draw(g, pos=pos, output="rewire_corr.pdf", output_size=(300, 300))
     <...>
@@ -594,8 +631,7 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
 
        gt.graph_draw(g, pos=pos, output="rewire_corr.png", output_size=(300, 300))
 
-    >>> gt.random_rewire(g)
-    197
+    >>> ret = gt.random_rewire(g)
     >>> pos = gt.arf_layout(g)
     >>> gt.graph_draw(g, pos=pos, output="rewire_uncorr.pdf", output_size=(300, 300))
     <...>
@@ -605,8 +641,7 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
 
        gt.graph_draw(g, pos=pos, output="rewire_uncorr.png", output_size=(300, 300))
 
-    >>> gt.random_rewire(g, "erdos")
-    26
+    >>> ret = gt.random_rewire(g, "erdos")
     >>> pos = gt.arf_layout(g)
     >>> gt.graph_draw(g, pos=pos, output="rewire_erdos.pdf", output_size=(300, 300))
     <...>
@@ -636,18 +671,15 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
     >>> corr = gt.avg_neighbour_corr(g, "out", "out")
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-", label="Original")
     <...>
-    >>> gt.random_rewire(g, "correlated")
-    206
+    >>> ret = gt.random_rewire(g, "correlated")
     >>> corr = gt.avg_neighbour_corr(g, "out", "out")
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="*", label="Correlated")
     <...>
-    >>> gt.random_rewire(g)
-    109
+    >>> ret = gt.random_rewire(g)
     >>> corr = gt.avg_neighbour_corr(g, "out", "out")
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-", label="Uncorrelated")
     <...>
-    >>> gt.random_rewire(g, "erdos")
-    13
+    >>> ret = gt.random_rewire(g, "erdos")
     >>> corr = gt.avg_neighbour_corr(g, "out", "out")
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-", label=r"Erd\H{o}s")
     <...>
@@ -657,6 +689,7 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
     <...>
     >>> legend(loc="best")
     <...>
+    >>> tight_layout()
     >>> savefig("shuffled-stats.pdf")
 
     .. testcode::
@@ -682,8 +715,6 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
     ...                     n_iter=100)
     >>> figure()
     <...>
-    >>> axes([0.1,0.15,0.6,0.8])
-    <...>
     >>> corr = gt.avg_neighbour_corr(g, "in", "out")
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-",
     ...          label=r"$\left<\text{o}\right>$ vs i")
@@ -692,8 +723,7 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-",
     ...          label=r"$\left<\text{i}\right>$ vs o")
     <...>
-    >>> gt.random_rewire(g, "correlated")
-    4323
+    >>> ret = gt.random_rewire(g, "correlated")
     >>> corr = gt.avg_neighbour_corr(g, "in", "out")
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-",
     ...          label=r"$\left<\text{o}\right>$ vs i, corr.")
@@ -702,8 +732,7 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-",
     ...          label=r"$\left<\text{i}\right>$ vs o, corr.")
     <...>
-    >>> gt.random_rewire(g, "uncorrelated")
-    153
+    >>> ret = gt.random_rewire(g, "uncorrelated")
     >>> corr = gt.avg_neighbour_corr(g, "in", "out")
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-",
     ...          label=r"$\left<\text{o}\right>$ vs i, uncorr.")
@@ -712,12 +741,13 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
     >>> errorbar(corr[2][:-1], corr[0], yerr=corr[1], fmt="o-",
     ...          label=r"$\left<\text{i}\right>$ vs o, uncorr.")
     <...>
-    >>> legend(bbox_to_anchor=(1.01, 0.5), loc="center left", borderaxespad=0.)
+    >>> legend(loc="lower right", borderaxespad=0., framealpha=0.8)
     <...>
     >>> xlabel("Source degree")
     <...>
     >>> ylabel("Average target degree")
     <...>
+    >>> tight_layout()
     >>> savefig("shuffled-deg-corr-dir.pdf")
 
     .. testcode::
@@ -749,18 +779,19 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
        no. 1: 016107 (2011) :doi:`10.1103/PhysRevE.83.016107` :arxiv:`1008.3926`
 
     """
-    if not parallel_edges:
-        p = label_parallel_edges(g)
-        if p.a.max() != 0:
-            raise ValueError("Parallel edge detected. Can't rewire " +
-                             "graph without parallel edges if it " +
-                             "already contains parallel edges!")
-    if not self_loops:
-        l = label_self_loops(g)
-        if l.a.max() != 0:
-            raise ValueError("Self-loop detected. Can't rewire graph " +
-                             "without self-loops if it already contains" +
-                             " self-loops!")
+    # if not parallel_edges:
+    #     p = label_parallel_edges(g)
+    #     if p.a.max() != 0:
+    #         raise ValueError("Parallel edge detected. Can't rewire " +
+    #                          "graph without parallel edges if it " +
+    #                          "already contains parallel edges!")
+
+    # if not self_loops:
+    #     l = label_self_loops(g)
+    #     if l.a.max() != 0:
+    #         raise ValueError("Self-loop detected. Can't rewire graph " +
+    #                          "without self-loops if it already contains" +
+    #                          " self-loops!")
 
     if (vertex_corr is not None and not g.is_directed()) and "blockmodel" not in model:
         corr = lambda i, j: vertex_corr(i[1], j[1])
@@ -770,15 +801,29 @@ def random_rewire(g, model="uncorrelated", n_iter=1, edge_sweep=True,
     if model not in ["probabilistic", "blockmodel", "blockmodel-traditional"]:
         g = GraphView(g, reversed=False)
 
+    if model == "blockmodel" and alias and edge_sweep:
+        edge_sweep = False
+        n_iter *= g.num_edges()
+
     traditional = False
     if model == "blockmodel-traditional":
         model = "blockmodel"
         traditional = True
-    pcount = libgraph_tool_generation.random_rewire(g._Graph__graph, model,
+
+    if pin is None:
+        pin = g.new_edge_property("bool")
+
+    if pin.value_type() != "bool":
+        pin = pin.copy(value_type="bool")
+
+    pcount = libgraph_tool_generation.random_rewire(g._Graph__graph,
+                                                    _c_str(model),
                                                     n_iter, not edge_sweep,
                                                     self_loops, parallel_edges,
                                                     alias, traditional, persist,
-                                                    corr, _prop("v", g, block_membership),
+                                                    corr,
+                                                    _prop("e", g, pin),
+                                                    _prop("v", g, block_membership),
                                                     cache_probs,
                                                     _get_rng(), verbose)
     return pcount
@@ -818,30 +863,25 @@ def line_graph(g):
 
     >>> g = gt.collection.data["lesmis"]
     >>> lg, vmap = gt.line_graph(g)
-    >>> gt.graph_draw(g, pos=g.vp["pos"], output="lesmis.pdf")
-    <...>
-    >>> pos = gt.graph_draw(lg, output="lesmis-lg.pdf")
+    >>> pos = gt.graph_draw(lg, output_size=(300, 300), output="lesmis-lg.pdf")
 
     .. testcode::
        :hide:
 
-       gt.graph_draw(g, pos=g.vp["pos"], output="lesmis.png")
-       pos = gt.graph_draw(lg, pos=pos, output="lesmis-lg.png")
+       gt.graph_draw(lg, pos=pos, output_size=(300, 300), output="lesmis-lg.png")
 
 
-    .. figure:: lesmis.png
-       :align: left
-
-       Coappearances of characters in Victor Hugo's novel "Les Miserables".
 
     .. figure:: lesmis-lg.png
-       :align: right
+       :align: center
 
-       Line graph of the coappearance network on the left.
+       Line graph of the coappearance of characters in Victor Hugo's novel "Les
+       Misérables".
 
     References
     ----------
     .. [line-wiki] http://en.wikipedia.org/wiki/Line_graph
+
     """
     lg = Graph(directed=g.is_directed())
 
@@ -853,9 +893,11 @@ def line_graph(g):
     return lg, vertex_map
 
 
-def graph_union(g1, g2, intersection=None, props=None, include=False):
-    """Return the union of graphs g1 and g2, composed of all edges and vertices
-    of g1 and g2, without overlap.
+def graph_union(g1, g2, intersection=None, props=None, include=False,
+                internal_props=False):
+    """Return the union of graphs ``g1`` and ``g2``, composed of all edges and
+    vertices of ``g1`` and ``g2``, without overlap (if ``intersection ==
+    None``).
 
     Parameters
     ----------
@@ -864,18 +906,21 @@ def graph_union(g1, g2, intersection=None, props=None, include=False):
     g2 : :class:`~graph_tool.Graph`
        Second graph in the union.
     intersection : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
-       Vertex property map owned by `g1` which maps each of its vertices
-       to vertex indexes belonging to `g2`. Negative values mean no mapping
+       Vertex property map owned by `g2` which maps each of its vertices
+       to vertex indexes belonging to `g1`. Negative values mean no mapping
        exists, and thus both vertices in `g1` and `g2` will be present in the
        union graph.
-    props : list of tuples of :class:`~graph_tool.PropertyMap` (optional, default: ``[]``)
+    props : list of tuples of :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
        Each element in this list must be a tuple of two PropertyMap objects. The
        first element must be a property of `g1`, and the second of `g2`. If either
        value is ``None``, an empty map is created. The values of the property
        maps are propagated into the union graph, and returned.
     include : bool (optional, default: ``False``)
-       If true, graph `g2` is inserted into `g1` which is modified. If false, a
+       If ``True``, graph `g2` is inserted into `g1` which is modified. If false, a
        new graph is created, and both graphs remain unmodified.
+    internal_props : bool (optional, default: ``False``)
+       If ``True``, all internal property maps are propagated, in addition
+       to ``props``.
 
     Returns
     -------
@@ -932,8 +977,24 @@ def graph_union(g1, g2, intersection=None, props=None, include=False):
     .. image:: graph_union2.*
 
     """
-    if props == None:
+    pnames = None
+    if props is None:
         props = []
+    if internal_props:
+        pnames = []
+        for (k, name), p1 in g1.properties.items():
+            if k == 'g':
+                continue
+            p2 = g2.properties.get((k, name), None)
+            props.append((p1, p2))
+            pnames.append(name)
+        for (k, name), p2 in g2.properties.items():
+            if k == 'g' or (k, name) in g1.properties:
+                continue
+            props.append((None, p2))
+            pnames.append(name)
+        gprops = [[(name, g1.properties[('g', name)]) for name in g1.graph_properties.keys()],
+                  [(name, g2.properties[('g', name)]) for name in g2.graph_properties.keys()]]
     if not include:
         g1 = GraphView(g1, skip_properties=True)
         p1s = []
@@ -960,7 +1021,7 @@ def graph_union(g1, g2, intersection=None, props=None, include=False):
         emask, emask_flip = g1.get_edge_filter()
         emask_flipped = False
         if emask is not None and not emask_flip:
-            emask.a = not emask.a
+            emask.a = numpy.logical_not(emask.a)
             emask_flipped = True
             g1.set_edge_filter(emask, True)
 
@@ -972,12 +1033,9 @@ def graph_union(g1, g2, intersection=None, props=None, include=False):
             vmask_flipped = True
 
     if intersection is None:
-        intersection = g1.new_vertex_property("int32_t")
-        intersection.a = 0
+        intersection = g2.new_vertex_property("int64_t", -1)
     else:
-        intersection = intersection.copy("int32_t")
-        intersection.a[intersection.a >= 0] += 1
-        intersection.a[intersection.a < 0] = 0
+        intersection = intersection.copy("int64_t")
 
     u1 = GraphView(g1, directed=True, skip_properties=True)
     u2 = GraphView(g2, directed=True, skip_properties=True)
@@ -990,12 +1048,12 @@ def graph_union(g1, g2, intersection=None, props=None, include=False):
     if include:
         emask, emask_flip = g1.get_edge_filter()
         if emask is not None and emask_flipped:
-            emask.a = not emask.a
+            emask.a = numpy.logical_not(emask.a)
             g1.set_edge_filter(emask, False)
 
         vmask, vmask_flip = g1.get_vertex_filter()
         if vmask is not None and vmask_flipped:
-            vmask.a = not vmask.a
+            vmask.a = numpy.logical_not(vmask.a)
             g1.set_vertex_filter(vmask, False)
 
     n_props = []
@@ -1022,6 +1080,17 @@ def graph_union(g1, g2, intersection=None, props=None, include=False):
                                       _prop(p2.key_type(), g2, p2))
         n_props.append(p1)
 
+    if pnames is not None:
+        for name, p in zip(pnames, n_props):
+            g1.properties[(p.key_type(), name)] = p
+        if not include:
+            for name, p in gprops[0]:
+                g1.graph_properties[name] = g1.own_property(p.copy())
+        for name, p in gprops[1]:
+            if name not in g1.graph_properties:
+                g1.graph_properties[name] = g1.own_property(p.copy())
+        n_props = []
+
     if len(n_props) > 0:
         return g1, n_props
     else:
@@ -1147,7 +1216,8 @@ def triangulation(points, type="simple", periodic=False):
     g = Graph(directed=False)
     pos = g.new_vertex_property("vector<double>")
     libgraph_tool_generation.triangulation(g._Graph__graph, points,
-                                           _prop("v", g, pos), type, periodic)
+                                           _prop("v", g, pos), _c_str(type),
+                                           periodic)
     return g, pos
 
 
@@ -1476,11 +1546,11 @@ def price_network(N, m=1, c=None, gamma=1, directed=True, seed_graph=None):
     Note that if `seed_graph` is not given, the algorithm will *always* start
     with one node if :math:`c > 0`, or with two nodes with a link between them
     otherwise. If :math:`m > 1`, the degree of the newly added vertices will be
-    vary dynamically as :math:`m'(t) = \min(m, N(t))`, where :math:`N(t)` is the
+    vary dynamically as :math:`m'(t) = \min(m, V(t))`, where :math:`V(t)` is the
     number of vertices added so far. If this behaviour is undesired, a proper
-    seed graph with :math:`N \ge m` vertices must be provided.
+    seed graph with :math:`V \ge m` vertices must be provided.
 
-    This algorithm runs in :math:`O(N\log N)` time.
+    This algorithm runs in :math:`O(V\log V)` time.
 
     See Also
     --------
@@ -1497,12 +1567,12 @@ def price_network(N, m=1, c=None, gamma=1, directed=True, seed_graph=None):
        gt.seed_rng(42)
 
     >>> g = gt.price_network(20000)
-    >>> gt.graph_draw(g, pos=gt.sfdp_layout(g, epsilon=1e-2, cooling_step=0.95),
+    >>> gt.graph_draw(g, pos=gt.sfdp_layout(g, cooling_step=0.99),
     ...               vertex_fill_color=g.vertex_index, vertex_size=2,
     ...               edge_pen_width=1, output="price-network.png")
     <...>
     >>> g = gt.price_network(20000, c=0.1)
-    >>> gt.graph_draw(g, pos=gt.sfdp_layout(g, epsilon=1e-2, cooling_step=0.95),
+    >>> gt.graph_draw(g, pos=gt.sfdp_layout(g, cooling_step=0.99),
     ...               vertex_fill_color=g.vertex_index, vertex_size=2,
     ...               edge_pen_width=1, output="price-network-broader.png")
     <...>
@@ -1554,3 +1624,202 @@ def price_network(N, m=1, c=None, gamma=1, directed=True, seed_graph=None):
         g = seed_graph
     libgraph_tool_generation.price(g._Graph__graph, N, gamma, c, m, _get_rng())
     return g
+
+def condensation_graph(g, prop, vweight=None, eweight=None, avprops=None,
+                       aeprops=None, self_loops=False, parallel_edges=False):
+    r"""
+    Obtain the condensation graph, where each vertex with the same 'prop' value
+    is condensed in one vertex.
+
+    Parameters
+    ----------
+    g : :class:`~graph_tool.Graph`
+        Graph to be modelled.
+    prop : :class:`~graph_tool.PropertyMap`
+        Vertex property map with the community partition.
+    vweight : :class:`~graph_tool.PropertyMap` (optional, default: None)
+        Vertex property map with the optional vertex weights.
+    eweight : :class:`~graph_tool.PropertyMap` (optional, default: None)
+        Edge property map with the optional edge weights.
+    avprops : list of :class:`~graph_tool.PropertyMap` (optional, default: None)
+        If provided, the sum of each property map in this list for
+        each vertex in the condensed graph will be computed and returned.
+    aeprops : list of :class:`~graph_tool.PropertyMap` (optional, default: None)
+        If provided, the sum of each property map in this list for
+        each edge in the condensed graph will be computed and returned.
+    self_loops : ``bool`` (optional, default: ``False``)
+        If ``True``, self-loops due to intra-block edges are also included in
+        the condensation graph.
+    parallel_edges : ``bool`` (optional, default: ``False``)
+        If ``True``, parallel edges will be included in the condensation graph,
+        such that the total number of edges will be the same as in the original
+        graph.
+
+    Returns
+    -------
+    condensation_graph : :class:`~graph_tool.Graph`
+        The community network
+    prop : :class:`~graph_tool.PropertyMap`
+        The community values.
+    vcount : :class:`~graph_tool.PropertyMap`
+        A vertex property map with the vertex count for each community.
+    ecount : :class:`~graph_tool.PropertyMap`
+        An edge property map with the inter-community edge count for each edge.
+    va : list of :class:`~graph_tool.PropertyMap`
+        A list of vertex property maps with summed values of the properties
+        passed via the ``avprops`` parameter.
+    ea : list of :class:`~graph_tool.PropertyMap`
+        A list of edge property maps with summed values of the properties
+        passed via the ``avprops`` parameter.
+
+    Notes
+    -----
+    Each vertex in the condensation graph represents one community in the
+    original graph (vertices with the same 'prop' value), and the edges
+    represent existent edges between vertices of the respective communities in
+    the original graph.
+
+    Examples
+    --------
+
+    .. testsetup:: condensation_graph
+
+       gt.seed_rng(43)
+       np.random.seed(42)
+
+    Let's first obtain the best block partition with ``B=5``.
+
+    .. doctest:: condensation_graph
+
+       >>> g = gt.collection.data["polbooks"]
+       >>> # fit a SBM with 5 groups
+       >>> state = gt.BlockState(g, B=5, deg_corr=True)
+       >>> gt.mcmc_equilibrate(state, wait=1000)
+       (...)
+       >>> b = state.get_blocks()
+       >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=b, vertex_shape=b,
+       ...               output="polbooks_blocks_B5.pdf")
+       <...>
+
+    Now we get the condensation graph:
+
+    .. doctest:: condensation_graph
+
+       >>> bg, bb, vcount, ecount, avp, aep = \
+       ...     gt.condensation_graph(g, b, avprops=[g.vp["pos"]],
+       ...                           self_loops=True)
+       >>> pos = avp[0]
+       >>> for v in bg.vertices():
+       ...     pos[v].a /= vcount[v]
+       >>> gt.graph_draw(bg, pos=avp[0], vertex_fill_color=bb, vertex_shape=bb,
+       ...               vertex_size=gt.prop_to_size(vcount, mi=40, ma=100),
+       ...               edge_pen_width=gt.prop_to_size(ecount, mi=2, ma=10),
+       ...               output="polbooks_blocks_B5_cond.pdf")
+       <...>
+
+    .. testcleanup:: condensation_graph
+
+       gt.graph_draw(g, pos=g.vp["pos"], vertex_fill_color=b, vertex_shape=b,
+                     output="polbooks_blocks_B5.png")
+       gt.graph_draw(bg, pos=avp[0], vertex_fill_color=bb, vertex_shape=bb,
+                     vertex_size=gt.prop_to_size(vcount, mi=40, ma=100),
+                     edge_pen_width=gt.prop_to_size(ecount, mi=2, ma=10),
+                     output="polbooks_blocks_B5_cond.png")
+
+    .. figure:: polbooks_blocks_B5.*
+       :align: center
+
+       Block partition of a political books network with :math:`B=5`.
+
+    .. figure:: polbooks_blocks_B5_cond.*
+       :align: center
+
+       Condensation graph of the obtained block partition.
+
+    """
+    gp = Graph(directed=g.is_directed())
+    if vweight is None:
+        vcount = gp.new_vertex_property("int32_t")
+    else:
+        vcount = gp.new_vertex_property(vweight.value_type())
+    if eweight is None:
+        ecount = gp.new_edge_property("int32_t")
+    else:
+        ecount = gp.new_edge_property(eweight.value_type())
+
+    if prop is g.vertex_index:
+        prop = prop.copy(value_type="int32_t")
+    cprop = gp.new_vertex_property(prop.value_type())
+
+    if avprops is None:
+        avprops = []
+    avp = []
+    r_avp = []
+    for p in avprops:
+        if p is g.vertex_index:
+            p = p.copy(value_type="int")
+        if "string" in p.value_type():
+            raise ValueError("Cannot compute sum of string properties!")
+        temp = g.new_vertex_property(p.value_type())
+        cp = gp.new_vertex_property(p.value_type())
+        avp.append((_prop("v", g, p), _prop("v", g, temp), _prop("v", g, cp)))
+        r_avp.append(cp)
+
+    if aeprops is None:
+        aeprops = []
+    aep = []
+    r_aep = []
+    for p in aeprops:
+        if p is g.edge_index:
+            p = p.copy(value_type="int")
+        if "string" in p.value_type():
+            raise ValueError("Cannot compute sum of string properties!")
+        temp = g.new_edge_property(p.value_type())
+        cp = gp.new_edge_property(p.value_type())
+        aep.append((_prop("e", g, p), _prop("e", g, temp), _prop("e", g, cp)))
+        r_aep.append(cp)
+
+    libgraph_tool_generation.community_network(g._Graph__graph,
+                                               gp._Graph__graph,
+                                               _prop("v", g, prop),
+                                               _prop("v", gp, cprop),
+                                               _prop("v", gp, vcount),
+                                               _prop("e", gp, ecount),
+                                               _prop("v", g, vweight),
+                                               _prop("e", g, eweight),
+                                               self_loops,
+                                               parallel_edges)
+
+    u = GraphView(g, directed=True, reversed=False)
+    libgraph_tool_generation.community_network_vavg(u._Graph__graph,
+                                                    gp._Graph__graph,
+                                                    _prop("v", g, prop),
+                                                    _prop("v", gp, cprop),
+                                                    _prop("v", g, vweight),
+                                                    avp)
+
+    u = GraphView(g, directed=True)
+    libgraph_tool_generation.community_network_eavg(u._Graph__graph,
+                                                    gp._Graph__graph,
+                                                    _prop("v", g, prop),
+                                                    _prop("v", gp, cprop),
+                                                    _prop("e", g, eweight),
+                                                    aep,
+                                                    self_loops)
+    return gp, cprop, vcount, ecount, r_avp, r_aep
+
+class Sampler(libgraph_tool_generation.Sampler):
+    def __init__(self, values, probs):
+        libgraph_tool_generation.Sampler.__init__(self, values, probs)
+
+    def sample(self):
+        return libgraph_tool_generation.Sampler.sample(self, _get_rng())
+
+class DynamicSampler(libgraph_tool_generation.DynamicSampler):
+    def __init__(self, values=None, probs=None):
+        if values == None:
+            values = probs = []
+        libgraph_tool_generation.DynamicSampler.__init__(self, values, probs)
+
+    def sample(self):
+        return libgraph_tool_generation.DynamicSampler.sample(self, _get_rng())
\ No newline at end of file
