@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2013 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -35,10 +35,14 @@ Summary
    :nosignatures:
 
    bfs_search
+   bfs_iterator
    dfs_search
+   dfs_iterator
    dijkstra_search
-   bellman_ford_search
+   dijkstra_iterator
    astar_search
+   astar_iterator
+   bellman_ford_search
    BFSVisitor
    DFSVisitor
    DijkstraVisitor
@@ -52,13 +56,14 @@ Examples
 In this module, most documentation examples will make use of the network
 :download:`search_example.xml <search_example.xml>`, shown below.
 
->>> gt.seed_rng(42)
 >>> g = gt.load_graph("search_example.xml")
->>> name = g.vertex_properties["name"]
->>> weight = g.edge_properties["weight"]
->>> pos = gt.graph_draw(g, vertex_text=name, vertex_font_size=12, vertex_shape="double_circle",
-...                     vertex_fill_color="#729fcf", vertex_pen_width=3,
-...                     edge_pen_width=weight, output="search_example.pdf")
+>>> name = g.vp["name"]
+>>> weight = g.ep["weight"]
+>>> pos = g.vp["pos"]
+>>> gt.graph_draw(g, pos, vertex_text=name, vertex_font_size=12, vertex_shape="double_circle",
+...               vertex_fill_color="#729fcf", vertex_pen_width=3,
+...               edge_pen_width=weight, output="search_example.pdf")
+<...>
 
 .. testcode::
    :hide:
@@ -89,43 +94,14 @@ dl_import("from . import libgraph_tool_search")
 
 from .. import _prop, _python_type
 import weakref
+import numpy
 
-__all__ = ["bfs_search", "BFSVisitor", "dfs_search", "DFSVisitor",
-           "dijkstra_search", "DijkstraVisitor", "bellman_ford_search",
-           "BellmanFordVisitor", "astar_search", "AStarVisitor",
-           "StopSearch"]
+__all__ = ["bfs_search", "bfs_iterator", "BFSVisitor", "dfs_search",
+           "dfs_iterator", "DFSVisitor", "dijkstra_search", "dijkstra_iterator",
+           "DijkstraVisitor", "bellman_ford_search", "BellmanFordVisitor",
+           "astar_search", "astar_iterator", "AStarVisitor", "StopSearch"]
 
 
-class VisitorWrapper(object):
-    def __init__(self, g, visitor, edge_members, vertex_members):
-        self.visitor = visitor
-        self.g = g
-        self.edge_members = set(edge_members)
-        self.vertex_members = set(vertex_members)
-
-    def __getattr__(self, attr):
-        try:
-            orig_attr = self.visitor.__getattribute__(attr)
-        except AttributeError:
-            return object.__getattribute__(self, attr)
-        if callable(orig_attr):
-            def wrapped_visitor_member(*args, **kwargs):
-                old_perms = dict(self.g._Graph__perms)
-                perms ={"del_vertex": False, "del_edge": False, "add_edge": False}
-                if attr in self.edge_members:
-                    perms.update({"del_edge": True, "add_edge": True})
-                elif attr in self.vertex_members:
-                    perms.update({"add_vertex": False})
-                self.g._Graph__perms.update(perms)
-                try:
-                    ret = orig_attr(*args, **kwargs)
-                finally:
-                    self.g._Graph__perms.update(old_perms)
-                return ret
-            return wrapped_visitor_member
-        else:
-            return orig_attr
-
 class BFSVisitor(object):
     r"""A visitor object that is invoked at the event-points inside the
     :func:`~graph_tool.search.bfs_search` algorithm. By default, it performs no
@@ -271,7 +247,7 @@ def bfs_search(g, source, visitor=BFSVisitor()):
     With the above class defined, we can perform the BFS search as follows.
 
     >>> dist = g.new_vertex_property("int")
-    >>> pred = g.new_vertex_property("int")
+    >>> pred = g.new_vertex_property("int64_t")
     >>> gt.bfs_search(g, g.vertex(0), VisitorExample(name, pred, dist))
     --> Bob has been discovered!
     Bob has been examined...
@@ -280,14 +256,14 @@ def bfs_search(g, source, visitor=BFSVisitor()):
     --> Carlos has been discovered!
     --> Isaac has been discovered!
     Eve has been examined...
-    --> Imothep has been discovered!
     --> Carol has been discovered!
+    --> Imothep has been discovered!
     Chuck has been examined...
     Carlos has been examined...
     --> Alice has been discovered!
     Isaac has been examined...
-    Imothep has been examined...
     Carol has been examined...
+    Imothep has been examined...
     Alice has been examined...
     --> Oscar has been discovered!
     --> Dave has been discovered!
@@ -307,17 +283,67 @@ def bfs_search(g, source, visitor=BFSVisitor()):
     .. [bfs-wikipedia] http://en.wikipedia.org/wiki/Breadth-first_search
     """
 
-    visitor = VisitorWrapper(g, visitor,
-                             ["initialize_vertex", "examine_vertex", "finish_vertex"],
-                             ["initialize_vertex"])
-
     try:
         libgraph_tool_search.bfs_search(g._Graph__graph,
-                                        weakref.ref(g),
                                         int(source), visitor)
     except StopSearch:
         pass
 
+def bfs_iterator(g, source):
+    r"""Return an iterator of the edges corresponding to a breath-first traversal of
+    the graph.
+
+    Parameters
+    ----------
+    g : :class:`~graph_tool.Graph`
+        Graph to be used.
+    source : :class:`~graph_tool.Vertex`
+        Source vertex.
+
+    Returns
+    -------
+    bfs_iterator : An iterator over the edges in breath-first order.
+
+    See Also
+    --------
+    dfs_iterator: Depth-first search
+    dijkstra_iterator: Dijkstra's search algorithm
+    astar_iterator: :math:`A^*` heuristic search algorithm
+
+    Notes
+    -----
+
+    See :func:`~graph_tool.search.bfs_search` for an explanation of the
+    algorithm.
+
+    The time complexity is :math:`O(1)` to create the generator and
+    :math:`O(V + E)` to traverse it completely.
+
+    Examples
+    --------
+
+    >>> for e in gt.bfs_iterator(g, g.vertex(0)):
+    ...    print(name[e.source()], "->", name[e.target()])
+    Bob -> Eve
+    Bob -> Chuck
+    Bob -> Carlos
+    Bob -> Isaac
+    Eve -> Carol
+    Eve -> Imothep
+    Carlos -> Alice
+    Alice -> Oscar
+    Alice -> Dave
+
+    References
+    ----------
+    .. [bfs] Edward Moore, "The shortest path through a maze", International
+             Symposium on the Theory of Switching, 1959
+    .. [bfs-bgl] http://www.boost.org/doc/libs/release/libs/graph/doc/breadth_first_search.html
+    .. [bfs-wikipedia] http://en.wikipedia.org/wiki/Breadth-first_search
+    """
+
+    return libgraph_tool_search.bfs_search_generator(g._Graph__graph, int(source))
+
 
 class DFSVisitor(object):
     r"""
@@ -482,28 +508,20 @@ def dfs_search(g, source, visitor=DFSVisitor()):
     With the above class defined, we can perform the DFS search as follows.
 
     >>> time = g.new_vertex_property("int")
-    >>> pred = g.new_vertex_property("int")
+    >>> pred = g.new_vertex_property("int64_t")
     >>> gt.dfs_search(g, g.vertex(0), VisitorExample(name, pred, time))
     --> Bob has been discovered!
     edge (Bob, Eve) has been examined...
     --> Eve has been discovered!
-    edge (Eve, Isaac) has been examined...
-    --> Isaac has been discovered!
-    edge (Isaac, Bob) has been examined...
-    edge (Isaac, Chuck) has been examined...
-    --> Chuck has been discovered!
-    edge (Chuck, Eve) has been examined...
-    edge (Chuck, Isaac) has been examined...
-    edge (Chuck, Imothep) has been examined...
-    --> Imothep has been discovered!
-    edge (Imothep, Carol) has been examined...
+    edge (Eve, Bob) has been examined...
+    edge (Eve, Carol) has been examined...
     --> Carol has been discovered!
     edge (Carol, Eve) has been examined...
     edge (Carol, Imothep) has been examined...
+    --> Imothep has been discovered!
+    edge (Imothep, Carol) has been examined...
     edge (Imothep, Carlos) has been examined...
     --> Carlos has been discovered!
-    edge (Carlos, Eve) has been examined...
-    edge (Carlos, Imothep) has been examined...
     edge (Carlos, Bob) has been examined...
     edge (Carlos, Alice) has been examined...
     --> Alice has been discovered!
@@ -512,28 +530,34 @@ def dfs_search(g, source, visitor=DFSVisitor()):
     edge (Oscar, Alice) has been examined...
     edge (Oscar, Dave) has been examined...
     --> Dave has been discovered!
-    edge (Dave, Oscar) has been examined...
     edge (Dave, Alice) has been examined...
+    edge (Dave, Oscar) has been examined...
     edge (Alice, Dave) has been examined...
     edge (Alice, Carlos) has been examined...
+    edge (Carlos, Eve) has been examined...
+    edge (Carlos, Imothep) has been examined...
     edge (Imothep, Chuck) has been examined...
-    edge (Imothep, Eve) has been examined...
+    --> Chuck has been discovered!
     edge (Chuck, Bob) has been examined...
+    edge (Chuck, Eve) has been examined...
+    edge (Chuck, Isaac) has been examined...
+    --> Isaac has been discovered!
+    edge (Isaac, Bob) has been examined...
+    edge (Isaac, Chuck) has been examined...
     edge (Isaac, Eve) has been examined...
-    edge (Eve, Imothep) has been examined...
-    edge (Eve, Bob) has been examined...
-    edge (Eve, Carol) has been examined...
+    edge (Chuck, Imothep) has been examined...
+    edge (Imothep, Eve) has been examined...
     edge (Eve, Carlos) has been examined...
     edge (Eve, Chuck) has been examined...
+    edge (Eve, Isaac) has been examined...
+    edge (Eve, Imothep) has been examined...
     edge (Bob, Chuck) has been examined...
     edge (Bob, Carlos) has been examined...
     edge (Bob, Isaac) has been examined...
     >>> print(time.a)
-    [0 7 5 6 3 9 1 2 8 4]
+    [0 5 2 4 8 7 1 9 6 3]
     >>> print(pred.a)
-    [0 3 9 9 7 8 0 6 1 4]
-
-
+    [0 3 6 9 9 8 0 4 1 2]
 
     References
     ----------
@@ -541,17 +565,64 @@ def dfs_search(g, source, visitor=DFSVisitor()):
     .. [dfs-wikipedia] http://en.wikipedia.org/wiki/Depth-first_search
     """
 
-    visitor = VisitorWrapper(g, visitor,
-                             ["initialize_vertex", "discover_vertex", "finish_vertex",
-                              "start_vertex"], ["initialize_vertex"])
-
     try:
         libgraph_tool_search.dfs_search(g._Graph__graph,
-                                        weakref.ref(g),
                                         int(source), visitor)
     except StopSearch:
         pass
 
+def dfs_iterator(g, source):
+    r"""Return an iterator of the edges corresponding to a depth-first traversal of
+    the graph.
+
+    Parameters
+    ----------
+    g : :class:`~graph_tool.Graph`
+        Graph to be used.
+    source : :class:`~graph_tool.Vertex`
+        Source vertex.
+
+    Returns
+    -------
+    dfs_iterator : An iterator over the edges in detpth-first order.
+
+    See Also
+    --------
+    bfs_iterator: Breadth-first search
+    dijkstra_iterator: Dijkstra's search algorithm
+    astar_iterator: :math:`A^*` heuristic search algorithm
+
+    Notes
+    -----
+
+    See :func:`~graph_tool.search.dfs_search` for an explanation of the
+    algorithm.
+
+    The time complexity is :math:`O(1)` to create the generator and
+    :math:`O(V + E)` to traverse it completely.
+
+    Examples
+    --------
+
+    >>> for e in gt.dfs_iterator(g, g.vertex(0)):
+    ...    print(name[e.source()], "->", name[e.target()])
+    Bob -> Eve
+    Eve -> Carol
+    Carol -> Imothep
+    Imothep -> Carlos
+    Carlos -> Alice
+    Alice -> Oscar
+    Oscar -> Dave
+    Imothep -> Chuck
+    Chuck -> Isaac
+
+    References
+    ----------
+    .. [dfs-bgl] http://www.boost.org/doc/libs/release/libs/graph/doc/depth_first_search.html
+    .. [dfs-wikipedia] http://en.wikipedia.org/wiki/Depth-first_search
+    """
+
+    return libgraph_tool_search.dfs_search_generator(g._Graph__graph, int(source))
 
 class DijkstraVisitor(object):
     r"""A visitor object that is invoked at the event-points inside the
@@ -617,7 +688,7 @@ class DijkstraVisitor(object):
 
 def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
                     pred_map=None, combine=lambda a, b: a + b,
-                    compare=lambda a, b: a < b, zero=0, infinity=float('inf')):
+                    compare=lambda a, b: a < b, zero=0, infinity=numpy.inf):
     r"""Dijsktra traversal of a directed or undirected graph, with non-negative weights.
 
     Parameters
@@ -637,7 +708,7 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
         stored.
     pred_map : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         A vertex property map where the predecessor map will be
-        stored (must have value type "int").
+        stored (must have value type "int64_t").
     combine : binary function (optional, default: ``lambda a, b: a + b``)
         This function is used to combine distances to compute the distance of a
         path.
@@ -647,7 +718,7 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
     zero : int or float (optional, default: ``0``)
          Value assumed to correspond to a distance of zero by the combine and
          compare functions.
-    infinity : int or float (optional, default: ``float('inf')``)
+    infinity : int or float (optional, default: ``numpy.inf``)
          Value assumed to correspond to a distance of infinity by the combine and
          compare functions.
 
@@ -680,7 +751,7 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
     top of the priority queue. The algorithm finishes when the priority queue is
     empty.
 
-    The time complexity is :math:`O(V \log V)`.
+    The time complexity is :math:`O(E + V \log V)`.
 
     The pseudo-code for Dijkstra's algorithm is listed below, with the annotated
     event points, for which the given visitor object will be called with the
@@ -758,29 +829,29 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
     edge (Bob, Isaac) has been examined...
     edge (Bob, Isaac) has been relaxed...
     --> Isaac has been discovered!
-    edge (Eve, Isaac) has been examined...
-    edge (Eve, Imothep) has been examined...
-    edge (Eve, Imothep) has been relaxed...
-    --> Imothep has been discovered!
     edge (Eve, Bob) has been examined...
     edge (Eve, Carol) has been examined...
     edge (Eve, Carol) has been relaxed...
     --> Carol has been discovered!
     edge (Eve, Carlos) has been examined...
     edge (Eve, Chuck) has been examined...
+    edge (Eve, Isaac) has been examined...
+    edge (Eve, Imothep) has been examined...
+    edge (Eve, Imothep) has been relaxed...
+    --> Imothep has been discovered!
     edge (Isaac, Bob) has been examined...
     edge (Isaac, Chuck) has been examined...
     edge (Isaac, Eve) has been examined...
+    edge (Chuck, Bob) has been examined...
     edge (Chuck, Eve) has been examined...
     edge (Chuck, Isaac) has been examined...
     edge (Chuck, Imothep) has been examined...
-    edge (Chuck, Bob) has been examined...
-    edge (Carlos, Eve) has been examined...
-    edge (Carlos, Imothep) has been examined...
     edge (Carlos, Bob) has been examined...
     edge (Carlos, Alice) has been examined...
     edge (Carlos, Alice) has been relaxed...
     --> Alice has been discovered!
+    edge (Carlos, Eve) has been examined...
+    edge (Carlos, Imothep) has been examined...
     edge (Imothep, Carol) has been examined...
     edge (Imothep, Carlos) has been examined...
     edge (Imothep, Chuck) has been examined...
@@ -796,10 +867,10 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
     edge (Carol, Imothep) has been examined...
     edge (Oscar, Alice) has been examined...
     edge (Oscar, Dave) has been examined...
-    edge (Dave, Oscar) has been examined...
     edge (Dave, Alice) has been examined...
+    edge (Dave, Oscar) has been examined...
     >>> print(time.a)
-    [0 7 6 3 2 9 1 4 8 5]
+    [0 7 5 3 2 9 1 4 8 6]
     >>> print(pred.a)
     [0 3 6 0 0 1 0 0 1 6]
     >>> print(dist.a)
@@ -814,18 +885,14 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
     .. [dijkstra-wikipedia] http://en.wikipedia.org/wiki/Dijkstra's_algorithm
     """
 
-    visitor = VisitorWrapper(g, visitor,
-                             ["initialize_vertex", "examine_vertex", "finish_vertex"],
-                             ["initialize_vertex"])
-
     if visitor is None:
         visitor = DijkstraVisitor()
     if dist_map is None:
         dist_map = g.new_vertex_property(weight.value_type())
     if pred_map is None:
-        pred_map = g.new_vertex_property("int")
-    if pred_map.value_type() != "int32_t":
-        raise ValueError("pred_map must be of value type 'int32_t', not '%s'." % \
+        pred_map = g.new_vertex_property("int64_t")
+    if pred_map.value_type() != "int64_t":
+        raise ValueError("pred_map must be of value type 'int64_t', not '%s'." % \
                              pred_map.value_type())
 
     try:
@@ -844,7 +911,6 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
 
     try:
         libgraph_tool_search.dijkstra_search(g._Graph__graph,
-                                             weakref.ref(g),
                                              int(source),
                                              _prop("v", g, dist_map),
                                              _prop("v", g, pred_map),
@@ -855,6 +921,113 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
 
     return dist_map, pred_map
 
+def dijkstra_iterator(g, source, weight, dist_map=None, combine=None,
+                      compare=None, zero=0, infinity=numpy.inf):
+    r"""Return an iterator of the edges corresponding to a Dijkstra traversal of
+    the graph.
+
+    Parameters
+    ----------
+    g : :class:`~graph_tool.Graph`
+        Graph to be used.
+    source : :class:`~graph_tool.Vertex`
+        Source vertex.
+    weight : :class:`~graph_tool.PropertyMap`
+        Edge property map with weight values.
+    dist_map : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
+        A vertex property map where the distances from the source will be
+        stored.
+    combine : binary function (optional, default: ``lambda a, b: a + b``)
+        This function is used to combine distances to compute the distance of a
+        path.
+    compare : binary function (optional, default: ``lambda a, b: a < b``)
+        This function is use to compare distances to determine which vertex is
+        closer to the source vertex.
+    zero : int or float (optional, default: ``0``)
+         Value assumed to correspond to a distance of zero by the combine and
+         compare functions.
+    infinity : int or float (optional, default: ``numpy.inf``)
+         Value assumed to correspond to a distance of infinity by the combine and
+         compare functions.
+
+    Returns
+    -------
+    djk_iterator : An iterator over the edges in Dijkstra order.
+
+    See Also
+    --------
+    bfs_iterator: Breadth-first search
+    dfs_iterator: Depth-first search
+    astar_iterator: :math:`A^*` heuristic search algorithm
+
+    Notes
+    -----
+
+    See :func:`~graph_tool.search.dijkstra_search` for an explanation of the
+    algorithm.
+
+    The time complexity is :math:`O(1)` to create the generator and
+    :math:`O(E + V\log V)` to traverse it completely.
+
+    Examples
+    --------
+
+    >>> for e in gt.dijkstra_iterator(g, g.vertex(0), weight):
+    ...    print(name[e.source()], "->", name[e.target()])
+    Bob -> Eve
+    Bob -> Chuck
+    Bob -> Carlos
+    Bob -> Isaac
+    Eve -> Carol
+    Eve -> Imothep
+    Carlos -> Alice
+    Alice -> Oscar
+    Alice -> Dave
+
+
+    References
+    ----------
+    .. [dijkstra] E. Dijkstra, "A note on two problems in connexion with
+        graphs", Numerische Mathematik, 1:269-271, 1959.
+    .. [dijkstra-bgl] http://www.boost.org/doc/libs/release/libs/graph/doc/dijkstra_shortest_paths_no_color_map.html
+    .. [dijkstra-wikipedia] http://en.wikipedia.org/wiki/Dijkstra's_algorithm
+    """
+
+    if dist_map is None:
+        dist_map = g.new_vertex_property(weight.value_type())
+
+    try:
+        if dist_map.value_type() != "python::object":
+            zero = _python_type(dist_map.value_type())(zero)
+    except OverflowError:
+        zero = (weight.a.max() + 1) * g.num_vertices()
+        zero = _python_type(dist_map.value_type())(zero)
+
+    try:
+        if dist_map.value_type() != "python::object":
+            infinity = _python_type(dist_map.value_type())(infinity)
+    except OverflowError:
+        infinity = (weight.a.max() + 1) * g.num_vertices()
+        infinity = _python_type(dist_map.value_type())(infinity)
+
+    if compare is None and combine is None:
+        return libgraph_tool_search.dijkstra_generator_fast(g._Graph__graph,
+                                                            int(source),
+                                                            _prop("v", g, dist_map),
+                                                            _prop("e", g, weight),
+                                                            zero, infinity)
+    else:
+        if compare is None:
+            compare = lambda a, b: a < b
+        if combine is None:
+            combine = lambda a, b: a + b
+        return libgraph_tool_search.dijkstra_generator(g._Graph__graph,
+                                                       int(source),
+                                                       _prop("v", g, dist_map),
+                                                       _prop("e", g, weight),
+                                                       compare, combine,
+                                                       zero, infinity)
+
 
 class BellmanFordVisitor(object):
     r"""A visitor object that is invoked at the event-points inside the
@@ -926,7 +1099,7 @@ def bellman_ford_search(g, source, weight, visitor=BellmanFordVisitor(),
         stored.
     pred_map : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         A vertex property map where the predecessor map will be
-        stored (must have value type "int").
+        stored (must have value type "int64_t").
     combine : binary function (optional, default: ``lambda a, b: a + b``)
         This function is used to combine distances to compute the distance of a
         path.
@@ -1082,14 +1255,12 @@ def bellman_ford_search(g, source, weight, visitor=BellmanFordVisitor(),
     .. [bellman-ford-wikipedia] http://en.wikipedia.org/wiki/Bellman-Ford_algorithm
     """
 
-    visitor = VisitorWrapper(g, visitor, [], [])
-
     if dist_map is None:
         dist_map = g.new_vertex_property(weight.value_type())
     if pred_map is None:
-        pred_map = g.new_vertex_property("int")
-    if pred_map.value_type() != "int32_t":
-        raise ValueError("pred_map must be of value type 'int32_t', not '%s'." % \
+        pred_map = g.new_vertex_property("int64_t")
+    if pred_map.value_type() != "int64_t":
+        raise ValueError("pred_map must be of value type 'int64_t', not '%s'." % \
                              pred_map.value_type())
 
     try:
@@ -1110,7 +1281,6 @@ def bellman_ford_search(g, source, weight, visitor=BellmanFordVisitor(),
     try:
         minimized = \
             libgraph_tool_search.bellman_ford_search(g._Graph__graph,
-                                                     weakref.ref(g),
                                                      int(source),
                                                      _prop("v", g, dist_map),
                                                      _prop("v", g, pred_map),
@@ -1216,13 +1386,13 @@ def astar_search(g, source, weight, visitor=AStarVisitor(),
     heuristic : unary function (optional, default: ``lambda v: 1``)
         The heuristic function that guides the search. It should take a single
         argument which is a :class:`~graph_tool.Vertex`, and output an estimated
-        distance from the source vertex.
+        distance from the supplied vertex to the target vertex.
     dist_map : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         A vertex property map where the distances from the source will be
         stored.
     pred_map : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         A vertex property map where the predecessor map will be
-        stored (must have value type "int").
+        stored (must have value type "int64_t").
     cost_map : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         A vertex property map where the vertex costs will be stored. It must
         have the same value type as ``dist_map``. This parameter is only used if
@@ -1248,6 +1418,8 @@ def astar_search(g, source, weight, visitor=AStarVisitor(),
     -------
     dist_map : :class:`~graph_tool.PropertyMap`
         A vertex property map with the computed distances from the source.
+    pred_map : :class:`~graph_tool.PropertyMap`
+        A vertex property map with the predecessor tree.
 
     See Also
     --------
@@ -1271,7 +1443,7 @@ def astar_search(g, source, weight, visitor=AStarVisitor(),
     efficiency of :math:`A^*` is highly dependent on the heuristic function with
     which it is used.
 
-    The time complexity is :math:`O((E + V) \log V)`.
+    The time complexity is :math:`O((E + V)\log V)`.
 
     The pseudo-code for the :math:`A^*` algorithm is listed below, with the
     annotated event points, for which the given visitor object will be called
@@ -1382,7 +1554,7 @@ def astar_search(g, source, weight, visitor=AStarVisitor(),
     >>> ewidth = g.new_edge_property("double")
     >>> ewidth.a = 1
     >>> for e in g.edges():
-    ...    ecolor[e] = "blue" if touch_e[e] else "black"
+    ...    ecolor[e] = "#3465a4" if touch_e[e] else "#d3d7cf"
     >>> v = target
     >>> while v != g.vertex(0):
     ...     p = g.vertex(pred[v])
@@ -1531,7 +1703,6 @@ def astar_search(g, source, weight, visitor=AStarVisitor(),
 
     References
     ----------
-
     .. [astar] Hart, P. E.; Nilsson, N. J.; Raphael, B. "A Formal Basis for the
        Heuristic Determination of Minimum Cost Paths". IEEE Transactions on
        Systems Science and Cybernetics SSC4 4 (2): 100-107, 1968.
@@ -1540,16 +1711,12 @@ def astar_search(g, source, weight, visitor=AStarVisitor(),
     .. [astar-wikipedia] http://en.wikipedia.org/wiki/A*_search_algorithm
     """
 
-    visitor = VisitorWrapper(g, visitor,
-                             ["initialize_vertex", "examine_vertex", "finish_vertex"],
-                             ["initialize_vertex"])
-
     if dist_map is None:
         dist_map = g.new_vertex_property(weight.value_type())
     if pred_map is None:
-        pred_map = g.new_vertex_property("int")
-    if pred_map.value_type() != "int32_t":
-        raise ValueError("pred_map must be of value type 'int32_t', not '%s'." % \
+        pred_map = g.new_vertex_property("int64_t")
+    if pred_map.value_type() != "int64_t":
+        raise ValueError("pred_map must be of value type 'int64_t', not '%s'." % \
                              pred_map.value_type())
 
     dist_type = dist_map.python_value_type()
@@ -1575,11 +1742,7 @@ def astar_search(g, source, weight, visitor=AStarVisitor(),
 
     try:
         if not implicit:
-            g._Graph__perms.update({"del_vertex": False, "del_edge": False,
-                                    "add_edge": False})
-
             libgraph_tool_search.astar_search(g._Graph__graph,
-                                              weakref.ref(g),
                                               int(source), _prop("v", g, dist_map),
                                               _prop("v", g, pred_map),
                                               _prop("e", g, weight), visitor,
@@ -1591,22 +1754,133 @@ def astar_search(g, source, weight, visitor=AStarVisitor(),
             elif cost_map.value_type() != dist_map.value_type():
                 raise ValueError("The cost_map value type must be the same as" +
                                  " dist_map.")
-            g._Graph__perms.update({"del_vertex": False})
             libgraph_tool_search.astar_search_implicit\
-                (g._Graph__graph, weakref.ref(g), int(source),
+                (g._Graph__graph, int(source),
                  _prop("v", g, dist_map), _prop("v", g, pred_map),
                  _prop("v", g, cost_map), _prop("e", g, weight), visitor,
                  compare, combine, zero, infinity, h)
     except StopSearch:
-        g._Graph__perms.update({"del_vertex": True, "del_edge": True,
-                                "add_edge": True})
-    finally:
-        g._Graph__perms.update({"del_vertex": True, "del_edge": True,
-                                "add_edge": True})
-
+        pass
     return dist_map, pred_map
 
 
+def astar_iterator(g, source, weight, heuristic=lambda v: 1, dist_map=None,
+                   combine=None, compare=None, zero=0, infinity=numpy.inf):
+    r"""Return an iterator of the edges corresponding to an :math:`A^*` traversal of
+    the graph.
+
+    Parameters
+    ----------
+    g : :class:`~graph_tool.Graph`
+        Graph to be used.
+    source : :class:`~graph_tool.Vertex`
+        Source vertex.
+    weight : :class:`~graph_tool.PropertyMap`
+        Edge property map with weight values.
+    heuristic : unary function (optional, default: ``lambda v: 1``)
+        The heuristic function that guides the search. It should take a single
+        argument which is a :class:`~graph_tool.Vertex`, and output an estimated
+        distance from the supplied vertex to the target vertex.
+    dist_map : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
+        A vertex property map where the distances from the source will be
+        stored.
+    combine : binary function (optional, default: ``lambda a, b: a + b``)
+        This function is used to combine distances to compute the distance of a
+        path.
+    compare : binary function (optional, default: ``lambda a, b: a < b``)
+        This function is use to compare distances to determine which vertex is
+        closer to the source vertex.
+    zero : int or float (optional, default: ``0``)
+         Value assumed to correspond to a distance of zero by the combine and
+         compare functions.
+    infinity : int or float (optional, default: ``numpy.inf``)
+         Value assumed to correspond to a distance of infinity by the combine and
+         compare functions.
+
+    Returns
+    -------
+    astar_iterator : An iterator over the edges in :math:`A^*` order.
+
+    See Also
+    --------
+    bfs_iterator: Breadth-first search
+    dfs_iterator: Depth-first search
+    dijkstra_iterator: Dijkstra search algorithm
+
+    Notes
+    -----
+
+    See :func:`~graph_tool.search.astar_search` for an explanation of the
+    algorithm.
+
+    The time complexity is :math:`O(1)` to create the generator and
+    :math:`O((E + V)\log V)` to traverse it completely.
+
+    Examples
+    --------
+
+    >>> g = gt.load_graph("search_example.xml")
+    >>> name = g.vp["name"]
+    >>> weight = g.ep["weight"]
+    >>> for e in gt.astar_iterator(g, g.vertex(0), weight):
+    ...    print(name[e.source()], "->", name[e.target()])
+    Bob -> Eve
+    Bob -> Chuck
+    Bob -> Carlos
+    Bob -> Isaac
+    Eve -> Carol
+    Eve -> Imothep
+    Carlos -> Alice
+    Alice -> Oscar
+    Alice -> Dave
+
+    References
+    ----------
+    .. [astar] Hart, P. E.; Nilsson, N. J.; Raphael, B. "A Formal Basis for the
+       Heuristic Determination of Minimum Cost Paths". IEEE Transactions on
+       Systems Science and Cybernetics SSC4 4 (2): 100-107, 1968.
+       :doi:`10.1109/TSSC.1968.300136`
+    .. [astar-bgl] http://www.boost.org/doc/libs/release/libs/graph/doc/astar_search.html
+    .. [astar-wikipedia] http://en.wikipedia.org/wiki/A*_search_algorithm
+    """
+
+    if dist_map is None:
+        dist_map = g.new_vertex_property(weight.value_type())
+
+    try:
+        if dist_map.value_type() != "python::object":
+            zero = _python_type(dist_map.value_type())(zero)
+    except OverflowError:
+        zero = (weight.a.max() + 1) * g.num_vertices()
+        zero = _python_type(dist_map.value_type())(zero)
+
+    try:
+        if dist_map.value_type() != "python::object":
+            infinity = _python_type(dist_map.value_type())(infinity)
+    except OverflowError:
+        infinity = (weight.a.max() + 1) * g.num_vertices()
+        infinity = _python_type(dist_map.value_type())(infinity)
+
+    if compare is None and combine is None:
+        return libgraph_tool_search.astar_generator_fast(g._Graph__graph,
+                                                         int(source),
+                                                         _prop("v", g, dist_map),
+                                                         _prop("e", g, weight),
+                                                         zero, infinity, heuristic)
+    else:
+        if compare is None:
+            compare = lambda a, b: a < b
+        if combine is None:
+            combine = lambda a, b: a + b
+        return libgraph_tool_search.astar_generator(g._Graph__graph,
+                                                    int(source),
+                                                    _prop("v", g, dist_map),
+                                                    _prop("e", g, weight),
+                                                    compare, combine,
+                                                    zero, infinity, heuristic)
+
+
+
 class StopSearch(Exception):
     """If this exception is raised from inside any search visitor object, the search is aborted."""
     pass
